@@ -6,7 +6,7 @@
     A small library wrapping various audio sample rate conversion
     implementations in C++.
 
-    Copyright 2007-2021 Particular Programs Ltd.
+    Copyright 2007-2023 Particular Programs Ltd.
 
     Permission is hereby granted, free of charge, to any person
     obtaining a copy of this software and associated documentation
@@ -33,7 +33,8 @@
     Software without prior written authorization.
 */
 
-#include "bqresample/Resampler.h"
+#include "../bqresample/Resampler.h"
+#include "common.h"
 
 #include <cstdlib>
 #include <cmath>
@@ -61,23 +62,29 @@
 #include <samplerate.h>
 #endif
 
-#ifdef HAVE_LIBRESAMPLE
-#include <libresample.h>
+#ifdef USE_SPEEX
+#include "../ext/speex/speex_resampler.h"
+#else
+#ifdef HAVE_LIBSPEEXDSP
+#include <speex/speex_resampler.h>
+#endif
 #endif
 
-#ifdef USE_SPEEX
-#include "../speex/speex_resampler.h"
+#ifdef USE_BQRESAMPLER
+#include "BQResampler.h"
 #endif
 
 #ifndef HAVE_IPP
 #ifndef HAVE_LIBSAMPLERATE
-#ifndef HAVE_LIBRESAMPLE
+#ifndef HAVE_LIBSPEEXDSP
 #ifndef USE_SPEEX
+#ifndef USE_BQRESAMPLER
 #error No resampler implementation selected!
 #endif
 #endif
 #endif
 #endif
+#endif
 
 using namespace std;
 
@@ -103,6 +110,7 @@ public:
                                     bool final) = 0;
 
     virtual int getChannelCount() const = 0;
+    virtual double getEffectiveRatio(double ratio) const = 0;
 
     virtual void reset() = 0;
 };
@@ -114,7 +122,8 @@ namespace Resamplers {
 class D_IPP : public Resampler::Impl
 {
 public:
-    D_IPP(Resampler::Quality quality, int channels, double initialSampleRate,
+    D_IPP(Resampler::Quality quality, Resampler::RatioChange,
+          int channels, double initialSampleRate,
           int maxBufferSize, int debugLevel);
     ~D_IPP();
 
@@ -133,6 +142,7 @@ public:
                             bool final = false);
 
     int getChannelCount() const { return m_channels; }
+    double getEffectiveRatio(double ratio) const { return ratio; }
 
     void reset();
 
@@ -159,6 +169,7 @@ protected:
 };
 
 D_IPP::D_IPP(Resampler::Quality /* quality */,
+             Resampler::RatioChange /* ratioChange */,
              int channels, double initialSampleRate,
              int maxBufferSize, int debugLevel) :
     m_state(0),
@@ -167,7 +178,7 @@ D_IPP::D_IPP(Resampler::Quality /* quality */,
     m_debugLevel(debugLevel)
 {
     if (m_debugLevel > 0) {
-        cerr << "Resampler::Resampler: using IPP implementation" << endl;
+        cerr << "Resampler::Resampler: using implementation: IPP" << endl;
     }
 
     m_window = 32;
@@ -543,7 +554,8 @@ D_IPP::reset()
 class D_SRC : public Resampler::Impl
 {
 public:
-    D_SRC(Resampler::Quality quality, int channels, double initialSampleRate,
+    D_SRC(Resampler::Quality quality, Resampler::RatioChange ratioChange,
+          int channels, double initialSampleRate,
           int maxBufferSize, int m_debugLevel);
     ~D_SRC();
 
@@ -562,6 +574,7 @@ public:
                             bool final = false);
 
     int getChannelCount() const { return m_channels; }
+    double getEffectiveRatio(double ratio) const { return ratio; }
 
     void reset();
 
@@ -574,11 +587,12 @@ protected:
     int m_ioutsize;
     double m_prevRatio;
     bool m_ratioUnset;
+    bool m_smoothRatios;
     int m_debugLevel;
 };
 
-D_SRC::D_SRC(Resampler::Quality quality, int channels, double,
-             int maxBufferSize, int debugLevel) :
+D_SRC::D_SRC(Resampler::Quality quality, Resampler::RatioChange ratioChange,
+             int channels, double, int maxBufferSize, int debugLevel) :
     m_src(0),
     m_iin(0),
     m_iout(0),
@@ -587,10 +601,11 @@ D_SRC::D_SRC(Resampler::Quality quality, int channels, double,
     m_ioutsize(0),
     m_prevRatio(1.0),
     m_ratioUnset(true),
+    m_smoothRatios(ratioChange == Resampler::SmoothRatioChange),
     m_debugLevel(debugLevel)
 {
     if (m_debugLevel > 0) {
-        cerr << "Resampler::Resampler: using libsamplerate implementation"
+        cerr << "Resampler::Resampler: using implementation: libsamplerate"
              << endl;
     }
 
@@ -692,7 +707,7 @@ D_SRC::resampleInterleaved(float *const BQ_R__ out,
         outcount = int(ceil(incount * ratio) + 5);
     }
 
-    if (m_ratioUnset) {
+    if (m_ratioUnset || !m_smoothRatios) {
 
         // The first time we set a ratio, we want to do it directly
         src_set_ratio(m_src, ratio);
@@ -759,14 +774,13 @@ D_SRC::reset()
 
 #endif /* HAVE_LIBSAMPLERATE */
 
-#ifdef HAVE_LIBRESAMPLE
-
-class D_Resample : public Resampler::Impl
+#ifdef USE_BQRESAMPLER
+    
+class D_BQResampler : public Resampler::Impl
 {
 public:
-    D_Resample(Resampler::Quality quality, int channels, double initialSampleRate,
-               int maxBufferSize, int m_debugLevel);
-    ~D_Resample();
+    D_BQResampler(Resampler::Parameters params, int channels);
+    ~D_BQResampler();
 
     int resample(float *const BQ_R__ *const BQ_R__ out,
                  int outcount,
@@ -780,178 +794,147 @@ public:
                             const float *const BQ_R__ in,
                             int incount,
                             double ratio,
-                            bool final);
+                            bool final = false);
 
-    int getChannelCount() const { return m_channels; }
+    int getChannelCount() const {
+        return m_channels;
+    }
+
+    double getEffectiveRatio(double ratio) const {
+        return m_resampler->getEffectiveRatio(ratio);
+    }
 
     void reset();
 
 protected:
-    void *m_src;
+    BQResampler *m_resampler;
     float *m_iin;
     float *m_iout;
-    double m_lastRatio;
     int m_channels;
     int m_iinsize;
     int m_ioutsize;
     int m_debugLevel;
 };
 
-D_Resample::D_Resample(Resampler::Quality quality,
-                       int channels, double, int maxBufferSize, int debugLevel) :
-    m_src(0),
+D_BQResampler::D_BQResampler(Resampler::Parameters params, int channels) :
+    m_resampler(0),
     m_iin(0),
     m_iout(0),
     m_channels(channels),
     m_iinsize(0),
     m_ioutsize(0),
-    m_debugLevel(debugLevel)
+    m_debugLevel(params.debugLevel)
 {
     if (m_debugLevel > 0) {
-        cerr << "Resampler::Resampler: using libresample implementation"
-                  << endl;
+        cerr << "Resampler::Resampler: using implementation: BQResampler" << endl;
     }
 
-    float min_factor = 0.125f;
-    float max_factor = 8.0f;
-
-    m_src = resample_open(quality == Resampler::Best ? 1 : 0, min_factor, max_factor);
-
-    if (!m_src) {
-        cerr << "Resampler::Resampler: failed to create libresample resampler: " 
-                  << endl;
-        throw Resampler::ImplementationError; //!!! of course, need to catch this!
+    BQResampler::Parameters rparams;
+    switch (params.quality) {
+    case Resampler::Best:
+        rparams.quality = BQResampler::Best;
+        break;
+    case Resampler::FastestTolerable:
+        rparams.quality = BQResampler::FastestTolerable;
+        break;
+    case Resampler::Fastest:
+        rparams.quality = BQResampler::Fastest;
+        break;
     }
+    switch (params.dynamism) {
+    case Resampler::RatioOftenChanging:
+        rparams.dynamism = BQResampler::RatioOftenChanging;
+        break;
+    case Resampler::RatioMostlyFixed:
+        rparams.dynamism = BQResampler::RatioMostlyFixed;
+        break;
+    }
+    switch (params.ratioChange) {
+    case Resampler::SmoothRatioChange:
+        rparams.ratioChange = BQResampler::SmoothRatioChange;
+        break;
+    case Resampler::SuddenRatioChange:
+        rparams.ratioChange = BQResampler::SuddenRatioChange;
+        break;
+    }
+    rparams.referenceSampleRate = params.initialSampleRate;
+    rparams.debugLevel = params.debugLevel;
 
-    if (maxBufferSize > 0 && m_channels > 1) {
-        m_iinsize = maxBufferSize * m_channels;
-        m_ioutsize = maxBufferSize * m_channels * 2;
+    m_resampler = new BQResampler(rparams, m_channels);
+    
+    if (params.maxBufferSize > 0 && m_channels > 1) {
+        m_iinsize = params.maxBufferSize * m_channels;
+        m_ioutsize = params.maxBufferSize * m_channels * 2;
         m_iin = allocate<float>(m_iinsize);
         m_iout = allocate<float>(m_ioutsize);
     }
-
-    reset();
 }
 
-D_Resample::~D_Resample()
+D_BQResampler::~D_BQResampler()
 {
-    resample_close(m_src);
-    if (m_iinsize > 0) {
-        deallocate(m_iin);
-    }
-    if (m_ioutsize > 0) {
-        deallocate(m_iout);
-    }
+    delete m_resampler;
+    deallocate(m_iin);
+    deallocate(m_iout);
 }
 
 int
-D_Resample::resample(float *const BQ_R__ *const BQ_R__ out,
-                     int outcount,
-                     const float *const BQ_R__ *const BQ_R__ in,
-                     int incount,
-                     double ratio,
-                     bool final)
+D_BQResampler::resample(float *const BQ_R__ *const BQ_R__ out,
+                        int outcount,
+                        const float *const BQ_R__ *const BQ_R__ in,
+                        int incount,
+                        double ratio,
+                        bool final)
 {
-    float *data_in;
-    float *data_out;
-    int input_frames, output_frames, end_of_input, source_used;
-    float src_ratio;
-
-    int outcount = (int)lrint(ceil(incount * ratio));
-
     if (m_channels == 1) {
-        data_in = const_cast<float *>(*in); //!!!???
-        data_out = *out;
-    } else {
-        if (incount * m_channels > m_iinsize) {
-            m_iin = reallocate<float>(m_iin, m_iinsize, incount * m_channels);
-            m_iinsize = incount * m_channels;
-        }
-        if (outcount * m_channels > m_ioutsize) {
-            m_iout = reallocate<float>(m_iout, m_ioutsize, outcount * m_channels);
-            m_ioutsize = outcount * m_channels;
-        }
-        v_interleave(m_iin, in, m_channels, incount);
-        data_in = m_iin;
-        data_out = m_iout;
+        return resampleInterleaved(*out, outcount, *in, incount, ratio, final);
     }
 
-    input_frames = incount;
-    output_frames = outcount;
-    src_ratio = ratio;
-    end_of_input = (final ? 1 : 0);
-
-    int output_frames_gen = resample_process(m_src,
-                                             src_ratio,
-                                             data_in,
-                                             input_frames,
-                                             end_of_input,
-                                             &source_used,
-                                             data_out,
-                                             output_frames);
-
-    if (output_frames_gen < 0) {
-        cerr << "Resampler::process: libresample error: "
-                  << endl;
-        throw Resampler::ImplementationError; //!!! of course, need to catch this!
+    if (incount * m_channels > m_iinsize) {
+        m_iin = reallocate<float>(m_iin, m_iinsize, incount * m_channels);
+        m_iinsize = incount * m_channels;
     }
-
-    if (m_channels > 1) {
-        v_deinterleave(out, m_iout, m_channels, output_frames_gen);
+    if (outcount * m_channels > m_ioutsize) {
+        m_iout = reallocate<float>(m_iout, m_ioutsize, outcount * m_channels);
+        m_ioutsize = outcount * m_channels;
     }
+    
+    v_interleave(m_iin, in, m_channels, incount);
+    
+    int n = resampleInterleaved(m_iout, outcount, m_iin, incount, ratio, final);
 
-    return output_frames_gen;
+    v_deinterleave(out, m_iout, m_channels, n);
+
+    return n;
 }
 
 int
-D_Resample::resampleInterleaved(float *const BQ_R__ out,
-                                int outcount,
-                                const float *const BQ_R__ in,
-                                int incount,
-                                double ratio,
-                                bool final)
+D_BQResampler::resampleInterleaved(float *const BQ_R__ out,
+                                   int outcount,
+                                   const float *const BQ_R__ in,
+                                   int incount,
+                                   double ratio,
+                                   bool final)
 {
-    int input_frames, output_frames, end_of_input, source_used;
-    float src_ratio;
-
-    int outcount = (int)lrint(ceil(incount * ratio));
-
-    input_frames = incount;
-    output_frames = outcount;
-    src_ratio = ratio;
-    end_of_input = (final ? 1 : 0);
-
-    int output_frames_gen = resample_process(m_src,
-                                             src_ratio,
-                                             const_cast<float *>(in),
-                                             input_frames,
-                                             end_of_input,
-                                             &source_used,
-                                             out,
-                                             output_frames);
-
-    if (output_frames_gen < 0) {
-        cerr << "Resampler::process: libresample error: "
-                  << endl;
-        throw Resampler::ImplementationError; //!!! of course, need to catch this!
-    }
-
-    return output_frames_gen;
+    return m_resampler->resampleInterleaved(out, outcount,
+                                            in, incount,
+                                            ratio, final);
 }
 
 void
-D_Resample::reset()
+D_BQResampler::reset()
 {
+    m_resampler->reset();
 }
 
-#endif /* HAVE_LIBRESAMPLE */
+#endif /* USE_BQRESAMPLER */
 
-#ifdef USE_SPEEX
+#if defined(USE_SPEEX) || defined(HAVE_LIBSPEEXDSP)
     
 class D_Speex : public Resampler::Impl
 {
 public:
-    D_Speex(Resampler::Quality quality, int channels, double initialSampleRate,
+    D_Speex(Resampler::Quality quality, Resampler::RatioChange,
+            int channels, double initialSampleRate,
             int maxBufferSize, int debugLevel);
     ~D_Speex();
 
@@ -970,6 +953,7 @@ public:
                             bool final = false);
 
     int getChannelCount() const { return m_channels; }
+    double getEffectiveRatio(double ratio) const { return ratio; }
 
     void reset();
 
@@ -991,7 +975,7 @@ protected:
                     double ratio, bool final);
 };
 
-D_Speex::D_Speex(Resampler::Quality quality,
+D_Speex::D_Speex(Resampler::Quality quality, Resampler::RatioChange,
                  int channels, double initialSampleRate,
                  int maxBufferSize, int debugLevel) :
     m_resampler(0),
@@ -1009,7 +993,7 @@ D_Speex::D_Speex(Resampler::Quality quality,
              quality == Resampler::Fastest ? 0 : 4);
 
     if (m_debugLevel > 0) {
-        cerr << "Resampler::Resampler: using Speex implementation with q = "
+        cerr << "Resampler::Resampler: using implementation: Speex with q = "
              << q << endl;
     }
 
@@ -1052,18 +1036,22 @@ D_Speex::setRatio(double ratio)
     // Speex wants a ratio of two unsigned integers, not a single
     // float.  Let's do that.
 
-    unsigned int big = 272408136U; 
-    unsigned int denom = 1, num = 1;
-
-    if (ratio < 1.f) {
-        denom = big;
-        double dnum = double(big) * double(ratio);
-        num = (unsigned int)dnum;
-    } else if (ratio > 1.f) {
-        num = big;
-        double ddenom = double(big) / double(ratio);
-        denom = (unsigned int)ddenom;
+    int max_denom = 48000;
+    if (ratio > 1.0) {
+        max_denom = int(ceil(48000 / ratio));
     }
+
+    int inum, idenom;
+    pickNearestRational(ratio, max_denom, inum, idenom);
+
+    if (inum < 0 || idenom < 0) {
+        cerr << "Resampler::setRatio: Internal error: "
+             << "numerator or denominator < 0 ("
+             << inum << "/" << idenom << ")" << endl;
+        return;
+    }
+    
+    unsigned int num = inum, denom = idenom;
     
     if (m_debugLevel > 1) {
         cerr << "D_Speex: Desired ratio " << ratio << ", requesting ratio "
@@ -1078,8 +1066,12 @@ D_Speex::setRatio(double ratio)
         (m_resampler, denom, num, fromRate, toRate);
 
     if (err) {
-        cerr << "Resampler::Resampler: failed to set rate on Speex resampler" 
-             << endl;
+        cerr << "Resampler::Resampler: failed to set rate on Speex resampler"
+             << " (with ratio = " << ratio << " [ratio-1 = " << ratio - 1.0
+             << "], denom = " << denom
+             << ", num = " << num << ", fromRate = " << fromRate
+             << ", toRate = " << toRate << ", err = " << err
+             << ")" << endl;
 #ifndef NO_EXCEPTIONS
         throw Resampler::ImplementationError;
 #endif
@@ -1190,26 +1182,37 @@ D_Speex::doResample(const float *data_in, unsigned int &uincount,
     if (final) {
         int actual = int(uoutcount);
         int expected = std::min(initial_outcount, int(round(uincount * ratio)));
-        if (actual < expected) {
+        float *pad = nullptr;
+        while (actual < expected) {
             unsigned int final_out = expected - actual;
             unsigned int final_in = (unsigned int)(round(final_out / ratio));
-            if (final_in > 0) {
-                float *pad = allocate_and_zero<float>(final_in * m_channels);
+            if (final_in == 0) {
+                break;
+            } else {
+                if (!pad) {
+                    pad = allocate_and_zero<float>(final_in * m_channels);
+                }
                 err = speex_resampler_process_interleaved_float
                     (m_resampler,
                      pad, &final_in,
                      data_out + actual * m_channels, &final_out);
-                deallocate(pad);
+                actual += final_out;
                 uoutcount += final_out;
                 if (err) {
                     cerr << "Resampler::Resampler: Speex resampler returned error "
                          << err << endl;
+                    if (pad) {
+                        deallocate(pad);
+                    }
 #ifndef NO_EXCEPTIONS
                     throw Resampler::ImplementationError;
 #endif
                 }
             }
         }
+        if (pad) {
+            deallocate(pad);
+        }
     }
 }
 
@@ -1242,7 +1245,10 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
 #ifdef USE_SPEEX
         m_method = 2;
 #endif
-#ifdef HAVE_LIBRESAMPLE
+#ifdef HAVE_LIBSPEEXDSP
+        m_method = 2;
+#endif
+#ifdef USE_BQRESAMPLER
         m_method = 3;
 #endif
 #ifdef HAVE_LIBSAMPLERATE
@@ -1254,14 +1260,17 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
 #ifdef HAVE_IPP
         m_method = 0;
 #endif
-#ifdef HAVE_LIBRESAMPLE
-        m_method = 3;
-#endif
 #ifdef USE_SPEEX
         m_method = 2;
 #endif
+#ifdef USE_BQRESAMPLER
+        m_method = 3;
+#endif
 #ifdef HAVE_LIBSAMPLERATE
         m_method = 1;
+#endif
+#ifdef HAVE_LIBSPEEXDSP
+        m_method = 2;
 #endif
         break;
 
@@ -1269,14 +1278,17 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
 #ifdef HAVE_IPP
         m_method = 0;
 #endif
-#ifdef HAVE_LIBRESAMPLE
-        m_method = 3;
-#endif
 #ifdef USE_SPEEX
         m_method = 2;
 #endif
+#ifdef USE_BQRESAMPLER
+        m_method = 3;
+#endif
 #ifdef HAVE_LIBSAMPLERATE
         m_method = 1;
+#endif
+#ifdef HAVE_LIBSPEEXDSP
+        m_method = 2;
 #endif
         break;
     }
@@ -1290,7 +1302,7 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
     case 0:
 #ifdef HAVE_IPP
         d = new Resamplers::D_IPP
-            (params.quality,
+            (params.quality, params.ratioChange,
              channels,
              params.initialSampleRate, params.maxBufferSize, params.debugLevel);
 #else
@@ -1302,7 +1314,7 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
     case 1:
 #ifdef HAVE_LIBSAMPLERATE
         d = new Resamplers::D_SRC
-            (params.quality,
+            (params.quality, params.ratioChange,
              channels,
              params.initialSampleRate, params.maxBufferSize, params.debugLevel);
 #else
@@ -1312,9 +1324,9 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
         break;
 
     case 2:
-#ifdef USE_SPEEX
+#if defined(USE_SPEEX) || defined(HAVE_LIBSPEEXDSP)
         d = new Resamplers::D_Speex
-            (params.quality,
+            (params.quality, params.ratioChange,
              channels,
              params.initialSampleRate, params.maxBufferSize, params.debugLevel);
 #else
@@ -1324,11 +1336,10 @@ Resampler::Resampler(Resampler::Parameters params, int channels)
         break;
 
     case 3:
-#ifdef HAVE_LIBRESAMPLE
-        d = new Resamplers::D_Resample
-            (params.quality,
-             channels,
-             params.initialSampleRate, params.maxBufferSize, params.debugLevel);
+
+    case 4:
+#ifdef USE_BQRESAMPLER
+        d = new Resamplers::D_BQResampler(params, channels);
 #else
         cerr << "Resampler::Resampler: No implementation available!" << endl;
         abort();
@@ -1376,6 +1387,12 @@ Resampler::getChannelCount() const
     return d->getChannelCount();
 }
 
+double
+Resampler::getEffectiveRatio(double ratio) const
+{
+    return d->getEffectiveRatio(ratio);
+}
+
 void
 Resampler::reset()
 {
