@@ -4,13 +4,13 @@
 
 set -eu
 
-current=$(hg id | awk '{ print $1; }')
-
-case "$current" in
-    *+) echo "ERROR: Current working copy has been modified - unmodified copy required so we know we can check it out separately and obtain the same contents"; exit 2;;
-    *);;
+case $(git status --porcelain --untracked-files=no) in
+    "") ;;
+    *) echo "ERROR: Current working copy has been modified - unmodified copy required so we know we can check it out separately and obtain the same contents"; exit 2;;
 esac
 
+current=$(git rev-parse --short HEAD)
+
 echo
 echo "Building appimage from revision $current..."
 
@@ -38,7 +38,7 @@ mkdir -p "$outdir"
 
 container=$(sudo docker create "$dockertag")
 
-sudo docker cp "$container":output-appimage.tar "$outdir"
+sudo docker cp "$container":/tmp/output-appimage.tar "$outdir"
 sudo docker rm "$container"
 
 ( cd "$outdir" ; tar xf output-appimage.tar && rm -f output-appimage.tar )
