@@ -22,7 +22,6 @@
 #include "base/PropertyContainer.h"
 #include "base/Preferences.h"
 #include "data/fileio/FileSource.h"
-#include "widgets/TipDialog.h"
 #include "widgets/InteractiveFileFinder.h"
 #include "framework/TransformUserConfigurator.h"
 #include "transform/TransformFactory.h"
@@ -182,6 +181,8 @@
 
 */
 
+using namespace sv;
+
 static QMutex cleanupMutex;
 static bool cleanedUp = false;
 
@@ -190,7 +191,7 @@ signalHandler(int /* signal */)
 {
     // Avoid this happening more than once across threads
 
-    cerr << "signalHandler: cleaning up and exiting" << endl;
+    std::cerr << "signalHandler: cleaning up and exiting" << std::endl;
 
     if (cleanupMutex.tryLock(5000)) {
         if (!cleanedUp) {
@@ -246,7 +247,7 @@ main(int argc, char **argv)
 {
     if (argc == 2 && (QString(argv[1]) == "--version" ||
                       QString(argv[1]) == "-v")) {
-        cerr << SV_VERSION << endl;
+        std::cerr << SV_VERSION << std::endl;
         exit(0);
     }
     
@@ -263,6 +264,8 @@ main(int argc, char **argv)
     QApplication::setDesktopFileName("sonic-visualiser");
 #endif
 
+    SVCerr::installQtMessageHandler();
+
     QCommandLineParser parser;
     parser.setApplicationDescription(QApplication::tr("\nSonic Visualiser is a program for viewing and exploring audio data\nfor semantic music analysis and annotation."));
     parser.addHelpOption();
@@ -318,8 +321,6 @@ main(int argc, char **argv)
     
     args = parser.positionalArguments();
 
-    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-
     signal(SIGINT,  signalHandler);
     signal(SIGTERM, signalHandler);
 
@@ -332,18 +333,25 @@ main(int argc, char **argv)
 
     QSettings settings;
 
-    QString language = QLocale::system().name();
-    SVDEBUG << "System language is: " << language << endl;
-
+    QLocale locale;
+    SVDEBUG << "System locale lists the following UI languages: "
+            << locale.uiLanguages().join("; ") << endl;
+    
     settings.beginGroup("Preferences");
-    QString prefLanguage = settings.value("locale", language).toString();
-    if (prefLanguage != QString()) language = prefLanguage;
+    QString prefLanguage = settings.value("locale", "").toString();
+    if (prefLanguage != QString()) {
+        locale = QLocale(prefLanguage);
+        SVDEBUG << "Loaded language preference \"" << prefLanguage
+                << "\" from settings; locale now lists: "
+                << locale.uiLanguages().join("; ") << endl;
+    }
     settings.endGroup();
 
     settings.beginGroup("Preferences");
     if (!(settings.value("always-use-default-font", false).toBool())) {
 #ifdef Q_OS_WIN32
-        if (!language.startsWith("ru_")) { // + any future non-Latin i18ns
+        if (locale.uiLanguages().empty() ||
+            !locale.uiLanguages()[0].startsWith("ru_")) { // + any future non-Latin i18ns
             QFont font(QApplication::font());
             QString preferredFamily = "Segoe UI";
             font.setFamily(preferredFamily);
@@ -379,7 +387,7 @@ main(int argc, char **argv)
     if (showSplash) {
         splash = new SVSplash();
         splash->show();
-        QTimer::singleShot(5000, splash, SLOT(hide()));
+        QTimer::singleShot(3000, splash, SLOT(hide()));
         application.processEvents();
     }
 
@@ -418,29 +426,27 @@ main(int argc, char **argv)
     }
 
     QTranslator qtTranslator;
-    QString qtTrName = QString("qt_%1").arg(language);
-    SVDEBUG << "Loading " << qtTrName << "... ";
-    bool success = false;
-    if (!(success = qtTranslator.load(qtTrName))) {
+    if (!qtTranslator.load(locale, "qtbase", "_", ":/i18n", ".qm")) {
         QString qtDir = getenv("QTDIR");
-        if (qtDir != "") {
-            success = qtTranslator.load
-                (qtTrName, QDir(qtDir).filePath("translations"));
-        }
-    }
-    if (!success) {
-        SVDEBUG << "Failed\nFailed to load Qt translation for locale" << endl;
+        if (qtDir == "" ||
+            !qtTranslator.load(locale, QDir(qtDir).filePath("translations"))) {
+            SVDEBUG << "Failed to load Qt translations for locale" << endl;
+        } else {
+            SVDEBUG << "Loaded Qt translations from QTDIR" << endl;
+            application.installTranslator(&qtTranslator);
+        }            
     } else {
-        SVDEBUG << "Done" << endl;
+        SVDEBUG << "Loaded Qt translations from bundle" << endl;
+        application.installTranslator(&qtTranslator);
     }
-    application.installTranslator(&qtTranslator);
 
     QTranslator svTranslator;
-    QString svTrName = QString("sonic-visualiser_%1").arg(language);
-    SVDEBUG << "Loading " << svTrName << "... ";
-    svTranslator.load(svTrName, ":i18n");
-    SVDEBUG << "Done" << endl;
-    application.installTranslator(&svTranslator);
+    if (!svTranslator.load(locale, "sonic-visualiser", "_", ":/i18n", ".qm")) {
+        SVDEBUG << "Failed to load SV translations for locale" << endl;
+    } else {
+        SVDEBUG << "Loaded SV translations" << endl;
+        application.installTranslator(&svTranslator);
+    }
 
     StoreStartupLocale();
 
@@ -577,7 +583,7 @@ main(int argc, char **argv)
     {
         char *cwisdom = fftwf_export_wisdom_to_string();
         if (cwisdom) {
-            settings.setValue("wisdom", cwisdom);
+            settings.setValue("wisdom", QString::fromLocal8Bit(cwisdom));
             free(cwisdom);
         }
     }
@@ -586,7 +592,7 @@ main(int argc, char **argv)
     {
         char *cwisdom = fftw_export_wisdom_to_string();
         if (cwisdom) {
-            settings.setValue("wisdom_d", cwisdom);
+            settings.setValue("wisdom_d", QString::fromLocal8Bit(cwisdom));
             free(cwisdom);
         }
     }
@@ -594,6 +600,7 @@ main(int argc, char **argv)
     settings.endGroup();
 
     FileSource::debugReport();
+    Profiles::getInstance()->dump();
     
     delete gui;
 
