@@ -19,6 +19,7 @@
 #include <QDebug>
 #include <QTextStream>
 #include <QElapsedTimer>
+#include <QThread>
 
 #include "RealTime.h"
 
@@ -33,21 +34,19 @@ QDebug &operator<<(QDebug &, const std::string &);
 std::ostream &operator<<(std::ostream &, const QString &);
 std::ostream &operator<<(std::ostream &, const QUrl &);
 
-using std::cout;
-using std::cerr;
-using std::endl;
-
 class SVDebug {
 public:
     SVDebug();
     ~SVDebug();
 
     template <typename T>
-    inline SVDebug &operator<<(const T &t) {
+    SVDebug &operator<<(const T &t) {
         if (m_silenced) return *this;
         if (m_ok) {
             if (m_eol) {
-                m_stream << m_prefix << "/" << m_timer.elapsed() << ": ";
+                m_stream << m_prefix << "/"
+                         << (uint64_t)QThread::currentThreadId() << ":"
+                         << m_timer.elapsed() << ": ";
             }
             m_stream << t;
             m_eol = false;
@@ -55,7 +54,7 @@ public:
         return *this;
     }
 
-    inline SVDebug &operator<<(QTextStreamFunction) {
+    SVDebug &operator<<(QTextStreamFunction) {
         if (m_silenced) return *this;
         m_stream << std::endl;
         m_eol = true;
@@ -63,6 +62,8 @@ public:
     }
 
     static void silence() { m_silenced = true; }
+
+    static void installQtMessageHandler();
     
 private:
     std::fstream m_stream;
@@ -81,18 +82,20 @@ public:
     inline SVCerr &operator<<(const T &t) {
         if (m_silenced) return *this;
         m_d << t;
-        cerr << t;
+        std::cerr << t;
         return *this;
     }
 
     inline SVCerr &operator<<(QTextStreamFunction f) {
         if (m_silenced) return *this;
         m_d << f;
-        cerr << std::endl;
+        std::cerr << std::endl;
         return *this;
     }
 
     static void silence() { m_silenced = true; }
+
+    static void installQtMessageHandler();
     
 private:
     SVDebug &m_d;
@@ -102,11 +105,28 @@ private:
 extern SVDebug &getSVDebug();
 extern SVCerr &getSVCerr();
 
+using Qt::endl;
+
 // Writes to debug log only
 #define SVDEBUG getSVDebug()
 
 // Writes to both SVDEBUG and cerr
 #define SVCERR getSVCerr()
 
+class FunctionLogger
+{
+public:
+    FunctionLogger(const char *name);
+    ~FunctionLogger();
+private:
+    const char *m_name;
+};
+
+#ifdef NDEBUG
+#define FUNCLOG
+#else
+#define FUNCLOG FunctionLogger functionLogger(__FUNCTION__)
+#endif
+
 #endif /* !_DEBUG_H_ */
 
