@@ -18,6 +18,8 @@
 
 #include <QString>
 
+namespace sv {
+
 class Playable
 {
 public:
@@ -28,4 +30,6 @@ public:
     virtual bool getDefaultPlayAudible() const { return true; }
 };
 
+} // end namespace sv
+
 #endif
