@@ -18,6 +18,8 @@
 
 #include "ProgressReporter.h"
 
+namespace sv {
+
 class ProgressPrinter : public ProgressReporter
 {
     Q_OBJECT
@@ -42,4 +44,6 @@ protected:
     bool m_definite;
 };
 
+} // end namespace sv
+
 #endif
