@@ -19,6 +19,8 @@
 #include <QObject>
 #include <QString>
 
+namespace sv {
+
 class ProgressReporter : public QObject
 {
     Q_OBJECT
@@ -40,4 +42,6 @@ public slots:
     virtual void setProgress(int percentage) = 0;
 };
 
+} // end namespace sv
+
 #endif
