@@ -29,6 +29,8 @@
 #include <QMutex>
 #include <iostream>
 
+namespace sv {
+
 /**
  * A very simple class that facilitates running things like plugins
  * without locking, by collecting unwanted objects and deleting them
@@ -199,4 +201,6 @@ Scavenger<T>::clearExcess(time_t sec)
     m_excessMutex.unlock();
 }
 
+} // end namespace sv
+
 #endif
