@@ -21,6 +21,8 @@
 
 #include <QString>
 
+namespace sv {
+
 QString
 DataFileReaderFactory::getKnownExtensions()
 {
@@ -144,3 +146,5 @@ DataFileReaderFactory::loadCSV(QString path, CSVFormat format,
     }
 }
 
+} // end namespace sv
+
