@@ -18,6 +18,7 @@
 #include "EncodingTest.h"
 #include "MIDIFileReaderTest.h"
 #include "CSVFormatTest.h"
+#include "CSVReaderTest.h"
 #include "CSVStreamWriterTest.h"
 
 #include "system/Init.h"
@@ -89,12 +90,20 @@ int main(int argc, char *argv[])
         else ++bad;
     }
 
+    {
+        CSVReaderTest t(testDir);
+        if (QTest::qExec(&t, argc, argv) == 0) ++good;
+        else ++bad;
+    }
+
     {
         CSVStreamWriterTest t;
         if (QTest::qExec(&t, argc, argv) == 0) ++good;
         else ++bad;
     }
 
+    (void)good;
+    
     if (bad > 0) {
         SVCERR << "\n********* " << bad << " test suite(s) failed!\n" << endl;
         return 1;
