@@ -22,6 +22,8 @@
 
 #include <vector>
 
+namespace sv {
+
 class EditableDenseThreeDimensionalModel : public DenseThreeDimensionalModel
 {
     Q_OBJECT
@@ -103,6 +105,12 @@ public:
      */
     Column getColumn(int x) const override;
 
+    /**
+     * Get the set of bin values at the given column, within the given
+     * bin range.
+     */
+    Column getColumn(int x, int minbin, int nbins) const override;
+
     /**
      * Get a single value, from the n'th bin of the given column.
      */
@@ -220,4 +228,6 @@ protected:
     mutable QMutex m_mutex;
 };
 
+} // end namespace sv
+
 #endif
