@@ -30,6 +30,8 @@
 
 #include <QMutexLocker>
 
+namespace sv {
+
 class NoteModel : public Model,
                   public TabularModel,
                   public NoteExportable,
@@ -211,7 +213,7 @@ public:
            
         m_events.add(e);
         float v = e.getValue();
-        if (!ISNAN(v) && !ISINF(v)) {
+        if (!std::isnan(v) && !std::isinf(v)) {
             if (!m_haveExtents || v < m_valueMinimum) {
                 m_valueMinimum = v; allChange = true;
             }
@@ -443,4 +445,6 @@ protected:
     EventSeries m_events;
 };
 
+} // end namespace sv
+
 #endif
