@@ -31,6 +31,8 @@
 
 #include <atomic>
 
+namespace sv {
+
 class AudioFileReader;
 
 class ReadOnlyWaveFileModel : public WaveFileModel
@@ -39,18 +41,28 @@ class ReadOnlyWaveFileModel : public WaveFileModel
 
 public:
     /**
-     * Construct a WaveFileModel from a source path and optional
-     * resampling target rate
+     * Construct a WaveFileModel from a source and optional resampling
+     * target rate.
      */
     ReadOnlyWaveFileModel(FileSource source, sv_samplerate_t targetRate = 0);
 
     /**
-     * Construct a WaveFileModel from a source path using an existing
+     * Construct a WaveFileModel from a source and an existing
      * AudioFileReader. The model does not take ownership of the
      * AudioFileReader, which remains managed by the caller and must
      * outlive the model.
      */
     ReadOnlyWaveFileModel(FileSource source, AudioFileReader *reader);
+
+    /**
+     * Construct a WaveFileModel from a nominal filename or path using
+     * an existing AudioFileReader. The path is not resolved, but used
+     * only to identify the source when serialised; it may be any
+     * string. The model does not take ownership of the
+     * AudioFileReader, which remains managed by the caller and must
+     * outlive the model.
+     */
+    ReadOnlyWaveFileModel(QString path, AudioFileReader *reader);
     
     ~ReadOnlyWaveFileModel();
 
@@ -108,7 +120,7 @@ protected slots:
 protected:
     void initialize();
 
-    class RangeCacheFillThread : public Thread
+    class RangeCacheFillThread : public QThread
     {
     public:
         RangeCacheFillThread(ReadOnlyWaveFileModel &model) :
@@ -126,7 +138,6 @@ protected:
          
     void fillCache();
 
-    FileSource m_source;
     QString m_path;
     AudioFileReader *m_reader;
     bool m_myReader;
@@ -148,4 +159,6 @@ protected:
     mutable QMutex m_directReadMutex;
 };    
 
+} // end namespace sv
+
 #endif
