@@ -17,6 +17,8 @@
 
 #include "base/BaseTypes.h"
 
+namespace sv {
+
 class DenseTimeValueModel;
 
 /** Oversample the sample data from a DenseTimeValueModel by an
@@ -54,4 +56,6 @@ private:
     static floatvec_t m_filter;
 };
 
+} // end namespace sv
+
 #endif
