@@ -26,6 +26,8 @@
 
 #include <memory>
 
+namespace sv {
+
 class FeatureExtractionPluginFactory
 {
 public:
@@ -66,4 +68,6 @@ public:
     virtual QString getPluginLibraryPath(QString identifier) = 0;
 };
 
+} // end namespace sv
+
 #endif
