@@ -31,6 +31,8 @@
 #include <set>
 #include <QString>
 
+namespace sv {
+
 class LADSPAPluginInstance;
 
 class LADSPAPluginFactory : public RealTimePluginFactory
@@ -102,5 +104,7 @@ protected:
     LibraryHandleMap m_libraryHandles;
 };
 
+} // end namespace sv
+
 #endif
 
