@@ -15,8 +15,7 @@
 
 #include "PluginXml.h"
 
-#include <QRegExp>
-#include <QXmlAttributes>
+#include <QRegularExpression>
 
 #include <QDomDocument>
 #include <QDomElement>
@@ -30,6 +29,8 @@
 
 #include <iostream>
 
+namespace sv {
+
 using std::shared_ptr;
 using std::dynamic_pointer_cast;
 
@@ -119,14 +120,14 @@ PluginXml::toXml(QTextStream &stream,
 #define CHECK_ATTRIBUTE(ATTRIBUTE, ACCESSOR) \
     QString ATTRIBUTE = attrs.value(#ATTRIBUTE); \
     if (ATTRIBUTE != "" && ATTRIBUTE != ACCESSOR().c_str()) { \
-        cerr << "WARNING: PluginXml::setParameters: Plugin " \
+        SVCERR << "WARNING: PluginXml::setParameters: Plugin " \
                   << #ATTRIBUTE << " does not match (attributes have \"" \
                   << ATTRIBUTE << "\", my " \
                   << #ATTRIBUTE << " is \"" << ACCESSOR() << "\")" << endl; \
     }
 
 void
-PluginXml::setParameters(const QXmlAttributes &attrs)
+PluginXml::setParameters(const Attributes &attrs)
 {
     CHECK_ATTRIBUTE(identifier, m_plugin->getIdentifier);
     CHECK_ATTRIBUTE(name, m_plugin->getName);
@@ -137,7 +138,7 @@ PluginXml::setParameters(const QXmlAttributes &attrs)
     bool ok;
     int version = attrs.value("version").trimmed().toInt(&ok);
     if (ok && version != m_plugin->getPluginVersion()) {
-        cerr << "WARNING: PluginXml::setParameters: Plugin version does not match (attributes have " << version << ", my version is " << m_plugin->getPluginVersion() << ")" << endl;
+        SVCERR << "WARNING: PluginXml::setParameters: Plugin version does not match (attributes have " << version << ", my version is " << m_plugin->getPluginVersion() << ")" << endl;
     }
 
     auto rtpi = dynamic_pointer_cast<RealTimePluginInstance>(m_plugin);
@@ -149,7 +150,7 @@ PluginXml::setParameters(const QXmlAttributes &attrs)
                  i != configList.end(); ++i) {
                 QStringList kv = i->split("=");
                 if (kv.count() < 2) {
-                    cerr << "WARNING: PluginXml::setParameters: Malformed configure pair string: \"" << *i << "\"" << endl;
+                    SVCERR << "WARNING: PluginXml::setParameters: Malformed configure pair string: \"" << *i << "\"" << endl;
                     continue;
                 }
                 QString key(kv[0]), value(kv[1]);
@@ -185,7 +186,7 @@ PluginXml::setParameters(const QXmlAttributes &attrs)
 //                      << i->identifier << "\" to value " << value << endl;
             m_plugin->setParameter(i->identifier, value);
         } else {
-            cerr << "WARNING: PluginXml::setParameters: Invalid value \"" << attrs.value(pname) << "\" for parameter \"" << i->identifier << "\" (attribute \"" << pname << "\")" << endl;
+            SVCERR << "WARNING: PluginXml::setParameters: Invalid value \"" << attrs.value(pname) << "\" for parameter \"" << i->identifier << "\" (attribute \"" << pname << "\")" << endl;
         }
     }
 }
@@ -203,22 +204,22 @@ PluginXml::setParametersFromXml(QString xml)
 //              << xml << "\"" << endl;
 
     if (!doc.setContent(xml, false, &error, &errorLine, &errorColumn)) {
-        cerr << "PluginXml::setParametersFromXml: Error in parsing XML: " << error << " at line " << errorLine << ", column " << errorColumn << endl;
-        cerr << "Input follows:" << endl;
-        cerr << xml << endl;
-        cerr << "Input ends." << endl;
+        SVCERR << "PluginXml::setParametersFromXml: Error in parsing XML: " << error << " at line " << errorLine << ", column " << errorColumn << endl;
+        SVCERR << "Input follows:" << endl;
+        SVCERR << xml << endl;
+        SVCERR << "Input ends." << endl;
         return;
     }
 
     QDomElement pluginElt = doc.firstChildElement("plugin");
     QDomNamedNodeMap attrNodes = pluginElt.attributes();
-    QXmlAttributes attrs;
+    Attributes attrs;
 
     for (int i = 0; i < attrNodes.length(); ++i) {
         QDomAttr attr = attrNodes.item(i).toAttr();
         if (attr.isNull()) continue;
 //        SVDEBUG << "PluginXml::setParametersFromXml: Adding attribute \"" << attr.name()//                  << "\" with value \"" << attr.value() << "\"" << endl;
-        attrs.append(attr.name(), "", "", attr.value());
+        attrs[attr.name()] = attr.value();
     }
 
     setParameters(attrs);
@@ -227,7 +228,9 @@ PluginXml::setParametersFromXml(QString xml)
 QString
 PluginXml::stripInvalidParameterNameCharacters(QString s) const
 {
-    s.replace(QRegExp("[^a-zA-Z0-9_]*"), "");
+    s.replace(QRegularExpression("[^a-zA-Z0-9_]*"), "");
     return s;
 }
 
+} // end namespace sv
+
