@@ -24,7 +24,7 @@
 
 #include <dataquay/BasicStore.h>
 
-#include <iostream>
+namespace sv {
 
 using Dataquay::Uri;
 using Dataquay::Node;
@@ -416,3 +416,5 @@ PluginRDFDescription::indexOutputs()
     return true;
 }
 
+} // end namespace sv
+
