@@ -41,9 +41,12 @@
 
 //#define DEBUG_FEATURE_EXTRACTION_TRANSFORMER_RUN 1
 
+namespace sv {
+
 FeatureExtractionModelTransformer::FeatureExtractionModelTransformer(Input in,
-                                                                     const Transform &transform) :
-    ModelTransformer(in, transform),
+                                                                     const Transform &transform,
+                                                                     CompletionReporter *reporter) :
+    ModelTransformer(in, transform, reporter),
     m_plugin(nullptr),
     m_haveOutputs(false)
 {
@@ -51,8 +54,9 @@ FeatureExtractionModelTransformer::FeatureExtractionModelTransformer(Input in,
 }
 
 FeatureExtractionModelTransformer::FeatureExtractionModelTransformer(Input in,
-                                                                     const Transforms &transforms) :
-    ModelTransformer(in, transforms),
+                                                                     const Transforms &transforms,
+                                                                     CompletionReporter *reporter) :
+    ModelTransformer(in, transforms, reporter),
     m_plugin(nullptr),
     m_haveOutputs(false)
 {
@@ -138,6 +142,7 @@ FeatureExtractionModelTransformer::initialise()
             .arg(m_plugin->getMaxChannelCount())
             .arg(input->getChannelCount());
         SVCERR << m_message << endl;
+        m_plugin = {};
         return false;
     }
 
@@ -166,6 +171,7 @@ FeatureExtractionModelTransformer::initialise()
                 
                 m_message = tr("Failed to initialise feature extraction plugin \"%1\"").arg(pluginId);
                 SVCERR << m_message << endl;
+                m_plugin = {};
                 return false;
 
             } else {
@@ -193,6 +199,7 @@ FeatureExtractionModelTransformer::initialise()
                 
             m_message = tr("Failed to initialise feature extraction plugin \"%1\"").arg(pluginId);
             SVCERR << m_message << endl;
+            m_plugin = {};
             return false;
         }
     } else {
@@ -220,6 +227,7 @@ FeatureExtractionModelTransformer::initialise()
     if (outputs.empty()) {
         m_message = tr("Plugin \"%1\" has no outputs").arg(pluginId);
         SVCERR << m_message << endl;
+        m_plugin = {};
         return false;
     }
 
@@ -243,6 +251,7 @@ FeatureExtractionModelTransformer::initialise()
                 .arg(pluginId)
                 .arg(m_transforms[j].getOutput());
             SVCERR << m_message << endl;
+            m_plugin = {};
             return false;
         }
     }
@@ -263,9 +272,13 @@ FeatureExtractionModelTransformer::initialise()
 void
 FeatureExtractionModelTransformer::deinitialise()
 {
-    SVDEBUG << "FeatureExtractionModelTransformer: deleting plugin for transform in thread "
+#ifdef DEBUG_FEATURE_EXTRACTION_TRANSFORMER_RUN
+    SVDEBUG << "FeatureExtractionModelTransformer::deinitialise: deleting plugin for transform in thread "
             << QThread::currentThreadId() << endl;
+#endif
 
+    abandon();
+    
     try {
         m_plugin = {}; // does not necessarily delete, as it's a
                        // shared_ptr, but in the design case it will
@@ -278,6 +291,11 @@ FeatureExtractionModelTransformer::deinitialise()
     }
 
     m_descriptors.clear();
+
+#ifdef DEBUG_FEATURE_EXTRACTION_TRANSFORMER_RUN
+    SVDEBUG << "FeatureExtractionModelTransformer::deinitialise returning"
+            << endl;
+#endif
 }
 
 void
@@ -686,13 +704,13 @@ FeatureExtractionModelTransformer::run()
             return;
         }
     } catch (const std::exception &e) {
-        abandon();
+        deinitialise();
         m_message = e.what();
         return;
     }
 
     if (m_outputs.empty()) {
-        abandon();
+        deinitialise();
         return;
     }
 
@@ -705,7 +723,7 @@ FeatureExtractionModelTransformer::run()
         { // scope so as to release input shared_ptr before sleeping
             auto input = ModelById::getAs<DenseTimeValueModel>(inputId);
             if (!input || !input->isOK()) {
-                abandon();
+                deinitialise();
                 return;
             }
             ready = input->isReady();
@@ -732,7 +750,7 @@ FeatureExtractionModelTransformer::run()
       // the edges of the process loop
         auto input = ModelById::getAs<DenseTimeValueModel>(inputId);
         if (!input) {
-            abandon();
+            deinitialise();
             return;
         }
 
@@ -783,7 +801,7 @@ FeatureExtractionModelTransformer::run()
                 for (int cch = 0; cch < ch; ++cch) {
                     delete fftModels[cch];
                 }
-                abandon();
+                deinitialise();
                 return;
             }
             fftModels.push_back(model);
@@ -854,6 +872,9 @@ FeatureExtractionModelTransformer::run()
             int completion = int
                 ((((blockFrame - contextStart) / stepSize) * 99) /
                  (contextDuration / stepSize + 1));
+            if (completion > 99) {
+                completion = 99; // 100 reserved for complete
+            }
 
             bool haveAllModels = true;
             if (!ModelById::get(inputId)) {
@@ -955,6 +976,10 @@ FeatureExtractionModelTransformer::run()
                     }
                 }
             }
+        } else {
+#ifdef DEBUG_FEATURE_EXTRACTION_TRANSFORMER_RUN
+            SVDEBUG << "FeatureExtractionModelTransformer::run: Abandoned, exited loop" << endl;
+#endif
         }
     } catch (const std::exception &e) {
         SVCERR << "FeatureExtractionModelTransformer::run: Exception caught: "
@@ -1240,12 +1265,18 @@ FeatureExtractionModelTransformer::setCompletion(int n, int completion)
     SVDEBUG << "FeatureExtractionModelTransformer::setCompletion("
               << completion << ")" << endl;
 #endif
-
+    
     (void)
         (setOutputCompletion<SparseOneDimensionalModel>(n, completion) ||
          setOutputCompletion<SparseTimeValueModel>(n, completion) ||
          setOutputCompletion<NoteModel>(n, completion) ||
          setOutputCompletion<RegionModel>(n, completion) ||
          setOutputCompletion<BasicCompressedDenseThreeDimensionalModel>(n, completion));
+
+    if (m_reporter) {
+        m_reporter->setCompletion(m_outputs[n], completion);
+    }
 }
 
+} // end namespace sv
+
