@@ -32,6 +32,8 @@
 #include <memory>
 #include <atomic>
 
+namespace sv {
+
 /**
  * TransformFactory catalogues both "installed" transforms (based on
  * plugins available to load) and "uninstalled" ones (based on RDF or
@@ -74,6 +76,15 @@ public:
     static void deleteInstance(); // only when exiting
 
 public slots:
+    /**
+     * Instruct the transform factory to query and load only certain
+     * types of transforms. Only transforms of the types specified
+     * will be made available. This must be called before
+     * startPopulatingUninstalledTransforms() to have any effect. The
+     * default is to load all transform types that are found.
+     */
+    void restrictTransformTypes(std::set<Transform::Type>);
+    
     /**
      * Start populating the installed transforms in a background
      * thread. Any call that depends on installed transform
@@ -115,8 +126,13 @@ public:
      * started at all.
      */
     bool havePopulatedUninstalledTransforms();
-    
+
+    /**
+     * Return descriptions for all installed transforms. The returned
+     * transforms are sorted in "display" order by name.
+     */
     TransformList getInstalledTransformDescriptions();
+    
     TransformDescription getInstalledTransformDescription(TransformId id);
     bool haveAnyInstalledTransforms();
 
@@ -268,6 +284,8 @@ signals:
     void uninstalledTransformsPopulated();
     
 protected:
+    std::set<Transform::Type> m_transformTypeRestriction;
+
     typedef std::map<TransformId, TransformDescription> TransformDescriptionMap;
 
     TransformDescriptionMap m_transforms;
@@ -320,4 +338,6 @@ protected:
     virtual ~TransformFactory();
 };
 
+} // end namespace sv
+
 #endif
