@@ -41,6 +41,8 @@
 #include <iostream>
 #include <cmath>
 
+namespace sv {
+
 BoxLayer::BoxLayer() :
     SingleColourLayer(),
     m_editing(false),
@@ -195,19 +197,25 @@ BoxLayer::isLayerScrollable(const LayerGeometryProvider *v) const
     return !v->shouldIlluminateLocalFeatures(this, discard);
 }
 
-bool
-BoxLayer::getValueExtents(double &min, double &max,
-                          bool &logarithmic, QString &unit) const
+Layer::ScaleExtents
+BoxLayer::getVerticalExtents() const
 {
     auto model = ModelById::getAs<BoxModel>(m_model);
-    if (!model) return false;
-    min = model->getValueMinimum();
-    max = model->getValueMaximum();
-    unit = getScaleUnits();
+    if (!model) return NO_VERTICAL_EXTENTS;
+    
+    double min = model->getValueMinimum();
+    double max = model->getValueMaximum();
+    bool logarithmic = (m_verticalScale == LogScale);
+    QString unit = getScaleUnits();
 
-    if (m_verticalScale == LogScale) logarithmic = true;
+    CoordinateScale scale(CoordinateScale::Direction::Vertical,
+                          unit, logarithmic, min, max);
 
-    return true;
+    if (m_verticalScale == AutoAlignScale) {
+        return { Layer::ScaleApplication::Deferring, scale };
+    } else {
+        return { Layer::ScaleApplication::Normal, scale };
+    }
 }
 
 bool
@@ -246,6 +254,8 @@ BoxLayer::getLocalPoint(LayerGeometryProvider *v, int x, int y,
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model || !model->isReady()) return false;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
     sv_frame_t frame = v->getFrameForX(x);
 
     EventVector onPoints = model->getEventsCovering(frame);
@@ -254,7 +264,8 @@ BoxLayer::getLocalPoint(LayerGeometryProvider *v, int x, int y,
     Event bestContaining;
     for (const auto &p: onPoints) {
         auto r = getRange(p);
-        if (y > getYForValue(v, r.first) || y < getYForValue(v, r.second)) {
+        if (y > scale.getCoordForValueRounded(v, r.first) ||
+            y < scale.getCoordForValueRounded(v, r.second)) {
             SVCERR << "inPoints: rejecting " << p.toXmlString() << endl;
             continue;
         }
@@ -286,8 +297,8 @@ BoxLayer::getLocalPoint(LayerGeometryProvider *v, int x, int y,
         for (const auto &p: onPoints) {
             const auto r = getRange(p);
             int distance = std::min
-                (getYForValue(v, r.first) - y,
-                 getYForValue(v, r.second) - y);
+                (scale.getCoordForValueRounded(v, r.first) - y,
+                 scale.getCoordForValueRounded(v, r.second) - y);
             if (distance < 0) distance = -distance;
             if (nearestDistance == -1 || distance < nearestDistance) {
                 nearestDistance = distance;
@@ -300,12 +311,12 @@ BoxLayer::getLocalPoint(LayerGeometryProvider *v, int x, int y,
 }
 
 QString
-BoxLayer::getLabelPreceding(sv_frame_t frame) const
+BoxLayer::getLabelAtOrPreceding(sv_frame_t frame) const
 {
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model) return "";
     EventVector points = model->getEventsStartingWithin
-        (model->getStartFrame(), frame - model->getStartFrame());
+        (model->getStartFrame(), frame - model->getStartFrame() + 1);
     if (!points.empty()) {
         for (auto i = points.rbegin(); i != points.rend(); ++i) {
             if (i->getLabel() != QString()) {
@@ -360,8 +371,9 @@ BoxLayer::getFeatureDescription(LayerGeometryProvider *v,
             .arg(box.getLabel());
     }
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
     pos = QPoint(v->getXForFrame(box.getFrame()),
-                 getYForValue(v, box.getValue()));
+                 scale.getCoordForValueRounded(v, box.getValue()));
     return text;
 }
 
@@ -446,91 +458,7 @@ BoxLayer::getScaleUnits() const
 }
 
 void
-BoxLayer::getScaleExtents(LayerGeometryProvider *v,
-                                       double &min, double &max,
-                                       bool &log) const
-{
-    min = 0.0;
-    max = 0.0;
-    log = false;
-
-    auto model = ModelById::getAs<BoxModel>(m_model);
-    if (!model) return;
-
-    QString queryUnits;
-    queryUnits = getScaleUnits();
-
-    if (m_verticalScale == AutoAlignScale) {
-
-        if (!v->getVisibleExtentsForUnit(queryUnits, min, max, log)) {
-
-            min = model->getValueMinimum();
-            max = model->getValueMaximum();
-
-//            cerr << "BoxLayer[" << this << "]::getScaleExtents: min = " << min << ", max = " << max << ", log = " << log << endl;
-
-        } else if (log) {
-
-            LogRange::mapRange(min, max);
-
-//            cerr << "BoxLayer[" << this << "]::getScaleExtents: min = " << min << ", max = " << max << ", log = " << log << endl;
-
-        }
-
-    } else {
-
-        min = model->getValueMinimum();
-        max = model->getValueMaximum();
-
-        if (m_verticalScale == LogScale) {
-            LogRange::mapRange(min, max);
-            log = true;
-        }
-    }
-
-    if (max == min) max = min + 1.0;
-}
-
-int
-BoxLayer::getYForValue(LayerGeometryProvider *v, double val) const
-{
-    double min = 0.0, max = 0.0;
-    bool logarithmic = false;
-    int h = v->getPaintHeight();
-
-    getScaleExtents(v, min, max, logarithmic);
-
-//    cerr << "BoxLayer[" << this << "]::getYForValue(" << val << "): min = " << min << ", max = " << max << ", log = " << logarithmic << endl;
-//    cerr << "h = " << h << ", margin = " << margin << endl;
-
-    if (logarithmic) {
-        val = LogRange::map(val);
-    }
-
-    return int(h - ((val - min) * h) / (max - min));
-}
-
-double
-BoxLayer::getValueForY(LayerGeometryProvider *v, int y) const
-{
-    double min = 0.0, max = 0.0;
-    bool logarithmic = false;
-    int h = v->getPaintHeight();
-
-    getScaleExtents(v, min, max, logarithmic);
-
-    double val = min + (double(h - y) * double(max - min)) / h;
-
-    if (logarithmic) {
-        val = pow(10.0, val);
-    }
-
-    return val;
-}
-
-void
-BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
-                             QRect rect) const
+BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) const
 {
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model || !model->isOK()) return;
@@ -538,6 +466,8 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
     sv_samplerate_t sampleRate = model->getSampleRate();
     if (!sampleRate) return;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
 //    Profiler profiler("BoxLayer::paint", true);
 
     int x0 = rect.left() - 40;
@@ -581,8 +511,8 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
 
         int x = v->getXForFrame(p.getFrame());
         int w = v->getXForFrame(p.getFrame() + p.getDuration()) - x;
-        int y = getYForValue(v, r.first);
-        int h = getYForValue(v, r.second) - y;
+        int y = scale.getCoordForValueRounded(v, r.first);
+        int h = scale.getCoordForValueRounded(v, r.second) - y;
         int ex = x + w;
         int gap = v->scalePixelSize(2);
 
@@ -604,11 +534,6 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
             (m_editing && m_editingPoint == p)) {
 
             paint.setPen(QPen(getBaseQColor(), v->scalePixelSize(2)));
-                
-            // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-            // replacement (horizontalAdvance) was only added in Qt 5.11
-            // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 
             if (abs(h) > 2 * fm.height()) {
             
@@ -622,13 +547,13 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
 
                 PaintAssistant::drawVisibleText
                     (v, paint, 
-                     x - fm.width(y0label) - gap,
+                     x - fm.horizontalAdvance(y0label) - gap,
                      y - fm.descent(), 
                      y0label, PaintAssistant::OutlinedText);
 
                 PaintAssistant::drawVisibleText
                     (v, paint, 
-                     x - fm.width(y1label) - gap,
+                     x - fm.horizontalAdvance(y1label) - gap,
                      y + h + fm.ascent(), 
                      y1label, PaintAssistant::OutlinedText);
 
@@ -642,7 +567,7 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
 
                 PaintAssistant::drawVisibleText
                     (v, paint, 
-                     x - fm.width(ylabel) - gap,
+                     x - fm.horizontalAdvance(ylabel) - gap,
                      y - fm.descent(), 
                      ylabel, PaintAssistant::OutlinedText);
             }
@@ -658,11 +583,11 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
                 (v, paint, x, y + fm.ascent() + gap,
                  t0label, PaintAssistant::OutlinedText);
 
-            if (w > fm.width(t0label) + fm.width(t1label) + gap * 3) {
+            if (w > fm.horizontalAdvance(t0label) + fm.horizontalAdvance(t1label) + gap * 3) {
 
                 PaintAssistant::drawVisibleText
                     (v, paint,
-                     x + w - fm.width(t1label),
+                     x + w - fm.horizontalAdvance(t1label),
                      y + fm.ascent() + gap,
                      t1label, PaintAssistant::OutlinedText);
 
@@ -670,7 +595,7 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
 
                 PaintAssistant::drawVisibleText
                     (v, paint,
-                     x + w - fm.width(t1label),
+                     x + w - fm.horizontalAdvance(t1label),
                      y + fm.ascent() + fm.height() + gap,
                      t1label, PaintAssistant::OutlinedText);
             }                
@@ -693,9 +618,9 @@ BoxLayer::paint(LayerGeometryProvider *v, QPainter &paint,
         
         int x = v->getXForFrame(p.getFrame());
         int w = v->getXForFrame(p.getFrame() + p.getDuration()) - x;
-        int y = getYForValue(v, p.getValue());
+        int y = scale.getCoordForValueRounded(v, p.getValue());
 
-        int labelWidth = fm.width(label);
+        int labelWidth = fm.horizontalAdvance(label);
 
         int gap = v->scalePixelSize(2);
 
@@ -738,18 +663,16 @@ BoxLayer::paintVerticalScale(LayerGeometryProvider *v,
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model || model->isEmpty()) return;
 
-    QString unit;
-    double min, max;
-    bool logarithmic;
+    // We are only asked to draw if we are the reference scale, so
+    // don't use getEffectiveVerticalExtentsForLayer here
+    CoordinateScale scale = getVerticalExtents().second;
 
     int w = getVerticalScaleWidth(v, false, paint);
 
-    getScaleExtents(v, min, max, logarithmic);
-
-    if (logarithmic) {
-        LogNumericalScale().paintVertical(v, this, paint, 0, min, max);
+    if (scale.isLogarithmic()) {
+        LogNumericalScale().paintVertical(v, scale, paint, 0);
     } else {
-        LinearNumericalScale().paintVertical(v, this, paint, 0, min, max);
+        LinearNumericalScale().paintVertical(v, scale, paint, 0);
     }
         
     if (getScaleUnits() != "") {
@@ -768,11 +691,13 @@ BoxLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double value = getValueForY(v, e->y());
+    double value = scale.getValueForCoord(v, e->position().y());
 
     m_editingPoint = Event(frame, float(value), 0, "");
     m_originalPoint = m_editingPoint;
@@ -791,7 +716,9 @@ BoxLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t dragFrame = v->getFrameForX(e->x());
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    sv_frame_t dragFrame = v->getFrameForX(e->position().x());
     if (dragFrame < 0) dragFrame = 0;
     dragFrame = dragFrame / model->getResolution() * model->getResolution();
 
@@ -804,7 +731,7 @@ BoxLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
         eventDuration = model->getResolution();
     }
 
-    double dragValue = getValueForY(v, e->y());
+    double dragValue = scale.getValueForCoord(v, e->position().y());
 
     double eventValue = m_originalPoint.getValue();
     double eventFreqDiff = dragValue - eventValue;
@@ -838,7 +765,7 @@ BoxLayer::eraseStart(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model) return;
 
-    if (!getLocalPoint(v, e->x(), e->y(), m_editingPoint)) return;
+    if (!getLocalPoint(v, e->position().x(), e->position().y(), m_editingPoint)) return;
 
     if (m_editingCommand) {
         finish(m_editingCommand);
@@ -862,7 +789,7 @@ BoxLayer::eraseEnd(LayerGeometryProvider *v, QMouseEvent *e)
     m_editing = false;
 
     Event p(0);
-    if (!getLocalPoint(v, e->x(), e->y(), p)) return;
+    if (!getLocalPoint(v, e->position().x(), e->position().y(), p)) return;
     if (p.getFrame() != m_editingPoint.getFrame() ||
         p.getValue() != m_editingPoint.getValue()) return;
 
@@ -882,12 +809,14 @@ BoxLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model) return;
 
-    if (!getLocalPoint(v, e->x(), e->y(), m_editingPoint)) {
+    if (!getLocalPoint(v, e->position().x(), e->position().y(), m_editingPoint)) {
         return;
     }
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
     m_dragPointX = v->getXForFrame(m_editingPoint.getFrame());
-    m_dragPointY = getYForValue(v, m_editingPoint.getValue());
+    m_dragPointY = scale.getCoordForValueRounded(v, m_editingPoint.getValue());
 
     m_originalPoint = m_editingPoint;
 
@@ -897,8 +826,8 @@ BoxLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
     }
 
     m_editing = true;
-    m_dragStartX = e->x();
-    m_dragStartY = e->y();
+    m_dragStartX = e->position().x();
+    m_dragStartY = e->position().y();
 }
 
 void
@@ -907,8 +836,10 @@ BoxLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<BoxModel>(m_model);
     if (!model || !m_editing) return;
 
-    int xdist = e->x() - m_dragStartX;
-    int ydist = e->y() - m_dragStartY;
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    int xdist = e->position().x() - m_dragStartX;
+    int ydist = e->position().y() - m_dragStartY;
     int newx = m_dragPointX + xdist;
     int newy = m_dragPointY + ydist;
 
@@ -916,7 +847,7 @@ BoxLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double value = getValueForY(v, newy);
+    double value = scale.getValueForCoord(v, newy);
 
     if (!m_editingCommand) {
         m_editingCommand = new ChangeEventsCommand
@@ -966,7 +897,9 @@ BoxLayer::editOpen(LayerGeometryProvider *v, QMouseEvent *e)
     if (!model) return false;
 
     Event region(0);
-    if (!getLocalPoint(v, e->x(), e->y(), region)) return false;
+    if (!getLocalPoint(v, e->position().x(), e->position().y(), region)) {
+        return false;
+    }
 
     ItemEditDialog::LabelOptions labelOptions;
     labelOptions.valueLabel = tr("Minimum Value");
@@ -1193,7 +1126,7 @@ BoxLayer::toXml(QTextStream &stream,
 }
 
 void
-BoxLayer::setProperties(const QXmlAttributes &attributes)
+BoxLayer::setProperties(const LayerAttributes &attributes)
 {
     SingleColourLayer::setProperties(attributes);
 
@@ -1204,3 +1137,5 @@ BoxLayer::setProperties(const QXmlAttributes &attributes)
 }
 
 
+} // end namespace sv
+
