@@ -38,6 +38,8 @@
 
 #include <cassert>
 
+namespace sv {
+
 using std::vector;
 
 //#define DEBUG_COLOUR_3D_PLOT_LAYER_PAINT 1
@@ -53,7 +55,7 @@ Colour3DPlotLayer::Colour3DPlotLayer() :
     m_normalization(ColumnNormalization::None),
     m_normalizeVisibleArea(false),
     m_invertVertical(false),
-    m_opaque(false),
+    m_opaque(true),
     m_smooth(false),
     m_peakResolution(256),
     m_miny(0),
@@ -158,6 +160,8 @@ Colour3DPlotLayer::setModel(ModelId modelId)
         throw std::logic_error("Not a DenseThreeDimensionalModel");
     }
     
+    ModelId oldModelId = m_model;
+    
     if (m_model == modelId) return;
     m_model = modelId;
 
@@ -182,6 +186,7 @@ Colour3DPlotLayer::setModel(ModelId modelId)
     invalidatePeakCache();
 
     emit modelReplaced();
+    emit sliceableModelReplaced(oldModelId, m_model);
 }
 
 void
@@ -310,7 +315,7 @@ Colour3DPlotLayer::getPropertyLabel(const PropertyName &name) const
     if (name == "Normalization") return tr("Normalization");
     if (name == "Invert Vertical Scale") return tr("Invert Vertical Scale");
     if (name == "Gain") return tr("Gain");
-    if (name == "Opaque") return tr("Always Opaque");
+    if (name == "Opaque") return tr("Opaque");
     if (name == "Smooth") return tr("Smooth");
     if (name == "Bin Scale") return tr("Bin Scale");
     return "";
@@ -423,7 +428,7 @@ Colour3DPlotLayer::getPropertyRangeAndValue(const PropertyName &name,
         
         *min = 0;
         *max = 1;
-        *deflt = 0;
+        *deflt = 1;
         val = (m_opaque ? 1 : 0);
         
     } else if (name == "Smooth") {
@@ -706,20 +711,30 @@ Colour3DPlotLayer::getCompletion(LayerGeometryProvider *) const
     else return 0;
 }
 
-bool
-Colour3DPlotLayer::getValueExtents(double &min, double &max,
-                                   bool &logarithmic, QString &unit) const
+Layer::ScaleExtents
+Colour3DPlotLayer::getVerticalExtents() const
 {
     auto model = ModelById::getAs<DenseThreeDimensionalModel>(m_model);
-    if (!model) return false;
+    if (!model) return NO_VERTICAL_EXTENTS;
 
-    min = 0;
-    max = double(model->getHeight());
+    int valueMax = model->getHeight();
+    
+    CoordinateScale scale(CoordinateScale::Direction::Vertical,
+                          m_binScale == BinScale::Log, 0, valueMax);
+    
+    int min = m_miny;
+    int max = m_maxy;
+    if (max <= min) {
+        min = 0;
+        max = valueMax;
+    }
+    if (min < 0) min = 0;
+    if (max > valueMax) max = valueMax;
 
-    logarithmic = (m_binScale == BinScale::Log);
-    unit = "";
-
-    return true;
+    return {
+        Layer::ScaleApplication::Normal,
+        scale.withDisplayExtents(min, max)
+    };
 }
 
 bool
@@ -754,13 +769,6 @@ Colour3DPlotLayer::setDisplayExtents(double min, double max)
     return true;
 }
 
-bool
-Colour3DPlotLayer::getYScaleValue(const LayerGeometryProvider *, int,
-                                  double &, QString &) const
-{
-    return false;//!!!
-}
-
 int
 Colour3DPlotLayer::getVerticalZoomSteps(int &defaultStep) const
 {
@@ -940,13 +948,8 @@ Colour3DPlotLayer::getVerticalScaleWidth(LayerGeometryProvider *, bool, QPainter
     auto model = ModelById::getAs<DenseThreeDimensionalModel>(m_model);
     if (!model) return 0;
 
-    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-    // replacement (horizontalAdvance) was only added in Qt 5.11 which
-    // is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
     QString sampleText = QString("[%1]").arg(model->getHeight());
-    int tw = paint.fontMetrics().width(sampleText);
+    int tw = paint.fontMetrics().horizontalAdvance(sampleText);
     bool another = false;
 
     for (int i = 0; i < model->getHeight(); ++i) {
@@ -956,7 +959,7 @@ Colour3DPlotLayer::getVerticalScaleWidth(LayerGeometryProvider *, bool, QPainter
         }
     }
     if (another) {
-        tw = std::max(tw, paint.fontMetrics().width(sampleText));
+        tw = std::max(tw, paint.fontMetrics().horizontalAdvance(sampleText));
     }
 
     return tw + 13 + getColourScaleWidth(paint);
@@ -1002,7 +1005,7 @@ Colour3DPlotLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &
             paint.setFont(font);
         }
 
-        int msw = paint.fontMetrics().width(maxstr);
+        int msw = paint.fontMetrics().horizontalAdvance(maxstr);
 
         QTransform m;
         m.translate(cw - 6, ch + 10);
@@ -1148,8 +1151,11 @@ Colour3DPlotLayer::getRenderer(const LayerGeometryProvider *v) const
         Colour3DPlotRenderer::Parameters params;
         params.colourScale = ColourScale(cparams);
         params.normalization = m_normalization;
-        params.binScale = m_binScale;
-        params.alwaysOpaque = m_opaque;
+        params.frequencyMapping = FrequencyMapping::Linear;
+        if (m_binScale == BinScale::Log) {
+            params.frequencyMapping = FrequencyMapping::Log;
+        }
+        params.opaque = m_opaque;
         params.invertVertical = m_invertVertical;
         params.interpolate = m_smooth;
 
@@ -1329,7 +1335,7 @@ Colour3DPlotLayer::toXml(QTextStream &stream,
 }
 
 void
-Colour3DPlotLayer::setProperties(const QXmlAttributes &attributes)
+Colour3DPlotLayer::setProperties(const LayerAttributes &attributes)
 {
     bool ok = false, alsoOk = false;
 
@@ -1417,3 +1423,5 @@ Colour3DPlotLayer::setProperties(const QXmlAttributes &attributes)
     //!!! with Tony v1 and v2 and their save files
 }
 
+} // end namespace sv
+
