@@ -29,6 +29,8 @@
 #include <QPainter>
 #include <QImage>
 
+namespace sv {
+
 class LayerGeometryProvider;
 class VerticalBinLayer;
 class RenderTimer;
@@ -41,7 +43,7 @@ enum class BinDisplay {
     PeakFrequencies
 };
 
-enum class BinScale {
+enum class BinScale { // Not actually used by this class, but by related ones
     Linear,
     Log
 };
@@ -64,8 +66,8 @@ public:
             colourScale(ColourScale::Parameters()),
             normalization(ColumnNormalization::None),
             binDisplay(BinDisplay::AllBins),
-            binScale(BinScale::Linear),
-            alwaysOpaque(false),
+            frequencyMapping(FrequencyMapping::Linear),
+            opaque(false),
             interpolate(false),
             invertVertical(false),
             showDerivative(false),
@@ -83,14 +85,18 @@ public:
         /** Selection of bins to display. */
         BinDisplay binDisplay;
 
-        /** Scale for vertical bin spacing (linear or logarithmic). */
-        BinScale binScale;
+        /** Adjustment for vertical bin spacing (linear, logarithmic,
+         *  mel). Only used for peak-frequency rendering, which
+         *  queries frequencies against the LayerGeometryProvider
+         *  directly. Other modes perform bin mapping using
+         *  verticalBinLayer->getBinForY. */
+        FrequencyMapping frequencyMapping;
 
-        /** Whether cells should always be opaque. If false, then
-         *  large cells (when zoomed in a long way) will be rendered
-         *  translucent in order not to obscure anything in a layer
-         *  beneath. */
-        bool alwaysOpaque;
+        /** Whether cells should be opaque. If not, some transparency
+         *  will be applied so that layers underneath are (at least
+         *  somewhat) visible.
+         */
+        bool opaque;
 
         /** Whether to apply smoothing when rendering cells at more
          *  than one pixel per cell.  !!! todo: decide about separating
@@ -116,12 +122,7 @@ public:
         int colourRotation;
     };
     
-    Colour3DPlotRenderer(Sources sources, Parameters parameters) :
-        m_sources(sources),
-        m_params(parameters),
-        m_secondsPerXPixel(0.0),
-        m_secondsPerXPixelValid(false)
-    { }
+    Colour3DPlotRenderer(Sources sources, Parameters parameters);
 
     struct RenderResult {
         /**
@@ -246,8 +247,9 @@ public:
     QRect findSimilarRegionExtents(QPoint point) const;
     
 private:
-    Sources m_sources;
-    Parameters m_params;
+    const Sources m_sources;
+    const Parameters m_params;
+    const std::vector<QRgb> m_colourmap;
 
     // Draw buffer is the target of each partial repaint. It is always
     // at view height (not model height) and is cleared and repainted
@@ -338,5 +340,7 @@ private:
     void updateTimings(const RenderTimer &timer, int xPixelCount);
 };
 
+} // end namespace sv
+
 #endif
 
