@@ -19,6 +19,8 @@
 #include <QString>
 #include <QColor>
 
+namespace sv {
+
 class LayerGeometryProvider;
 
 /**
@@ -36,5 +38,7 @@ public:
     virtual QColor getColourForValue(LayerGeometryProvider *v, double value) const = 0;
 };
 
+} // end namespace sv
+
 #endif
 
