@@ -46,6 +46,8 @@
 #include <iostream>
 #include <cmath>
 #include <utility>
+namespace sv {
+
 #include <limits> // GF: included to compile std::numerical_limits on linux
 #include <vector>
 
@@ -230,25 +232,51 @@ FlexiNoteLayer::getCompletion(LayerGeometryProvider *) const
     else return 0;
 }
 
-bool
-FlexiNoteLayer::getValueExtents(double &min, double &max,
-                                bool &logarithmic, QString &unit) const
+Layer::ScaleExtents
+FlexiNoteLayer::getVerticalExtents() const
 {
     auto model = ModelById::getAs<NoteModel>(m_model);
-    if (!model) return false;
-    min = model->getValueMinimum();
-    max = model->getValueMaximum();
+    if (!model) return NO_VERTICAL_EXTENTS;
+    
+    double min = model->getValueMinimum();
+    double max = model->getValueMaximum();
 
+    bool logarithmic = (m_verticalScale == MIDIRangeScale ||
+                        m_verticalScale == LogScale);
+    
+    QString unit;
+    
     if (shouldConvertMIDIToHz()) {
         unit = "Hz";
         min = Pitch::getFrequencyForPitch(int(lrint(min)));
         max = Pitch::getFrequencyForPitch(int(lrint(max + 1)));
-    } else unit = getScaleUnits();
+    } else {
+        unit = getScaleUnits();
+    }
+    
+    CoordinateScale scale(CoordinateScale::Direction::Vertical,
+                          unit, logarithmic, min, max);
 
-    if (m_verticalScale == MIDIRangeScale ||
-        m_verticalScale == LogScale) logarithmic = true;
-
-    return true;
+    
+    if (m_verticalScale == MIDIRangeScale) {
+        min = Pitch::getFrequencyForPitch(0);
+        max = Pitch::getFrequencyForPitch(127);
+        scale = scale.withDisplayExtents(min, max);
+    } else if (m_scaleMinimum != m_scaleMaximum) {
+        min = m_scaleMinimum;
+        max = m_scaleMaximum;
+        if (shouldConvertMIDIToHz()) {
+            min = Pitch::getFrequencyForPitch(int(lrint(min)));
+            max = Pitch::getFrequencyForPitch(int(lrint(max + 1)));
+        }
+        scale = scale.withDisplayExtents(min, max);
+    }
+        
+    if (m_verticalScale == AutoAlignScale) {
+        return { Layer::ScaleApplication::Deferring, scale };
+    } else {
+        return { Layer::ScaleApplication::Normal, scale };
+    }
 }
 
 bool
@@ -280,7 +308,7 @@ FlexiNoteLayer::getDisplayExtents(double &min, double &max) const
     }
 
 #ifdef DEBUG_NOTE_LAYER
-    cerr << "NoteLayer::getDisplayExtents: min = " << min << ", max = " << max << " (m_scaleMinimum = " << m_scaleMinimum << ", m_scaleMaximum = " << m_scaleMaximum << ")" << endl;
+    SVCERR << "NoteLayer::getDisplayExtents: min = " << min << ", max = " << max << " (m_scaleMinimum = " << m_scaleMinimum << ", m_scaleMaximum = " << m_scaleMaximum << ")" << endl;
 #endif
 
     return true;
@@ -304,7 +332,7 @@ FlexiNoteLayer::setDisplayExtents(double min, double max)
     m_scaleMaximum = max;
 
 #ifdef DEBUG_NOTE_LAYER
-    cerr << "FlexiNoteLayer::setDisplayExtents: min = " << min << ", max = " << max << endl;
+    SVCERR << "FlexiNoteLayer::setDisplayExtents: min = " << min << ", max = " << max << endl;
 #endif
     
     emit layerParametersChanged();
@@ -354,26 +382,26 @@ FlexiNoteLayer::setVerticalZoomStep(int step)
     RangeMapper *mapper = getNewVerticalZoomRangeMapper();
     if (!mapper) return;
     
-    double min, max;
-    bool logarithmic;
-    QString unit;
-    getValueExtents(min, max, logarithmic, unit);
+    CoordinateScale scale = getVerticalExtents().second;
     
-    double dmin, dmax;
-    getDisplayExtents(dmin, dmax);
+    double min = scale.getValueMinimum();
+    double max = scale.getValueMaximum();
+    
+    double dmin = scale.getDisplayMinimum();
+    double dmax = scale.getDisplayMaximum();
 
     double newdist = mapper->getValueForPosition(100 - step);
 
     double newmin, newmax;
 
-    if (logarithmic) {
+    if (scale.isLogarithmic()) {
 
         // see SpectrogramLayer::setVerticalZoomStep
 
         newmax = (newdist + sqrt(newdist*newdist + 4*dmin*dmax)) / 2;
         newmin = newmax - newdist;
 
-//        cerr << "newmin = " << newmin << ", newmax = " << newmax << endl;
+//        SVCERR << "newmin = " << newmin << ", newmax = " << newmax << endl;
 
     } else {
         double dmid = (dmax + dmin) / 2;
@@ -390,7 +418,7 @@ FlexiNoteLayer::setVerticalZoomStep(int step)
     }
     
 #ifdef DEBUG_NOTE_LAYER
-    cerr << "FlexiNoteLayer::setVerticalZoomStep: " << step << ": " << newmin << " -> " << newmax << " (range " << newdist << ")" << endl;
+    SVCERR << "FlexiNoteLayer::setVerticalZoomStep: " << step << ": " << newmin << " -> " << newmax << " (range " << newdist << ")" << endl;
 #endif
 
     setDisplayExtents(newmin, newmax);
@@ -404,14 +432,15 @@ FlexiNoteLayer::getNewVerticalZoomRangeMapper() const
     
     RangeMapper *mapper;
 
-    double min, max;
-    bool logarithmic;
-    QString unit;
-    getValueExtents(min, max, logarithmic, unit);
+    CoordinateScale scale = getVerticalExtents().second;
+    
+    double min = scale.getValueMinimum();
+    double max = scale.getValueMaximum();
+    QString unit = scale.getUnit();
 
     if (min == max) return nullptr;
     
-    if (logarithmic) {
+    if (scale.isLogarithmic()) {
         mapper = new LogRangeMapper(0, 100, min, max, unit);
     } else {
         mapper = new LinearRangeMapper(0, 100, min, max, unit);
@@ -455,9 +484,11 @@ FlexiNoteLayer::getPointToDrag(LayerGeometryProvider *v, int x, int y, Event &po
     EventVector onPoints = model->getEventsCovering(frame);
     if (onPoints.empty()) return false;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+    
     int nearestDistance = -1;
     for (const auto &p: onPoints) {
-        int distance = getYForValue(v, p.getValue()) - y;
+        int distance = scale.getCoordForValueRounded(v, p.getValue()) - y;
         if (distance < 0) distance = -distance;
         if (nearestDistance == -1 || distance < nearestDistance) {
             nearestDistance = distance;
@@ -480,9 +511,11 @@ FlexiNoteLayer::getNoteToEdit(LayerGeometryProvider *v, int x, int y, Event &poi
     EventVector onPoints = model->getEventsCovering(frame);
     if (onPoints.empty()) return false;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+    
     int nearestDistance = -1;
     for (const auto &p: onPoints) {
-        int distance = getYForValue(v, p.getValue()) - y;
+        int distance = scale.getCoordForValueRounded(v, p.getValue()) - y;
         if (distance < 0) distance = -distance;
         if (nearestDistance == -1 || distance < nearestDistance) {
             nearestDistance = distance;
@@ -514,13 +547,16 @@ FlexiNoteLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) con
     Event note(0);
     EventVector::iterator i;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+    
     for (i = points.begin(); i != points.end(); ++i) {
 
-        int y = getYForValue(v, i->getValue());
+        int y = scale.getCoordForValueRounded(v, i->getValue());
         int h = NOTE_HEIGHT; // GF: larger notes
 
         if (model->getValueQuantization() != 0.0) {
-            h = y - getYForValue
+            h = y -
+                scale.getCoordForValueRounded
                 (v, i->getValue() + model->getValueQuantization());
             if (h < NOTE_HEIGHT) h = NOTE_HEIGHT;
         }
@@ -579,7 +615,7 @@ FlexiNoteLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) con
     }
 
     pos = QPoint(v->getXForFrame(note.getFrame()),
-                 getYForValue(v, note.getValue()));
+                 scale.getCoordForValueRounded(v, note.getValue()));
     return text;
 }
 
@@ -656,127 +692,12 @@ FlexiNoteLayer::snapToFeatureFrame(LayerGeometryProvider *v, sv_frame_t &frame,
         }
     }
 
-    cerr << "snapToFeatureFrame: frame " << frame << " -> snapped " << snapped << ", found = " << found << endl;
+    SVCERR << "snapToFeatureFrame: frame " << frame << " -> snapped " << snapped << ", found = " << found << endl;
 
     frame = snapped;
     return found;
 }
 
-void
-FlexiNoteLayer::getScaleExtents(LayerGeometryProvider *v, double &min, double &max, bool &log) const
-{
-    min = 0.0;
-    max = 0.0;
-    log = false;
-
-    QString queryUnits;
-    if (shouldConvertMIDIToHz()) queryUnits = "Hz";
-    else queryUnits = getScaleUnits();
-
-    if (shouldAutoAlign()) {
-
-        if (!v->getVisibleExtentsForUnit(queryUnits, min, max, log)) {
-
-            auto model = ModelById::getAs<NoteModel>(m_model);
-            min = model->getValueMinimum();
-            max = model->getValueMaximum();
-
-            if (shouldConvertMIDIToHz()) {
-                min = Pitch::getFrequencyForPitch(int(lrint(min)));
-                max = Pitch::getFrequencyForPitch(int(lrint(max + 1)));
-            }
-
-#ifdef DEBUG_NOTE_LAYER
-            cerr << "FlexiNoteLayer[" << this << "]::getScaleExtents: min = " << min << ", max = " << max << ", log = " << log << endl;
-#endif
-
-        } else if (log) {
-
-            LogRange::mapRange(min, max);
-
-#ifdef DEBUG_NOTE_LAYER
-            cerr << "FlexiNoteLayer[" << this << "]::getScaleExtents: min = " << min << ", max = " << max << ", log = " << log << endl;
-#endif
-        }
-
-    } else {
-
-        getDisplayExtents(min, max);
-
-        if (m_verticalScale == MIDIRangeScale) {
-            min = Pitch::getFrequencyForPitch(0);
-            max = Pitch::getFrequencyForPitch(70);
-        } else if (shouldConvertMIDIToHz()) {
-            min = Pitch::getFrequencyForPitch(int(lrint(min)));
-            max = Pitch::getFrequencyForPitch(int(lrint(max + 1)));
-        }
-
-        if (m_verticalScale == LogScale || m_verticalScale == MIDIRangeScale) {
-            LogRange::mapRange(min, max);
-            log = true;
-        }
-    }
-
-    if (max == min) max = min + 1.0;
-}
-
-int
-FlexiNoteLayer::getYForValue(LayerGeometryProvider *v, double val) const
-{
-    double min = 0.0, max = 0.0;
-    bool logarithmic = false;
-    int h = v->getPaintHeight();
-
-    getScaleExtents(v, min, max, logarithmic);
-
-#ifdef DEBUG_NOTE_LAYER
-    cerr << "FlexiNoteLayer[" << this << "]::getYForValue(" << val << "): min = " << min << ", max = " << max << ", log = " << logarithmic << endl;
-#endif
-
-    if (shouldConvertMIDIToHz()) {
-        val = Pitch::getFrequencyForPitch(int(lrint(val)),
-                                          int(lrint((val - floor(val)) * 100.0)));
-#ifdef DEBUG_NOTE_LAYER
-        cerr << "shouldConvertMIDIToHz true, val now = " << val << endl;
-#endif
-    }
-
-    if (logarithmic) {
-        val = LogRange::map(val);
-#ifdef DEBUG_NOTE_LAYER
-        cerr << "logarithmic true, val now = " << val << endl;
-#endif
-    }
-
-    int y = int(h - ((val - min) * h) / (max - min)) - 1;
-#ifdef DEBUG_NOTE_LAYER
-    cerr << "y = " << y << endl;
-#endif
-    return y;
-}
-
-double
-FlexiNoteLayer::getValueForY(LayerGeometryProvider *v, int y) const
-{
-    double min = 0.0, max = 0.0;
-    bool logarithmic = false;
-    int h = v->getPaintHeight();
-
-    getScaleExtents(v, min, max, logarithmic);
-
-    double val = min + (double(h - y) * double(max - min)) / h;
-
-    if (logarithmic) {
-        val = pow(10.f, val);
-    }
-
-    if (shouldConvertMIDIToHz()) {
-        val = Pitch::getPitchForFrequency(val);
-    }
-
-    return val;
-}
-
 bool
 FlexiNoteLayer::shouldAutoAlign() const
 {
@@ -826,6 +747,8 @@ FlexiNoteLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) con
     paint.setRenderHint(QPainter::Antialiasing, false);
 
     int noteNumber = -1;
+    
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
 
     for (EventVector::const_iterator i = points.begin();
          i != points.end(); ++i) {
@@ -839,12 +762,13 @@ FlexiNoteLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) con
         }
 
         int x = v->getXForFrame(p.getFrame());
-        int y = getYForValue(v, p.getValue());
+        int y = scale.getCoordForValueRounded(v, p.getValue());
         int w = v->getXForFrame(p.getFrame() + p.getDuration()) - x;
         int h = NOTE_HEIGHT; //GF: larger notes
     
         if (model->getValueQuantization() != 0.0) {
-            h = y - getYForValue(v, p.getValue() + model->getValueQuantization());
+            h = y - scale.getCoordForValueRounded
+                (v, p.getValue() + model->getValueQuantization());
             if (h < NOTE_HEIGHT) h = NOTE_HEIGHT; //GF: larger notes
         }
 
@@ -919,26 +843,22 @@ FlexiNoteLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pai
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || model->isEmpty()) return;
 
-    QString unit;
-    double min, max;
-    bool logarithmic;
+    // We are only asked to draw if we are the reference scale, so
+    // don't use getEffectiveVerticalExtentsForLayer here
+    CoordinateScale scale = getVerticalExtents().second;
 
     int w = getVerticalScaleWidth(v, false, paint);
     int h = v->getPaintHeight();
 
-    getScaleExtents(v, min, max, logarithmic);
-
-    if (logarithmic) {
-        LogNumericalScale().paintVertical(v, this, paint, 0, min, max);
+    if (scale.isLogarithmic()) {
+        LogNumericalScale().paintVertical(v, scale, paint, 0);
     } else {
-        LinearNumericalScale().paintVertical(v, this, paint, 0, min, max);
+        LinearNumericalScale().paintVertical(v, scale, paint, 0);
     }
     
-    if (logarithmic && (getScaleUnits() == "Hz")) {
+    if (!scale.isLinear() && scale.getUnit() == "Hz") {
         PianoScale().paintPianoVertical
-            (v, paint, QRect(w - 10, 0, 10, h), 
-             LogRange::unmap(min), 
-             LogRange::unmap(max));
+            (v, paint, QRect(w - 10, 0, 10, h), scale);
         paint.drawLine(w, 0, w, h);
     }
         
@@ -955,16 +875,18 @@ FlexiNoteLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pai
 void
 FlexiNoteLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "FlexiNoteLayer::drawStart(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "FlexiNoteLayer::drawStart(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double value = getValueForY(v, e->y());
+    double value = scale.getValueForCoord(v, e->position().y());
 
     m_editingPoint = Event(frame, float(value), 0, 0.8f, tr("New Point"));
     m_originalPoint = m_editingPoint;
@@ -979,16 +901,18 @@ FlexiNoteLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 void
 FlexiNoteLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "FlexiNoteLayer::drawDrag(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "FlexiNoteLayer::drawDrag(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double newValue = getValueForY(v, e->y());
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    double newValue = scale.getValueForCoord(v, e->position().y());
 
     sv_frame_t newFrame = m_editingPoint.getFrame();
     sv_frame_t newDuration = frame - newFrame;
@@ -1010,7 +934,7 @@ FlexiNoteLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 void
 FlexiNoteLayer::drawEnd(LayerGeometryProvider *, QMouseEvent *)
 {
-//    SVDEBUG << "FlexiNoteLayer::drawEnd(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "FlexiNoteLayer::drawEnd(" << e->position().x() << "," << e->position().y() << ")" << endl;
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
     finish(m_editingCommand);
@@ -1024,7 +948,7 @@ FlexiNoteLayer::eraseStart(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model) return;
 
-    if (!getPointToDrag(v, e->x(), e->y(), m_editingPoint)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), m_editingPoint)) return;
 
     if (m_editingCommand) {
         finish(m_editingCommand);
@@ -1046,7 +970,7 @@ FlexiNoteLayer::eraseEnd(LayerGeometryProvider *v, QMouseEvent *e)
     m_editing = false;
 
     Event p(0);
-    if (!getPointToDrag(v, e->x(), e->y(), p)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), p)) return;
     if (p.getFrame() != m_editingPoint.getFrame() ||
         p.getValue() != m_editingPoint.getValue()) return;
 
@@ -1060,15 +984,17 @@ FlexiNoteLayer::eraseEnd(LayerGeometryProvider *v, QMouseEvent *e)
 void
 FlexiNoteLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "FlexiNoteLayer::editStart(" << e->x() << "," << e->y() << ")" << endl;
-    std::cerr << "FlexiNoteLayer::editStart(" << e->x() << "," << e->y() << ")" << std::endl;
+//    SVDEBUG << "FlexiNoteLayer::editStart(" << e->position().x() << "," << e->position().y() << ")" << endl;
+    std::cerr << "FlexiNoteLayer::editStart(" << e->position().x() << "," << e->position().y() << ")" << std::endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model) return;
 
-    if (!getPointToDrag(v, e->x(), e->y(), m_editingPoint)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), m_editingPoint)) return;
     m_originalPoint = m_editingPoint;
     
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
     if (m_editMode == RightBoundary) {
         m_dragPointX = v->getXForFrame
             (m_editingPoint.getFrame() + m_editingPoint.getDuration());
@@ -1076,7 +1002,8 @@ FlexiNoteLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
         m_dragPointX = v->getXForFrame
             (m_editingPoint.getFrame());
     }
-    m_dragPointY = getYForValue(v, m_editingPoint.getValue());
+
+    m_dragPointY = scale.getCoordForValueRounded(v, m_editingPoint.getValue());
 
     if (m_editingCommand) {
         finish(m_editingCommand);
@@ -1084,8 +1011,8 @@ FlexiNoteLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
     }
 
     m_editing = true;
-    m_dragStartX = e->x();
-    m_dragStartY = e->y();
+    m_dragStartX = e->position().x();
+    m_dragStartY = e->position().y();
     
     sv_frame_t onset = m_originalPoint.getFrame();
     sv_frame_t offset =
@@ -1118,14 +1045,14 @@ FlexiNoteLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
 void
 FlexiNoteLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "FlexiNoteLayer::editDrag(" << e->x() << "," << e->y() << ")" << endl;
-    std::cerr << "FlexiNoteLayer::editDrag(" << e->x() << "," << e->y() << ")" << std::endl;
+//    SVDEBUG << "FlexiNoteLayer::editDrag(" << e->position().x() << "," << e->position().y() << ")" << endl;
+    std::cerr << "FlexiNoteLayer::editDrag(" << e->position().x() << "," << e->position().y() << ")" << std::endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
 
-    int xdist = e->x() - m_dragStartX;
-    int ydist = e->y() - m_dragStartY;
+    int xdist = e->position().x() - m_dragStartX;
+    int ydist = e->position().y() - m_dragStartY;
     int newx = m_dragPointX + xdist;
     int newy = m_dragPointY + ydist;
 
@@ -1133,7 +1060,9 @@ FlexiNoteLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
     if (dragFrame < 0) dragFrame = 0;
     dragFrame = dragFrame / model->getResolution() * model->getResolution();
     
-    double value = getValueForY(v, newy);
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    double value = scale.getValueForCoord(v, newy);
 
     if (!m_editingCommand) {
         m_editingCommand =
@@ -1221,7 +1150,7 @@ void
 FlexiNoteLayer::editEnd(LayerGeometryProvider *v, QMouseEvent *e)
 {
     std::cerr << "FlexiNoteLayer::editEnd("
-              << e->x() << "," << e->y() << ")" << std::endl;
+              << e->position().x() << "," << e->position().y() << ")" << std::endl;
     
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
@@ -1266,7 +1195,7 @@ FlexiNoteLayer::splitStart(LayerGeometryProvider *v, QMouseEvent *e)
     // GF: note splitting starts (!! remove printing soon)
     std::cerr << "splitStart (n.b. editStart will be called later, if the user drags the mouse)" << std::endl;
 
-    if (!getPointToDrag(v, e->x(), e->y(), m_editingPoint)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), m_editingPoint)) return;
     // m_originalPoint = m_editingPoint;
     // 
     // m_dragPointX = v->getXForFrame(m_editingPoint.getFrame());
@@ -1278,8 +1207,8 @@ FlexiNoteLayer::splitStart(LayerGeometryProvider *v, QMouseEvent *e)
     }
 
     m_editing = true;
-    m_dragStartX = e->x();
-    m_dragStartY = e->y();
+    m_dragStartX = e->position().x();
+    m_dragStartY = e->position().y();
 }
 
 void
@@ -1290,14 +1219,14 @@ FlexiNoteLayer::splitEnd(LayerGeometryProvider *v, QMouseEvent *e)
     std::cerr << "splitEnd" << std::endl;
     if (!model || !m_editing || m_editMode != SplitNote) return;
 
-    int xdist = e->x() - m_dragStartX;
-    int ydist = e->y() - m_dragStartY;
+    int xdist = e->position().x() - m_dragStartX;
+    int ydist = e->position().y() - m_dragStartY;
     if (xdist != 0 || ydist != 0) { 
         std::cerr << "mouse moved" << std::endl;    
         return; 
     }
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
 
     splitNotesAt(v, frame, e);
 }
@@ -1358,9 +1287,11 @@ FlexiNoteLayer::addNote(LayerGeometryProvider *v, QMouseEvent *e)
     if (!model) return;
 
     sv_frame_t duration = 10000;
+
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
     
-    sv_frame_t frame = v->getFrameForX(e->x());
-    double value = getValueForY(v, e->y());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
+    double value = scale.getValueForCoord(v, e->position().y());
     
     EventVector noteList = model->getAllEvents();
 
@@ -1394,22 +1325,22 @@ FlexiNoteLayer::getAssociatedPitchModel(LayerGeometryProvider *v) const
 {
     // Better than we used to do, but still not very satisfactory
 
-//    cerr << "FlexiNoteLayer::getAssociatedPitchModel()" << endl;
+//    SVCERR << "FlexiNoteLayer::getAssociatedPitchModel()" << endl;
 
     for (int i = 0; i < v->getView()->getLayerCount(); ++i) {
         Layer *layer = v->getView()->getLayer(i);
         if (layer &&
             layer->getLayerPresentationName() != "candidate") {
-//            cerr << "FlexiNoteLayer::getAssociatedPitchModel: looks like our layer is " << layer << endl;
+//            SVCERR << "FlexiNoteLayer::getAssociatedPitchModel: looks like our layer is " << layer << endl;
             auto modelId = layer->getModel();
             auto model = ModelById::getAs<SparseTimeValueModel>(modelId);
             if (model && model->getScaleUnits() == "Hz") {
-//                cerr << "FlexiNoteLayer::getAssociatedPitchModel: it's good, returning " << model << endl;
+//                SVCERR << "FlexiNoteLayer::getAssociatedPitchModel: it's good, returning " << model << endl;
                 return modelId;
             }
         }
     }
-//    cerr << "FlexiNoteLayer::getAssociatedPitchModel: failed to find a model" << endl;
+//    SVCERR << "FlexiNoteLayer::getAssociatedPitchModel: failed to find a model" << endl;
     return {};
 }
 
@@ -1424,21 +1355,21 @@ FlexiNoteLayer::snapSelectedNotesToPitchTrack(LayerGeometryProvider *v, Selectio
 
     auto command = new ChangeEventsCommand(m_model.untyped, tr("Snap Notes"));
 
-    cerr << "snapSelectedNotesToPitchTrack: selection is from " << s.getStartFrame() << " to " << s.getEndFrame() << endl;
+    SVCERR << "snapSelectedNotesToPitchTrack: selection is from " << s.getStartFrame() << " to " << s.getEndFrame() << endl;
 
     for (EventVector::iterator i = points.begin();
          i != points.end(); ++i) {
 
         Event note(*i);
 
-        cerr << "snapSelectedNotesToPitchTrack: looking at note from " << note.getFrame() << " to " << note.getFrame() + note.getDuration() << endl;
+        SVCERR << "snapSelectedNotesToPitchTrack: looking at note from " << note.getFrame() << " to " << note.getFrame() + note.getDuration() << endl;
 
         if (!s.contains(note.getFrame()) &&
             !s.contains(note.getFrame() + note.getDuration() - 1)) {
             continue;
         }
 
-        cerr << "snapSelectedNotesToPitchTrack: making new note" << endl;
+        SVCERR << "snapSelectedNotesToPitchTrack: making new note" << endl;
         Event newNote(note);
 
         command->remove(note);
@@ -1539,33 +1470,33 @@ FlexiNoteLayer::mouseMoveEvent(LayerGeometryProvider *v, QMouseEvent *e)
     // GF: context sensitive cursors
     // v->getView()->setCursor(Qt::ArrowCursor);
     Event note(0);
-    if (!getNoteToEdit(v, e->x(), e->y(), note)) { 
+    if (!getNoteToEdit(v, e->position().x(), e->position().y(), note)) { 
         // v->getView()->setCursor(Qt::UpArrowCursor);
         return; 
     }
 
     bool closeToLeft = false, closeToRight = false,
         closeToTop = false, closeToBottom = false;
-    getRelativeMousePosition(v, note, e->x(), e->y(),
+    getRelativeMousePosition(v, note, e->position().x(), e->position().y(),
                              closeToLeft, closeToRight,
                              closeToTop, closeToBottom);
     
     if (closeToLeft) {
         v->getView()->setCursor(Qt::SizeHorCursor);
         m_editMode = LeftBoundary;
-        cerr << "edit mode -> LeftBoundary" << endl;
+        SVCERR << "edit mode -> LeftBoundary" << endl;
     } else if (closeToRight) {
         v->getView()->setCursor(Qt::SizeHorCursor);
         m_editMode = RightBoundary;
-        cerr << "edit mode -> RightBoundary" << endl;
+        SVCERR << "edit mode -> RightBoundary" << endl;
     } else if (closeToTop) {
         v->getView()->setCursor(Qt::CrossCursor);
         m_editMode = DragNote;
-        cerr << "edit mode -> DragNote" << endl;
+        SVCERR << "edit mode -> DragNote" << endl;
     } else if (closeToBottom) {
         v->getView()->setCursor(Qt::UpArrowCursor);
         m_editMode = SplitNote;
-        cerr << "edit mode -> SplitNote" << endl;
+        SVCERR << "edit mode -> SplitNote" << endl;
     } else {
         v->getView()->setCursor(Qt::ArrowCursor);
     }
@@ -1576,10 +1507,12 @@ FlexiNoteLayer::getRelativeMousePosition(LayerGeometryProvider *v, Event &note,
 {
     // GF: TODO: consolidate the tolerance values
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
     int ctol = 0;
     int noteStartX = v->getXForFrame(note.getFrame());
     int noteEndX = v->getXForFrame(note.getFrame() + note.getDuration());
-    int noteValueY = getYForValue(v,note.getValue());
+    int noteValueY = scale.getCoordForValueRounded(v,note.getValue());
     int noteStartY = noteValueY - (NOTE_HEIGHT / 2);
     int noteEndY = noteValueY + (NOTE_HEIGHT / 2);
     
@@ -1595,7 +1528,7 @@ FlexiNoteLayer::getRelativeMousePosition(LayerGeometryProvider *v, Event &note,
     if (y >= noteStartY - tol && y <= noteStartY + tol) closeToTop = true;
     if (y >= noteEndY - tol && y <= noteEndY + tol) closeToBottom = true;
 
-//    cerr << "FlexiNoteLayer::getRelativeMousePosition: close to: left " << closeToLeft << " right " << closeToRight << " top " << closeToTop << " bottom " << closeToBottom << endl;
+//    SVCERR << "FlexiNoteLayer::getRelativeMousePosition: close to: left " << closeToLeft << " right " << closeToRight << " top " << closeToTop << " bottom " << closeToBottom << endl;
 }
 
 
@@ -1607,7 +1540,9 @@ FlexiNoteLayer::editOpen(LayerGeometryProvider *v, QMouseEvent *e)
     if (!model) return false;
 
     Event note(0);
-    if (!getPointToDrag(v, e->x(), e->y(), note)) return false;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), note)) {
+        return false;
+    }
 
     ItemEditDialog *dialog = new ItemEditDialog
         (model->getSampleRate(),
@@ -1877,7 +1812,7 @@ FlexiNoteLayer::toXml(QTextStream &stream,
 }
 
 void
-FlexiNoteLayer::setProperties(const QXmlAttributes &attributes)
+FlexiNoteLayer::setProperties(const LayerAttributes &attributes)
 {
     SingleColourLayer::setProperties(attributes);
 
@@ -1916,3 +1851,5 @@ FlexiNoteLayer::setVerticalRangeToNoteRange(LayerGeometryProvider *v)
 }
 
 
+} // end namespace sv
+
