@@ -16,6 +16,8 @@
 #ifndef SV_HORIZONTAL_SCALE_PROVIDER_H
 #define SV_HORIZONTAL_SCALE_PROVIDER_H
 
+namespace sv {
+
 class LayerGeometryProvider;
 
 /**
@@ -37,4 +39,6 @@ public:
         const = 0;
 };
 
+} // end namespace sv
+
 #endif
