@@ -32,15 +32,22 @@
 #include <QMutexLocker>
 #include <QTextStream>
 #include <QMessageBox>
+#include <QFileInfo>
+#include <QImageReader>
 
 #include <iostream>
 #include <cmath>
 
+namespace sv {
+
 ImageLayer::ImageMap
 ImageLayer::m_images;
 
+ImageLayer::FileSourceMap
+ImageLayer::m_fileSources;
+
 QMutex
-ImageLayer::m_imageMapMutex;
+ImageLayer::m_staticMutex;
 
 ImageLayer::ImageLayer() :
     m_editing(false),
@@ -121,10 +128,10 @@ ImageLayer::setProperty(const PropertyName &name, int value)
     Layer::setProperty(name, value);
 }
 
-bool
-ImageLayer::getValueExtents(double &, double &, bool &, QString &) const
+Layer::ScaleExtents
+ImageLayer::getVerticalExtents() const
 {
-    return false;
+    return NO_VERTICAL_EXTENTS;
 }
 
 bool
@@ -359,12 +366,7 @@ ImageLayer::drawImage(LayerGeometryProvider *v, QPainter &paint, const Event &p,
             likelyWidth = availableWidth;
         }
 
-        // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-        // replacement (horizontalAdvance) was only added in Qt 5.11
-        // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
-        int singleWidth = paint.fontMetrics().width(label);
+        int singleWidth = paint.fontMetrics().horizontalAdvance(label);
         if (singleWidth < availableWidth && singleWidth < likelyWidth * 2) {
             likelyWidth = singleWidth + 4;
         }
@@ -402,7 +404,7 @@ ImageLayer::drawImage(LayerGeometryProvider *v, QPainter &paint, const Event &p,
         font.setItalic(true);
         paint.setFont(font);
 
-        int tw = paint.fontMetrics().width(additionalText);
+        int tw = paint.fontMetrics().horizontalAdvance(additionalText);
         if (tw > availableWidth) {
             tw = availableWidth;
         }
@@ -464,7 +466,7 @@ ImageLayer::setLayerDormant(const LayerGeometryProvider *v, bool dormant)
         // Delete the images named in the view's scaled map from the
         // general image map as well.  They can always be re-loaded
         // if it turns out another view still needs them.
-        QMutexLocker locker(&m_imageMapMutex);
+        QMutexLocker locker(&m_staticMutex);
         for (ImageMap::iterator i = m_scaled[v].begin();
              i != m_scaled[v].end(); ++i) {
             m_images.erase(i->first);
@@ -480,7 +482,7 @@ ImageLayer::getImageOriginalSize(QString name, QSize &size) const
 {
 //    cerr << "getImageOriginalSize: \"" << name << "\"" << endl;
 
-    QMutexLocker locker(&m_imageMapMutex);
+    QMutexLocker locker(&m_staticMutex);
     if (m_images.find(name) == m_images.end()) {
 //        cerr << "don't have, trying to open local" << endl;
         m_images[name] = QImage(getLocalFilename(name));
@@ -509,7 +511,7 @@ ImageLayer::getImage(LayerGeometryProvider *v, QString name, QSize maxSize) cons
         return m_scaled[v][name];
     }
 
-    QMutexLocker locker(&m_imageMapMutex);
+    QMutexLocker locker(&m_staticMutex);
 
     if (m_images.find(name) == m_images.end()) {
         m_images[name] = QImage(getLocalFilename(name));
@@ -534,7 +536,7 @@ ImageLayer::getImage(LayerGeometryProvider *v, QString name, QSize maxSize) cons
 void
 ImageLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "ImageLayer::drawStart(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "ImageLayer::drawStart(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model) {
@@ -542,7 +544,7 @@ ImageLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
         return;
     }
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
@@ -559,12 +561,12 @@ ImageLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 void
 ImageLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "ImageLayer::drawDrag(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "ImageLayer::drawDrag(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
@@ -577,7 +579,7 @@ ImageLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 void
 ImageLayer::drawEnd(LayerGeometryProvider *, QMouseEvent *)
 {
-//    SVDEBUG << "ImageLayer::drawEnd(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "ImageLayer::drawEnd(" << e->position().x() << "," << e->position().y() << ")" << endl;
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model || !m_editing) return;
 
@@ -587,7 +589,7 @@ ImageLayer::drawEnd(LayerGeometryProvider *, QMouseEvent *)
 
     if (dialog.exec() == QDialog::Accepted) {
 
-        checkAddSource(dialog.getImage());
+        checkAddSourceAndConnect(dialog.getImage());
 
         m_editingPoint = m_editingPoint
             .withURI(dialog.getImage())
@@ -600,15 +602,33 @@ ImageLayer::drawEnd(LayerGeometryProvider *, QMouseEvent *)
     m_editing = false;
 }
 
+bool
+ImageLayer::isImageFileSupported(QString url)
+{
+    QMutexLocker locker(&m_staticMutex);
+    QString filename = getLocalFilename(url);
+    QString ext = QFileInfo(filename).suffix().toLower();
+    auto formats = QImageReader::supportedImageFormats();
+    for (auto f: formats) {
+        if (QString::fromLatin1(f).toLower() == ext) {
+            return true;
+        }
+    }
+    return false;
+}
+
 bool
 ImageLayer::addImage(sv_frame_t frame, QString url)
 {
-    QImage image(getLocalFilename(url));
-    if (image.isNull()) {
-        cerr << "Failed to open image from url \"" << url << "\" (local filename \"" << getLocalFilename(url) << "\"" << endl;
-        delete m_fileSources[url];
-        m_fileSources.erase(url);
-        return false;
+    {
+        QMutexLocker locker(&m_staticMutex);
+        QImage image(getLocalFilename(url));
+        if (image.isNull()) {
+            SVCERR << "Failed to open image from url \"" << url << "\" (local filename \"" << getLocalFilename(url) << "\"" << endl;
+            delete m_fileSources[url];
+            m_fileSources.erase(url);
+            return false;
+        }
     }
 
     Event point = Event(frame).withURI(url);
@@ -622,12 +642,12 @@ ImageLayer::addImage(sv_frame_t frame, QString url)
 void
 ImageLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "ImageLayer::editStart(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "ImageLayer::editStart(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model) return;
 
-    EventVector points = getLocalPoints(v, e->x(), e->y());
+    EventVector points = getLocalPoints(v, e->position().x(), e->position().y());
     if (points.empty()) return;
 
     m_editOrigin = e->pos();
@@ -648,7 +668,7 @@ ImageLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t frameDiff = v->getFrameForX(e->x()) - v->getFrameForX(m_editOrigin.x());
+    sv_frame_t frameDiff = v->getFrameForX(e->position().x()) - v->getFrameForX(m_editOrigin.x());
     sv_frame_t frame = m_originalPoint.getFrame() + frameDiff;
 
     if (frame < 0) frame = 0;
@@ -667,7 +687,7 @@ ImageLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
 void
 ImageLayer::editEnd(LayerGeometryProvider *, QMouseEvent *)
 {
-//    SVDEBUG << "ImageLayer::editEnd(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "ImageLayer::editEnd(" << e->position().x() << "," << e->position().y() << ")" << endl;
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model || !m_editing) return;
 
@@ -685,7 +705,7 @@ ImageLayer::editOpen(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<ImageModel>(m_model);
     if (!model) return false;
 
-    EventVector points = getLocalPoints(v, e->x(), e->y());
+    EventVector points = getLocalPoints(v, e->position().x(), e->position().y());
     if (points.empty()) return false;
 
     QString image = points.begin()->getURI();
@@ -697,7 +717,7 @@ ImageLayer::editOpen(LayerGeometryProvider *v, QMouseEvent *e)
 
     if (dialog.exec() == QDialog::Accepted) {
 
-        checkAddSource(dialog.getImage());
+        checkAddSourceAndConnect(dialog.getImage());
 
         auto command =
             new ChangeEventsCommand(m_model.untyped, tr("Edit Image"));
@@ -865,22 +885,39 @@ ImageLayer::paste(LayerGeometryProvider *v, const Clipboard &from,
 }
 
 QString
-ImageLayer::getLocalFilename(QString img) const
+ImageLayer::getLocalFilename(QString img)
 {
+    // called with mutex held
+    
     if (m_fileSources.find(img) == m_fileSources.end()) {
-        checkAddSource(img);
+        checkAddSource(img, false);
         if (m_fileSources.find(img) == m_fileSources.end()) {
             return img;
         }
     }
+    
     return m_fileSources[img]->getLocalFilename();
 }
 
 void
-ImageLayer::checkAddSource(QString img) const
+ImageLayer::checkAddSourceAndConnect(QString img)
+{
+    checkAddSource(img, true);
+
+    QMutexLocker locker(&m_staticMutex);
+    if (m_fileSources.find(img) != m_fileSources.end()) {
+        connect(m_fileSources.at(img), SIGNAL(ready()),
+                this, SLOT(fileSourceReady()));
+    }
+}
+
+void
+ImageLayer::checkAddSource(QString img, bool synchronise)
 {
     SVDEBUG << "ImageLayer::checkAddSource(" << img << "): yes, trying..." << endl;
 
+    QMutexLocker locker(synchronise ? &m_staticMutex : nullptr);
+
     if (m_fileSources.find(img) != m_fileSources.end()) {
         return;
     }
@@ -888,9 +925,8 @@ ImageLayer::checkAddSource(QString img) const
     ProgressDialog dialog(tr("Opening image URL..."), true, 2000);
     FileSource *rf = new FileSource(img, &dialog);
     if (rf->isOK()) {
-        cerr << "ok, adding it (local filename = " << rf->getLocalFilename() << ")" << endl;
+        SVDEBUG << "ok, adding it (local filename = " << rf->getLocalFilename() << ")" << endl;
         m_fileSources[img] = rf;
-        connect(rf, SIGNAL(ready()), this, SLOT(fileSourceReady()));
     } else {
         delete rf;
     }
@@ -905,7 +941,7 @@ ImageLayer::checkAddSources()
     for (EventVector::const_iterator i = points.begin();
          i != points.end(); ++i) {
         
-        checkAddSource((*i).getURI());
+        checkAddSourceAndConnect(i->getURI());
     }
 }
 
@@ -917,21 +953,30 @@ ImageLayer::fileSourceReady()
     FileSource *rf = dynamic_cast<FileSource *>(sender());
     if (!rf) return;
 
-    QString img;
-    for (FileSourceMap::const_iterator i = m_fileSources.begin();
-         i != m_fileSources.end(); ++i) {
-        if (i->second == rf) {
-            img = i->first;
+    bool shouldEmit = false;
+    
+    {
+        QMutexLocker locker(&m_staticMutex);
+    
+        QString img;
+        for (FileSourceMap::const_iterator i = m_fileSources.begin();
+             i != m_fileSources.end(); ++i) {
+            if (i->second == rf) {
+                img = i->first;
 //            cerr << "it's image \"" << img << "\"" << endl;
-            break;
+                break;
+            }
+        }
+        if (img == "") return;
+
+        m_images.erase(img);
+        for (ViewImageMap::iterator i = m_scaled.begin(); i != m_scaled.end(); ++i) {
+            i->second.erase(img);
+            shouldEmit = true;
         }
     }
-    if (img == "") return;
 
-    QMutexLocker locker(&m_imageMapMutex);
-    m_images.erase(img);
-    for (ViewImageMap::iterator i = m_scaled.begin(); i != m_scaled.end(); ++i) {
-        i->second.erase(img);
+    if (shouldEmit) {
         emit modelChanged(getModel());
     }
 }
@@ -944,7 +989,9 @@ ImageLayer::toXml(QTextStream &stream,
 }
 
 void
-ImageLayer::setProperties(const QXmlAttributes &)
+ImageLayer::setProperties(const LayerAttributes &)
 {
 }
 
+} // end namespace sv
+
