@@ -26,9 +26,10 @@
 
 #include "system/System.h"
 
+#include "CoordinateScale.h"
+
 #include <QObject>
 #include <QRect>
-#include <QXmlAttributes>
 #include <QMutex>
 #include <QPixmap>
 
@@ -37,20 +38,32 @@
 
 #include <iostream>
 
-class ZoomConstraint;
 class QPainter;
+class QMouseEvent;
+
+namespace sv {
+
+class ZoomConstraint;
 class View;
 class LayerGeometryProvider;
-class QMouseEvent;
 class Clipboard;
 class RangeMapper;
 
+/**
+ * This replaces the Qt5 QXmlAttributes, which is no longer in
+ * Qt6. QXmlAttributes was effectively a QList underneath rather than
+ * a QMap, since attributes in XML were ordered and could be
+ * duplicated. But QMap is closer to the proper API and discouraging
+ * duplicate attribute names is actually a good thing for us. We don't
+ * need support for XML namespaces, only name and value.
+ */
+typedef QMap<QString, QString> LayerAttributes;
+
 /**
  * The base class for visual representations of the data found in a
  * Model.  Layers are expected to be able to draw themselves onto a
  * View, and may also be editable.
  */
-
 class Layer : public PropertyContainer,
               public XmlExportable
 {
@@ -188,7 +201,7 @@ public:
         return "";
     }
 
-    virtual QString getLabelPreceding(sv_frame_t /* frame */) const {
+    virtual QString getLabelAtOrPreceding(sv_frame_t /* frame */) const {
         return "";
     }
 
@@ -407,14 +420,14 @@ public:
      * their particular properties.
      */
     void toXml(QTextStream &stream, QString indent = "",
-                       QString extraAttributes = "") const override;
+               QString extraAttributes = "") const override;
 
     /**
      * Set the particular properties of a layer (those specific to the
      * subclass) from a set of XML attributes.  This is the effective
      * inverse of the toXml method.
      */
-    virtual void setProperties(const QXmlAttributes &) = 0;
+    virtual void setProperties(const LayerAttributes &) = 0;
 
     /**
      * Produce XML containing the layer's ID and type.  This is used
@@ -431,7 +444,7 @@ public:
      * (presumably taken from a measurement element).
      * Does not use a command.
      */
-    virtual void addMeasurementRect(const QXmlAttributes &);
+    virtual void addMeasurementRect(const LayerAttributes &);
 
     /**
      * Indicate that a layer is not currently visible in the given
@@ -485,18 +498,54 @@ public:
     }
 
     /**
-     * Return the minimum and maximum values for the y axis of the
-     * model in this layer, as well as whether the layer is configured
-     * to use a logarithmic y axis display.  Also return the unit for
-     * these values if known.
-     *
-     * This function returns the "normal" extents for the layer, not
-     * necessarily the extents actually in use in the display (see
-     * getDisplayExtents).
+     * ScaleApplication describes the current applicability of an
+     * extent range or scale (in any given dimension) in this layer.
      */
-    virtual bool getValueExtents(double &min, double &max,
-                                 bool &logarithmic, QString &unit) const = 0;
+    enum class ScaleApplication {
+        /**
+         * The layer has extents and a scale mapping and they are in
+         * use. The extents reported by the layer are those used to
+         * determine placement of elements in the layer. If requested,
+         * the layer can draw a scale for them. Other layers may be
+         * aligned to the same scale if they share a unit.
+         */
+        Normal,
 
+        /**
+         * The layer has extents and a scale mapping, but they might
+         * not be in use, for example because the layer would prefer
+         * to auto-align with another layer. In that situation, they
+         * may still be used if nothing better is available to align
+         * with. If requested, the layer can draw a scale for them.
+         */
+        Deferring,
+
+        /**
+         * The layer has extents and a scale mapping and, if
+         * requested, can draw a scale. But there is something about
+         * the scale that makes it unsuitable to align another layer
+         * with (e.g. it is not continuous) so it should never be
+         * deferred to.
+         */
+        Personal,
+        
+        /**
+         * The layer has no meaningful extents.
+         */
+        None
+    };
+
+    typedef std::pair<ScaleApplication, CoordinateScale> ScaleExtents;
+
+    static ScaleExtents NO_VERTICAL_EXTENTS;
+    
+    /**
+     * Return the vertical extent and scale mapping for this layer and
+     * an indication of whether the extents are actually in use for
+     * positioning.
+     */
+    virtual ScaleExtents getVerticalExtents() const = 0;
+    
     /**
      * Return the minimum and maximum values within the visible area
      * for the y axis of this layer.
@@ -545,11 +594,21 @@ public:
 
     /** 
      * Return the value and unit at the given y coordinate in the
-     * given view.
+     * given view. Used for measurement rects. May be supported even
+     * when no well-defined scale is returned by getVerticalExtents,
+     * but the default implementation just calls that.
      */
-    virtual bool getYScaleValue(const LayerGeometryProvider *, int /* y */,
-                                double &/* value */, QString &/* unit */) const {
-        return false;
+    virtual bool getYScaleValue(const LayerGeometryProvider *v, int y,
+                                double &value, QString &unit) const {
+
+        auto extents = getVerticalExtents();
+        if (extents.first == ScaleApplication::Normal) {
+            value = extents.second.getValueForCoord(v, y);
+            unit = extents.second.getUnit();
+            return true;
+        } else {
+            return false;
+        }
     }
 
     /**
@@ -711,7 +770,7 @@ protected:
     void paintMeasurementRect(LayerGeometryProvider *v, QPainter &paint,
                               const MeasureRect &r, bool focus) const;
 
-    bool valueExtentsMatchMine(LayerGeometryProvider *v) const;
+    bool verticalExtentsMatchMine(LayerGeometryProvider *v) const;
     
     QString m_presentationName;
 
@@ -720,5 +779,7 @@ private:
     mutable std::map<const void *, bool> m_dormancy;
 };
 
+} // end namespace sv
+
 #endif
 
