@@ -53,6 +53,8 @@
 
 #include <QSettings>
 
+namespace sv {
+
 LayerFactory *
 LayerFactory::m_instance = new LayerFactory;
 
@@ -97,7 +99,7 @@ LayerFactory::getLayerPresentationName(LayerType type)
 
     case UnknownLayer:
     default:
-        cerr << "WARNING: LayerFactory::getLayerPresentationName passed unknown layer" << endl;
+        SVCERR << "WARNING: LayerFactory::getLayerPresentationName passed unknown layer" << endl;
         return Layer::tr("Unknown Layer");
     }
 }
@@ -259,7 +261,7 @@ LayerFactory::getLayerIconName(LayerType type)
     case PeakFrequencySpectrogram: return "spectrogram";
     case UnknownLayer:
     default:
-        cerr << "WARNING: LayerFactory::getLayerIconName passed unknown layer" << endl;
+        SVCERR << "WARNING: LayerFactory::getLayerIconName passed unknown layer" << endl;
         return "unknown";
     }
 }
@@ -286,7 +288,7 @@ LayerFactory::getLayerTypeName(LayerType type)
     case PeakFrequencySpectrogram: return "peakfrequency";
     case UnknownLayer:
     default:
-        cerr << "WARNING: LayerFactory::getLayerTypeName passed unknown layer" << endl;
+        SVCERR << "WARNING: LayerFactory::getLayerTypeName passed unknown layer" << endl;
         return "unknown";
     }
 }
@@ -490,12 +492,12 @@ LayerFactory::createLayer(LayerType type)
 
     case UnknownLayer:
     default:
-        cerr << "WARNING: LayerFactory::createLayer passed unknown layer" << endl;
+        SVCERR << "WARNING: LayerFactory::createLayer passed unknown layer" << endl;
         break;
     }
 
     if (!layer) {
-        cerr << "LayerFactory::createLayer: Unknown layer type " 
+        SVCERR << "LayerFactory::createLayer: Unknown layer type " 
                   << type << endl;
     } else {
 //        SVDEBUG << "LayerFactory::createLayer: Setting object name "
@@ -536,7 +538,7 @@ LayerFactory::setLayerProperties(Layer *layer, QString newXml)
         return;
     }
         
-    QXmlAttributes attrs;
+    LayerAttributes attrs;
         
     QDomElement layerElt = docNew.firstChildElement("layer");
     QDomNamedNodeMap attrNodes = layerElt.attributes();
@@ -544,10 +546,10 @@ LayerFactory::setLayerProperties(Layer *layer, QString newXml)
     for (int i = 0; i < attrNodes.length(); ++i) {
         QDomAttr attr = attrNodes.item(i).toAttr();
         if (attr.isNull()) continue;
-//            cerr << "append \"" << attr.name()
+//            SVCERR << "append \"" << attr.name()
 //                      << "\" -> \"" << attr.value() << "\""
 //                      << endl;
-        attrs.append(attr.name(), "", "", attr.value());
+        attrs[attr.name()] = attr.value();
     }
     
     layerElt = docOld.firstChildElement("layer");
@@ -556,10 +558,10 @@ LayerFactory::setLayerProperties(Layer *layer, QString newXml)
         QDomAttr attr = attrNodes.item(i).toAttr();
         if (attr.isNull()) continue;
         if (attrs.value(attr.name()) == "") {
-//                cerr << "append \"" << attr.name()
+//                SVCERR << "append \"" << attr.name()
 //                          << "\" -> \"" << attr.value() << "\""
 //                          << endl;
-            attrs.append(attr.name(), "", "", attr.value());
+            attrs[attr.name()] = attr.value();
         }
     }
     
@@ -588,3 +590,5 @@ LayerFactory::getLayerTypeForClipboardContents(const Clipboard &clip)
     return TimeInstants;
 }
     
+} // end namespace sv
+
