@@ -19,8 +19,11 @@
 #include <QRect>
 
 class QPainter;
+
+namespace sv {
+
 class LayerGeometryProvider;
-class VerticalScaleLayer;
+class CoordinateScale;
 
 class LinearNumericalScale
 {
@@ -28,9 +31,11 @@ public:
     int getWidth(LayerGeometryProvider *v, QPainter &paint);
 
     void paintVertical
-    (LayerGeometryProvider *v, const VerticalScaleLayer *layer,
-     QPainter &paint, int x0, double minf, double maxf);
+    (LayerGeometryProvider *v, const CoordinateScale &verticalScale,
+     QPainter &paint, int x0);
 };
 
+} // end namespace sv
+
 #endif
 
