@@ -19,6 +19,8 @@
 #include <QRect>
 
 class QPainter;
+namespace sv {
+
 class LayerGeometryProvider;
 class ColourScaleLayer;
 
@@ -32,5 +34,7 @@ public:
      double minf, double maxf);
 };
 
+} // end namespace sv
+
 #endif
 
