@@ -46,6 +46,8 @@
 
 //#define DEBUG_NOTE_LAYER 1
 
+namespace sv {
+
 NoteLayer::NoteLayer() :
     SingleColourLayer(),
     m_modelUsesHz(true),
@@ -257,51 +259,51 @@ NoteLayer::convertValueToEventValue(double value) const
     }
 }
 
-bool
-NoteLayer::getValueExtents(double &min, double &max,
-                           bool &logarithmic, QString &unit) const
+Layer::ScaleExtents
+NoteLayer::getVerticalExtents() const
 {
     auto model = ModelById::getAs<NoteModel>(m_model);
-    if (!model) return false;
+    if (!model) return NO_VERTICAL_EXTENTS;
+
+    double min = convertValueFromEventValue(model->getValueMinimum());
+    double max = convertValueFromEventValue(model->getValueMaximum());
+    bool logarithmic = (m_verticalScale != LinearScale);
+    QString unit = "Hz";
 
-    min = convertValueFromEventValue(model->getValueMinimum());
-    max = convertValueFromEventValue(model->getValueMaximum());
     min /= 1.06;
     max *= 1.06;
-    unit = "Hz";
 
-    if (m_verticalScale != LinearScale) {
-        logarithmic = true;
+    CoordinateScale scale(CoordinateScale::Direction::Vertical,
+                          unit, logarithmic, min, max);
+
+    if (m_verticalScale == MIDIRangeScale) {
+        min = Pitch::getFrequencyForPitch(0);
+        max = Pitch::getFrequencyForPitch(127);
+        scale = scale.withDisplayExtents(min, max);
+    } else if (m_scaleMinimum != m_scaleMaximum) {
+        min = m_scaleMinimum;
+        max = m_scaleMaximum;
+        scale = scale.withDisplayExtents(min, max);
     }
 
-    return true;
+    if (m_verticalScale == AutoAlignScale) {
+        return { Layer::ScaleApplication::Deferring, scale };
+    } else {
+        return { Layer::ScaleApplication::Normal, scale };
+    }
 }
 
 bool
 NoteLayer::getDisplayExtents(double &min, double &max) const
 {
-    auto model = ModelById::getAs<NoteModel>(m_model);
-    if (!model || shouldAutoAlign()) return false;
-
-    if (m_verticalScale == MIDIRangeScale) {
-        min = Pitch::getFrequencyForPitch(0);
-        max = Pitch::getFrequencyForPitch(127);
-        return true;
+    auto extents = getVerticalExtents();
+    if (extents.first == ScaleApplication::None ||
+        extents.first == ScaleApplication::Deferring) {
+        return false;
     }
-
-    if (m_scaleMinimum == m_scaleMaximum) {
-        QString unit;
-        bool log = false;
-        getValueExtents(min, max, log, unit);
-    } else {
-        min = m_scaleMinimum;
-        max = m_scaleMaximum;
-    }
-
-#ifdef DEBUG_NOTE_LAYER
-    SVCERR << "NoteLayer::getDisplayExtents: min = " << min << ", max = " << max << " (m_scaleMinimum = " << m_scaleMinimum << ", m_scaleMaximum = " << m_scaleMaximum << ")" << endl;
-#endif
-
+    const auto &scale = extents.second;
+    min = scale.getDisplayMinimum();
+    max = scale.getDisplayMaximum();
     return true;
 }
 
@@ -365,26 +367,26 @@ NoteLayer::setVerticalZoomStep(int step)
     RangeMapper *mapper = getNewVerticalZoomRangeMapper();
     if (!mapper) return;
     
-    double min, max;
-    bool logarithmic;
-    QString unit;
-    getValueExtents(min, max, logarithmic, unit);
+    CoordinateScale scale = getVerticalExtents().second;
     
-    double dmin, dmax;
-    getDisplayExtents(dmin, dmax);
+    double min = scale.getValueMinimum();
+    double max = scale.getValueMaximum();
+    
+    double dmin = scale.getDisplayMinimum();
+    double dmax = scale.getDisplayMaximum();
 
     double newdist = mapper->getValueForPosition(100 - step);
 
     double newmin, newmax;
 
-    if (logarithmic) {
+    if (scale.isLogarithmic()) {
 
         // see SpectrogramLayer::setVerticalZoomStep
 
         newmax = (newdist + sqrt(newdist*newdist + 4*dmin*dmax)) / 2;
         newmin = newmax - newdist;
 
-//        cerr << "newmin = " << newmin << ", newmax = " << newmax << endl;
+//        SVCERR << "newmin = " << newmin << ", newmax = " << newmax << endl;
 
     } else {
         double dmid = (dmax + dmin) / 2;
@@ -414,14 +416,15 @@ NoteLayer::getNewVerticalZoomRangeMapper() const
     
     RangeMapper *mapper;
 
-    double min, max;
-    bool logarithmic;
-    QString unit;
-    getValueExtents(min, max, logarithmic, unit);
+    CoordinateScale scale = getVerticalExtents().second;
+    
+    double min = scale.getValueMinimum();
+    double max = scale.getValueMaximum();
+    QString unit = scale.getUnit();
 
     if (min == max) return nullptr;
     
-    if (logarithmic) {
+    if (scale.isLogarithmic()) {
         mapper = new LogRangeMapper(0, 100, min, max, unit);
     } else {
         mapper = new LinearRangeMapper(0, 100, min, max, unit);
@@ -465,9 +468,12 @@ NoteLayer::getPointToDrag(LayerGeometryProvider *v, int x, int y, Event &point)
     EventVector onPoints = model->getEventsCovering(frame);
     if (onPoints.empty()) return false;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+    
     int nearestDistance = -1;
     for (const auto &p: onPoints) {
-        int distance = getYForValue(v, valueOf(p)) - y;
+        int py = scale.getCoordForValueRounded(v, valueOf(p));
+        int distance = py - y;
         if (distance < 0) distance = -distance;
         if (nearestDistance == -1 || distance < nearestDistance) {
             nearestDistance = distance;
@@ -499,13 +505,16 @@ NoteLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) const
     Event note;
     EventVector::iterator i;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+    
     for (i = points.begin(); i != points.end(); ++i) {
 
-        int y = getYForValue(v, valueOf(*i));
+        int y = scale.getCoordForValueRounded(v, valueOf(*i));
         int h = 3;
 
         if (model->getValueQuantization() != 0.0) {
-            h = y - getYForValue
+            h = y -
+                scale.getCoordForValueRounded
                 (v, convertValueFromEventValue(i->getValue() +
                                                model->getValueQuantization()));
             if (h < 3) h = 3;
@@ -565,7 +574,7 @@ NoteLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) const
     }
 
     pos = QPoint(v->getXForFrame(note.getFrame()),
-                 getYForValue(v, valueOf(note)));
+                 scale.getCoordForValueRounded(v, valueOf(note)));
     return text;
 }
 
@@ -608,94 +617,6 @@ NoteLayer::snapToFeatureFrame(LayerGeometryProvider *v, sv_frame_t &frame,
     return false;
 }
 
-void
-NoteLayer::getScaleExtents(LayerGeometryProvider *v, double &min, double &max, bool &log) const
-{
-    min = 0.0;
-    max = 0.0;
-    log = false;
-
-    auto model = ModelById::getAs<NoteModel>(m_model);
-    if (!model) return;
-    
-    if (shouldAutoAlign()) {
-
-        if (!v->getVisibleExtentsForUnit("Hz", min, max, log)) {
-
-            QString unit;
-            getValueExtents(min, max, log, unit);
-
-#ifdef DEBUG_NOTE_LAYER
-            SVCERR << "NoteLayer[" << this << "]::getScaleExtents: min = " << min << ", max = " << max << ", log = " << log << endl;
-#endif
-
-        } else if (log) {
-
-            LogRange::mapRange(min, max);
-
-#ifdef DEBUG_NOTE_LAYER
-            SVCERR << "NoteLayer[" << this << "]::getScaleExtents: min = " << min << ", max = " << max << ", log = " << log << endl;
-#endif
-        }
-
-    } else {
-
-        getDisplayExtents(min, max);
-
-        if (m_verticalScale != LinearScale) {
-            LogRange::mapRange(min, max);
-            log = true;
-        }
-    }
-
-    if (max == min) max = min + 1.0;
-}
-
-int
-NoteLayer::getYForValue(LayerGeometryProvider *v, double val) const
-{
-    double min = 0.0, max = 0.0;
-    bool logarithmic = false;
-    int h = v->getPaintHeight();
-
-    getScaleExtents(v, min, max, logarithmic);
-
-#ifdef DEBUG_NOTE_LAYER
-    SVCERR << "NoteLayer[" << this << "]::getYForValue(" << val << "): min = " << min << ", max = " << max << ", log = " << logarithmic << endl;
-#endif
-
-    if (logarithmic) {
-        val = LogRange::map(val);
-#ifdef DEBUG_NOTE_LAYER
-        SVCERR << "logarithmic true, val now = " << val << endl;
-#endif
-    }
-
-    int y = int(h - ((val - min) * h) / (max - min)) - 1;
-#ifdef DEBUG_NOTE_LAYER
-    SVCERR << "y = " << y << endl;
-#endif
-    return y;
-}
-
-double
-NoteLayer::getValueForY(LayerGeometryProvider *v, int y) const
-{
-    double min = 0.0, max = 0.0;
-    bool logarithmic = false;
-    int h = v->getPaintHeight();
-
-    getScaleExtents(v, min, max, logarithmic);
-
-    double val = min + (double(h - y) * double(max - min)) / h;
-
-    if (logarithmic) {
-        val = pow(10.0, val);
-    }
-
-    return val;
-}
-
 bool
 NoteLayer::shouldAutoAlign() const
 {
@@ -752,19 +673,22 @@ NoteLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) const
 
     paint.save();
     paint.setRenderHint(QPainter::Antialiasing, false);
-    
+
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
     for (EventVector::const_iterator i = points.begin();
          i != points.end(); ++i) {
 
         const Event &p(*i);
 
         int x = v->getXForFrame(p.getFrame());
-        int y = getYForValue(v, valueOf(p));
+        int y = scale.getCoordForValueRounded(v, valueOf(p));
         int w = v->getXForFrame(p.getFrame() + p.getDuration()) - x;
         int h = 3;
         
         if (model->getValueQuantization() != 0.0) {
-            h = y - getYForValue
+            h = y -
+                scale.getCoordForValueRounded
                 (v, convertValueFromEventValue
                  (p.getValue() + model->getValueQuantization()));
             if (h < 3) h = 3;
@@ -779,11 +703,6 @@ NoteLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) const
             paint.setPen(v->getForeground());
             paint.setBrush(v->getForeground());
 
-    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-    // replacement (horizontalAdvance) was only added in Qt 5.11
-    // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
             QString vlabel;
             if (m_modelUsesHz) {
                 vlabel = QString("%1%2")
@@ -796,7 +715,7 @@ NoteLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) const
             }
             
             PaintAssistant::drawVisibleText(v, paint, 
-                               x - paint.fontMetrics().width(vlabel) - 2,
+                               x - paint.fontMetrics().horizontalAdvance(vlabel) - 2,
                                y + paint.fontMetrics().height()/2
                                  - paint.fontMetrics().descent(), 
                                vlabel, PaintAssistant::OutlinedText);
@@ -822,7 +741,7 @@ NoteLayer::getVerticalScaleWidth(LayerGeometryProvider *v, bool, QPainter &paint
         return 0;
     }
 
-    if (shouldAutoAlign() && !valueExtentsMatchMine(v)) {
+    if (shouldAutoAlign() && !verticalExtentsMatchMine(v)) {
         return 0;
     }
 
@@ -839,26 +758,22 @@ NoteLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &paint, Q
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || model->isEmpty()) return;
 
-    QString unit;
-    double min, max;
-    bool logarithmic;
+    // We are only asked to draw if we are the reference scale, so
+    // don't use getEffectiveVerticalExtentsForLayer here
+    CoordinateScale scale = getVerticalExtents().second;
 
     int w = getVerticalScaleWidth(v, false, paint);
     int h = v->getPaintHeight();
 
-    getScaleExtents(v, min, max, logarithmic);
-
-    if (logarithmic) {
-        LogNumericalScale().paintVertical(v, this, paint, 0, min, max);
+    if (scale.isLogarithmic()) {
+        LogNumericalScale().paintVertical(v, scale, paint, 0);
     } else {
-        LinearNumericalScale().paintVertical(v, this, paint, 0, min, max);
+        LinearNumericalScale().paintVertical(v, scale, paint, 0);
     }
     
-    if (logarithmic) {
+    if (!scale.isLinear() && scale.getUnit() == "Hz") {
         PianoScale().paintPianoVertical
-            (v, paint, QRect(w - 10, 0, 10, h), 
-             LogRange::unmap(min), 
-             LogRange::unmap(max));
+            (v, paint, QRect(w - 10, 0, 10, h), scale);
         paint.drawLine(w, 0, w, h);
     }
         
@@ -875,16 +790,18 @@ NoteLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &paint, Q
 void
 NoteLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "NoteLayer::drawStart(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "NoteLayer::drawStart(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double value = getValueForY(v, e->y());
+    double value = scale.getValueForCoord(v, e->position().y());
     float eventValue = convertValueToEventValue(value);
     eventValue = roundf(eventValue);
 
@@ -901,16 +818,18 @@ NoteLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 void
 NoteLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "NoteLayer::drawDrag(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "NoteLayer::drawDrag(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double newValue = getValueForY(v, e->y());
+    double newValue = scale.getValueForCoord(v, e->position().y());
     float newEventValue = convertValueToEventValue(newValue);
     newEventValue = roundf(newEventValue);
 
@@ -934,7 +853,7 @@ NoteLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 void
 NoteLayer::drawEnd(LayerGeometryProvider *, QMouseEvent *)
 {
-//    SVDEBUG << "NoteLayer::drawEnd(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "NoteLayer::drawEnd(" << e->position().x() << "," << e->position().y() << ")" << endl;
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
     finish(m_editingCommand);
@@ -948,7 +867,7 @@ NoteLayer::eraseStart(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model) return;
 
-    if (!getPointToDrag(v, e->x(), e->y(), m_editingPoint)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), m_editingPoint)) return;
 
     if (m_editingCommand) {
         finish(m_editingCommand);
@@ -972,7 +891,7 @@ NoteLayer::eraseEnd(LayerGeometryProvider *v, QMouseEvent *e)
     m_editing = false;
 
     Event p(0);
-    if (!getPointToDrag(v, e->x(), e->y(), p)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), p)) return;
     if (p.getFrame() != m_editingPoint.getFrame() ||
         p.getValue() != m_editingPoint.getValue()) return;
 
@@ -988,16 +907,18 @@ NoteLayer::eraseEnd(LayerGeometryProvider *v, QMouseEvent *e)
 void
 NoteLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "NoteLayer::editStart(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "NoteLayer::editStart(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model) return;
 
-    if (!getPointToDrag(v, e->x(), e->y(), m_editingPoint)) return;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), m_editingPoint)) return;
     m_originalPoint = m_editingPoint;
 
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
     m_dragPointX = v->getXForFrame(m_editingPoint.getFrame());
-    m_dragPointY = getYForValue(v, valueOf(m_editingPoint));
+    m_dragPointY = scale.getCoordForValueRounded(v, valueOf(m_editingPoint));
 
     if (m_editingCommand) {
         finish(m_editingCommand);
@@ -1005,20 +926,22 @@ NoteLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
     }
 
     m_editing = true;
-    m_dragStartX = e->x();
-    m_dragStartY = e->y();
+    m_dragStartX = e->position().x();
+    m_dragStartY = e->position().y();
 }
 
 void
 NoteLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
-//    SVDEBUG << "NoteLayer::editDrag(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "NoteLayer::editDrag(" << e->position().x() << "," << e->position().y() << ")" << endl;
 
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
 
-    int xdist = e->x() - m_dragStartX;
-    int ydist = e->y() - m_dragStartY;
+    CoordinateScale scale = v->getEffectiveVerticalExtentsForLayer(this);
+
+    int xdist = e->position().x() - m_dragStartX;
+    int ydist = e->position().y() - m_dragStartY;
     int newx = m_dragPointX + xdist;
     int newy = m_dragPointY + ydist;
 
@@ -1026,7 +949,7 @@ NoteLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
-    double newValue = getValueForY(v, newy);
+    double newValue = scale.getValueForCoord(v, newy);
     float newEventValue = convertValueToEventValue(newValue);
     newEventValue = roundf(newEventValue);
 
@@ -1045,7 +968,7 @@ NoteLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
 void
 NoteLayer::editEnd(LayerGeometryProvider *, QMouseEvent *)
 {
-//    SVDEBUG << "NoteLayer::editEnd(" << e->x() << "," << e->y() << ")" << endl;
+//    SVDEBUG << "NoteLayer::editEnd(" << e->position().x() << "," << e->position().y() << ")" << endl;
     auto model = ModelById::getAs<NoteModel>(m_model);
     if (!model || !m_editing) return;
 
@@ -1078,7 +1001,9 @@ NoteLayer::editOpen(LayerGeometryProvider *v, QMouseEvent *e)
     if (!model) return false;
 
     Event note(0);
-    if (!getPointToDrag(v, e->x(), e->y(), note)) return false;
+    if (!getPointToDrag(v, e->position().x(), e->position().y(), note)) {
+        return false;
+    }
 
     ItemEditDialog *dialog = new ItemEditDialog
         (model->getSampleRate(),
@@ -1347,7 +1272,7 @@ NoteLayer::toXml(QTextStream &stream,
 }
 
 void
-NoteLayer::setProperties(const QXmlAttributes &attributes)
+NoteLayer::setProperties(const LayerAttributes &attributes)
 {
     SingleColourLayer::setProperties(attributes);
 
@@ -1362,3 +1287,5 @@ NoteLayer::setProperties(const QXmlAttributes &attributes)
 }
 
 
+} // end namespace sv
+
