@@ -23,6 +23,8 @@
 #include "base/AudioLevel.h"
 
 class QPainter;
+namespace sv {
+
 class Layer;
 class LayerGeometryProvider;
 
@@ -52,4 +54,6 @@ public:
                                 QString text, TextStyle style);
 };
 
+} // end namespace sv
+
 #endif
