@@ -18,6 +18,8 @@
 
 #include "Layer.h"
 
+namespace sv {
+
 /**
  * Base class for layers that can be sliced, that is, that contain
  * models appropriate for use in a SliceLayer.
@@ -35,8 +37,13 @@ public:
     // The SliceableLayer retains ownership of the model, and will
     // emit sliceableModelReplaced if it is about to become invalid.
     virtual ModelId getSliceableModel() const = 0;
+
+signals:
+    void sliceableModelReplaced(ModelId, ModelId);
 };
 
+} // end namespace sv
+
 #endif
 
 
