@@ -37,8 +37,6 @@
 #include "Colour3DPlotExporter.h"
 
 #include <QPainter>
-#include <QImage>
-#include <QPixmap>
 #include <QRect>
 #include <QApplication>
 #include <QMessageBox>
@@ -56,6 +54,11 @@
 
 using namespace std;
 
+namespace sv {
+
+static const int lowestSettableThreshold_dB = -160;
+static const int negInfThreshold_dB = lowestSettableThreshold_dB - 1;
+
 SpectrogramLayer::SpectrogramLayer(Configuration config) :
     m_channel(0),
     m_windowSize(1024),
@@ -64,8 +67,8 @@ SpectrogramLayer::SpectrogramLayer(Configuration config) :
     m_oversampling(1),
     m_gain(1.0),
     m_initialGain(1.0),
-    m_threshold(1.0e-8f),
-    m_initialThreshold(1.0e-8f),
+    m_threshold(AudioLevel::dB_to_voltage(lowestSettableThreshold_dB)),
+    m_initialThreshold(AudioLevel::dB_to_voltage(lowestSettableThreshold_dB)),
     m_colourRotation(0),
     m_initialRotation(0),
     m_minFrequency(10),
@@ -76,10 +79,12 @@ SpectrogramLayer::SpectrogramLayer(Configuration config) :
     m_colourScaleMultiple(1.0),
     m_colourMap(0),
     m_colourInverted(false),
-    m_binScale(BinScale::Linear),
+    m_frequencyMapping(FrequencyMapping::Linear),
     m_binDisplay(BinDisplay::AllBins),
     m_normalization(ColumnNormalization::None),
     m_normalizeVisibleArea(false),
+    m_opaque(true),
+    m_smooth(true),
     m_lastEmittedZoomStep(-1),
     m_synchronous(false),
     m_haveDetailedScale(false),
@@ -100,7 +105,7 @@ SpectrogramLayer::SpectrogramLayer(Configuration config) :
         setMinFrequency(40);
         setColourScale(ColourScaleType::Linear);
         setColourMap(ColourMapper::Sunset);
-        setBinScale(BinScale::Log);
+        setFrequencyMapping(FrequencyMapping::Log);
         colourConfigName = "spectrogram-melodic-colour";
         colourConfigDefault = int(ColourMapper::Sunset);
 //        setGain(20);
@@ -110,7 +115,7 @@ SpectrogramLayer::SpectrogramLayer(Configuration config) :
         m_initialMaxFrequency = 2000;
         setMaxFrequency(2000);
         setMinFrequency(40);
-        setBinScale(BinScale::Log);
+        setFrequencyMapping(FrequencyMapping::Log);
         setColourScale(ColourScaleType::Linear);
         setBinDisplay(BinDisplay::PeakFrequencies);
         setNormalization(ColumnNormalization::Max1);
@@ -127,6 +132,9 @@ SpectrogramLayer::SpectrogramLayer(Configuration config) :
     connect(prefs, SIGNAL(propertyChanged(PropertyContainer::PropertyName)),
             this, SLOT(preferenceChanged(PropertyContainer::PropertyName)));
     setWindowType(prefs->getWindowType());
+        
+    auto smoothing = prefs->getSpectrogramSmoothing();
+    m_smooth = (smoothing != Preferences::NoSpectrogramSmoothing);
 }
 
 SpectrogramLayer::~SpectrogramLayer()
@@ -259,6 +267,8 @@ SpectrogramLayer::setModel(ModelId modelId)
     if (!modelId.isNone() && !newModel) {
         throw std::logic_error("Not a DenseTimeValueModel");
     }
+
+    ModelId oldModelId = m_model;
     
     if (modelId == m_model) return;
     m_model = modelId;
@@ -277,6 +287,7 @@ SpectrogramLayer::setModel(ModelId modelId)
     }
     
     emit modelReplaced();
+    emit sliceableModelReplaced(oldModelId, m_model);
 }
 
 Layer::PropertyList
@@ -296,6 +307,8 @@ SpectrogramLayer::getProperties() const
 //    list.push_back("Min Frequency");
 //    list.push_back("Max Frequency");
     list.push_back("Frequency Scale");
+    list.push_back("Opaque");
+    list.push_back("Smooth");
     return list;
 }
 
@@ -315,12 +328,16 @@ SpectrogramLayer::getPropertyLabel(const PropertyName &name) const
     if (name == "Min Frequency") return tr("Min Frequency");
     if (name == "Max Frequency") return tr("Max Frequency");
     if (name == "Frequency Scale") return tr("Frequency Scale");
+    if (name == "Opaque") return tr("Opaque");
+    if (name == "Smooth") return tr("Smooth");
     return "";
 }
 
 QString
-SpectrogramLayer::getPropertyIconName(const PropertyName &) const
+SpectrogramLayer::getPropertyIconName(const PropertyName &name) const
 {
+    if (name == "Opaque") return "opaque";
+    if (name == "Smooth") return "smooth";
     return "";
 }
 
@@ -331,6 +348,8 @@ SpectrogramLayer::getPropertyType(const PropertyName &name) const
     if (name == "Colour Rotation") return RangeProperty;
     if (name == "Threshold") return RangeProperty;
     if (name == "Colour") return ColourMapProperty;
+    if (name == "Opaque") return ToggleProperty;
+    if (name == "Smooth") return ToggleProperty;
     return ValueProperty;
 }
 
@@ -344,7 +363,9 @@ SpectrogramLayer::getPropertyGroupName(const PropertyName &name) const
         name == "Oversampling") return tr("Window");
     if (name == "Colour" ||
         name == "Threshold" ||
-        name == "Colour Rotation") return tr("Colour");
+        name == "Colour Rotation" ||
+        name == "Opaque" ||
+        name == "Smooth") return tr("Colour");
     if (name == "Normalization" ||
         name == "Gain" ||
         name == "Colour Scale") return tr("Scale");
@@ -377,7 +398,7 @@ SpectrogramLayer::getPropertyRangeAndValue(const PropertyName &name,
 
     } else if (name == "Threshold") {
 
-        *min = -81;
+        *min = negInfThreshold_dB;
         *max = -1;
 
         *deflt = int(round(AudioLevel::voltage_to_dB(m_initialThreshold)));
@@ -482,9 +503,9 @@ SpectrogramLayer::getPropertyRangeAndValue(const PropertyName &name,
     } else if (name == "Frequency Scale") {
 
         *min = 0;
-        *max = 1;
-        *deflt = int(BinScale::Linear);
-        val = (int)m_binScale;
+        *max = 2;
+        *deflt = int(FrequencyMapping::Linear);
+        val = (int)m_frequencyMapping;
 
     } else if (name == "Bin Display") {
 
@@ -501,6 +522,20 @@ SpectrogramLayer::getPropertyRangeAndValue(const PropertyName &name,
         
         val = convertFromColumnNorm(m_normalization, m_normalizeVisibleArea);
 
+    } else if (name == "Opaque") {
+        
+        *min = 0;
+        *max = 1;
+        *deflt = 1;
+        val = (m_opaque ? 1 : 0);
+        
+    } else if (name == "Smooth") {
+        
+        *min = 0;
+        *max = 1;
+        *deflt = 0;
+        val = (m_smooth ? 1 : 0);
+        
     } else {
         val = Layer::getPropertyRangeAndValue(name, min, max, deflt);
     }
@@ -592,7 +627,8 @@ SpectrogramLayer::getPropertyValueLabel(const PropertyName &name,
         switch (value) {
         default:
         case 0: return tr("Linear");
-        case 1: return tr("Log");
+        case 1: return tr("Mel");
+        case 2: return tr("Log");
         }
     }
     if (name == "Bin Display") {
@@ -629,8 +665,11 @@ SpectrogramLayer::getNewPropertyRangeMapper(const PropertyName &name) const
         return new LinearRangeMapper(-50, 50, -25, 25, tr("dB"));
     }
     if (name == "Threshold") {
-        return new LinearRangeMapper(-81, -1, -81, -1, tr("dB"), false,
-                                     { { -81, Strings::minus_infinity } });
+        return new LinearRangeMapper(negInfThreshold_dB, -1,
+                                     negInfThreshold_dB, -1,
+                                     tr("dB"), false,
+                                     { { negInfThreshold_dB,
+                                           Strings::minus_infinity } });
     }
     return nullptr;
 }
@@ -641,7 +680,7 @@ SpectrogramLayer::setProperty(const PropertyName &name, int value)
     if (name == "Gain") {
         setGain(AudioLevel::dB_to_voltage(value));
     } else if (name == "Threshold") {
-        if (value == -81) setThreshold(0.0);
+        if (value == negInfThreshold_dB) setThreshold(0.0);
         else setThreshold(AudioLevel::dB_to_voltage(value));
     } else if (name == "Colour Rotation") {
         setColourRotation(value);
@@ -707,8 +746,9 @@ SpectrogramLayer::setProperty(const PropertyName &name, int value)
     } else if (name == "Frequency Scale") {
         switch (value) {
         default:
-        case 0: setBinScale(BinScale::Linear); break;
-        case 1: setBinScale(BinScale::Log); break;
+        case 0: setFrequencyMapping(FrequencyMapping::Linear); break;
+        case 1: setFrequencyMapping(FrequencyMapping::Mel); break;
+        case 2: setFrequencyMapping(FrequencyMapping::Log); break;
         }
     } else if (name == "Bin Display") {
         switch (value) {
@@ -717,6 +757,10 @@ SpectrogramLayer::setProperty(const PropertyName &name, int value)
         case 1: setBinDisplay(BinDisplay::PeakBins); break;
         case 2: setBinDisplay(BinDisplay::PeakFrequencies); break;
         }
+    } else if (name == "Opaque") {
+        setOpaque(value ? true : false);
+    } else if (name == "Smooth") {
+        setSmooth(value ? true : false);
     } else if (name == "Normalization") {
         auto n = convertToColumnNorm(value);
         setNormalization(n.first);
@@ -728,7 +772,7 @@ void
 SpectrogramLayer::invalidateRenderers()
 {
 #ifdef DEBUG_SPECTROGRAM
-    cerr << "SpectrogramLayer::invalidateRenderers called" << endl;
+    SVDEBUG << "SpectrogramLayer::invalidateRenderers called" << endl;
 #endif
 
     for (ViewRendererMap::iterator i = m_renderers.begin();
@@ -747,15 +791,10 @@ SpectrogramLayer::preferenceChanged(PropertyContainer::PropertyName name)
         setWindowType(Preferences::getInstance()->getWindowType());
         return;
     }
-    if (name == "Spectrogram Y Smoothing") {
-        invalidateRenderers();
-        invalidateMagnitudes();
-        emit layerParametersChanged();
-    }
-    if (name == "Spectrogram X Smoothing") {
-        invalidateRenderers();
-        invalidateMagnitudes();
-        emit layerParametersChanged();
+    if (name == "Spectrogram Smoothing") {
+        setSmooth(Preferences::getInstance()->getSpectrogramSmoothing() ==
+                  Preferences::SpectrogramInterpolated);
+        return;
     }
     if (name == "Tuning Frequency") {
         emit layerParametersChanged();
@@ -834,6 +873,36 @@ SpectrogramLayer::getOversampling() const
     return m_oversampling;
 }
 
+void
+SpectrogramLayer::setOpaque(bool n)
+{
+    if (m_opaque == n) return;
+    m_opaque = n;
+    invalidateRenderers();
+    emit layerParametersChanged();
+}
+
+void
+SpectrogramLayer::setSmooth(bool n)
+{
+    if (m_smooth == n) return;
+    m_smooth = n;
+    invalidateRenderers();
+    emit layerParametersChanged();
+}
+
+bool
+SpectrogramLayer::getOpaque() const
+{
+    return m_opaque;
+}
+
+bool
+SpectrogramLayer::getSmooth() const
+{
+    return m_smooth;
+}
+
 void
 SpectrogramLayer::setWindowType(WindowType w)
 {
@@ -1020,18 +1089,28 @@ SpectrogramLayer::getColourMap() const
 void
 SpectrogramLayer::setBinScale(BinScale binScale)
 {
-    if (m_binScale == binScale) return;
+    if (binScale == BinScale::Log) {
+        setFrequencyMapping(FrequencyMapping::Log);
+    } else {
+        setFrequencyMapping(FrequencyMapping::Linear);
+    }
+}
+
+void
+SpectrogramLayer::setFrequencyMapping(FrequencyMapping mapping)
+{
+    if (m_frequencyMapping == mapping) return;
 
     invalidateRenderers();
-    m_binScale = binScale;
+    m_frequencyMapping = mapping;
 
     emit layerParametersChanged();
 }
 
-BinScale
-SpectrogramLayer::getBinScale() const
+FrequencyMapping
+SpectrogramLayer::getFrequencyMapping() const
 {
-    return m_binScale;
+    return m_frequencyMapping;
 }
 
 void
@@ -1093,7 +1172,7 @@ SpectrogramLayer::setLayerDormant(const LayerGeometryProvider *v, bool dormant)
     if (dormant) {
 
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-        cerr << "SpectrogramLayer::setLayerDormant(" << dormant << ")"
+        SVDEBUG << "SpectrogramLayer::setLayerDormant(" << dormant << ")"
                   << endl;
 #endif
 
@@ -1124,7 +1203,7 @@ void
 SpectrogramLayer::cacheInvalid(ModelId)
 {
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "SpectrogramLayer::cacheInvalid()" << endl;
+    SVDEBUG << "SpectrogramLayer::cacheInvalid()" << endl;
 #endif
 
     invalidateRenderers();
@@ -1142,7 +1221,7 @@ SpectrogramLayer::cacheInvalid(
     )
 {
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "SpectrogramLayer::cacheInvalid(" << from << ", " << to << ")" << endl;
+    SVDEBUG << "SpectrogramLayer::cacheInvalid(" << from << ", " << to << ")" << endl;
 #endif
 
     // We used to call invalidateMagnitudes(from, to) to invalidate
@@ -1165,37 +1244,22 @@ SpectrogramLayer::hasLightBackground() const
 double
 SpectrogramLayer::getEffectiveMinFrequency() const
 {
-    auto model = ModelById::getAs<DenseTimeValueModel>(m_model);
-    if (!model) return 0.0;
-    
-    sv_samplerate_t sr = model->getSampleRate();
-    double minf = double(sr) / getFFTSize();
-
-    if (m_minFrequency > 0.0) {
-        int minbin = int((double(m_minFrequency) * getFFTSize()) / sr + 0.01);
-        if (minbin < 1) minbin = 1;
-        minf = minbin * sr / getFFTSize();
-    }
-
-    return minf;
+    return m_minFrequency;
 }
 
 double
 SpectrogramLayer::getEffectiveMaxFrequency() const
 {
-    auto model = ModelById::getAs<DenseTimeValueModel>(m_model);
-    if (!model) return 0.0;
+    if (m_maxFrequency > m_minFrequency) {
+        return m_maxFrequency;
+    } else {
+        auto model = ModelById::getAs<DenseTimeValueModel>(m_model);
+        if (!model) return 0.0;
     
-    sv_samplerate_t sr = model->getSampleRate();
-    double maxf = double(sr) / 2;
-
-    if (m_maxFrequency > 0.0) {
-        int maxbin = int((double(m_maxFrequency) * getFFTSize()) / sr + 0.1);
-        if (maxbin > getFFTSize() / 2) maxbin = getFFTSize() / 2;
-        maxf = maxbin * sr / getFFTSize();
+        sv_samplerate_t sr = model->getSampleRate();
+        double maxf = double(sr) / 2;
+        return maxf;
     }
-
-    return maxf;
 }
 
 bool
@@ -1217,12 +1281,11 @@ SpectrogramLayer::getYForBin(const LayerGeometryProvider *v, double bin) const
     
     double minf = getEffectiveMinFrequency();
     double maxf = getEffectiveMaxFrequency();
-    bool logarithmic = (m_binScale == BinScale::Log);
     sv_samplerate_t sr = model->getSampleRate();
 
     double freq = (bin * sr) / getFFTSize();
     
-    double y = v->getYForFrequency(freq, minf, maxf, logarithmic);
+    double y = v->getYForFrequency(freq, minf, maxf, m_frequencyMapping);
     
     return y;
 }
@@ -1237,9 +1300,7 @@ SpectrogramLayer::getBinForY(const LayerGeometryProvider *v, double y) const
     double minf = getEffectiveMinFrequency();
     double maxf = getEffectiveMaxFrequency();
 
-    bool logarithmic = (m_binScale == BinScale::Log);
-
-    double freq = v->getFrequencyForY(y, minf, maxf, logarithmic);
+    double freq = v->getFrequencyForY(y, minf, maxf, m_frequencyMapping);
 
     // Now map on to ("proportion of") actual bins
     double bin = (freq * getFFTSize()) / sr;
@@ -1559,7 +1620,7 @@ void
 SpectrogramLayer::invalidateMagnitudes()
 {
 #ifdef DEBUG_SPECTROGRAM
-    cerr << "SpectrogramLayer::invalidateMagnitudes called" << endl;
+    SVDEBUG << "SpectrogramLayer::invalidateMagnitudes called" << endl;
 #endif
     m_viewMags.clear();
 }
@@ -1622,8 +1683,7 @@ SpectrogramLayer::getRenderer(LayerGeometryProvider *v) const
         params.colourScale = ColourScale(cparams);
         params.normalization = m_normalization;
         params.binDisplay = m_binDisplay;
-        params.binScale = m_binScale;
-        params.alwaysOpaque = true;
+        params.frequencyMapping = m_frequencyMapping;
         params.invertVertical = false;
         params.scaleFactor = 1.0;
         params.colourRotation = m_colourRotation;
@@ -1633,10 +1693,8 @@ SpectrogramLayer::getRenderer(LayerGeometryProvider *v) const
             params.scaleFactor *= 2.f / float(getWindowSize());
         }
 
-        Preferences::SpectrogramSmoothing smoothing = 
-            Preferences::getInstance()->getSpectrogramSmoothing();
-        params.interpolate = 
-            (smoothing != Preferences::NoSpectrogramSmoothing);
+        params.opaque = m_opaque;
+        params.interpolate = m_smooth;
 
         m_renderers[viewId] = new Colour3DPlotRenderer(sources, params);
 
@@ -1672,7 +1730,7 @@ SpectrogramLayer::paintWithRenderer(LayerGeometryProvider *v, QPainter &paint, Q
         result = renderer->renderTimeConstrained(v, paint, rect);
 
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-        cerr << "rect width from this paint: " << result.rendered.width()
+        SVDEBUG << "rect width from this paint: " << result.rendered.width()
              << ", mag range in this paint: " << result.range.getMin() << " -> "
              << result.range.getMax() << endl;
 #endif
@@ -1689,7 +1747,7 @@ SpectrogramLayer::paintWithRenderer(LayerGeometryProvider *v, QPainter &paint, Q
         if (m_viewMags[viewId] != magRange) {
             m_viewMags[viewId] = magRange;
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-            cerr << "mag range in this view has changed: "
+            SVDEBUG << "mag range in this view has changed: "
                  << magRange.getMin() << " -> " << magRange.getMax() << endl;
 #endif
         }
@@ -1698,7 +1756,7 @@ SpectrogramLayer::paintWithRenderer(LayerGeometryProvider *v, QPainter &paint, Q
     if (!continuingPaint && m_normalizeVisibleArea &&
         m_viewMags[viewId] != m_lastRenderedMags[viewId]) {
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-        cerr << "mag range has changed from last rendered range: re-rendering"
+        SVDEBUG << "mag range has changed from last rendered range: re-rendering"
              << endl;
 #endif
         delete m_renderers[viewId];
@@ -1713,9 +1771,9 @@ SpectrogramLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) c
     Profiler profiler("SpectrogramLayer::paint", false);
 
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "SpectrogramLayer::paint() entering: m_model is " << m_model << ", zoom level is " << v->getZoomLevel() << endl;
+    SVDEBUG << "SpectrogramLayer::paint() entering: m_model is " << m_model << ", zoom level is " << v->getZoomLevel() << endl;
     
-    cerr << "SpectrogramLayer::paint(): rect is " << rect.x() << "," << rect.y() << " " << rect.width() << "x" << rect.height() << endl;
+    SVDEBUG << "SpectrogramLayer::paint(): rect is " << rect.x() << "," << rect.y() << " " << rect.width() << "x" << rect.height() << endl;
 #endif
 
     auto model = ModelById::getAs<DenseTimeValueModel>(m_model);
@@ -1741,7 +1799,7 @@ SpectrogramLayer::illuminateLocalFeatures(LayerGeometryProvider *v, QPainter &pa
     }
 
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "SpectrogramLayer: illuminateLocalFeatures("
+    SVDEBUG << "SpectrogramLayer: illuminateLocalFeatures("
               << localPos.x() << "," << localPos.y() << ")" << endl;
 #endif
 
@@ -1761,7 +1819,7 @@ SpectrogramLayer::illuminateLocalFeatures(LayerGeometryProvider *v, QPainter &pa
         int y0 = int(getYForFrequency(v, f0));
         
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-        cerr << "SpectrogramLayer: illuminate "
+        SVDEBUG << "SpectrogramLayer: illuminate "
                   << x0 << "," << y1 << " -> " << x1 << "," << y0 << endl;
 #endif
         
@@ -1779,7 +1837,7 @@ SpectrogramLayer::getYForFrequency(const LayerGeometryProvider *v, double freque
     return v->getYForFrequency(frequency,
                                getEffectiveMinFrequency(),
                                getEffectiveMaxFrequency(),
-                               m_binScale == BinScale::Log);
+                               m_frequencyMapping);
 }
 
 double
@@ -1788,7 +1846,7 @@ SpectrogramLayer::getFrequencyForY(const LayerGeometryProvider *v, int y) const
     return v->getFrequencyForY(y,
                                getEffectiveMinFrequency(),
                                getEffectiveMaxFrequency(),
-                               m_binScale == BinScale::Log);
+                               m_frequencyMapping);
 }
 
 int
@@ -1798,7 +1856,7 @@ SpectrogramLayer::getCompletion(LayerGeometryProvider *) const
     if (!fftModel) return 100;
     int completion = fftModel->getCompletion();
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "SpectrogramLayer::getCompletion: completion = " << completion << endl;
+    SVDEBUG << "SpectrogramLayer::getCompletion: completion = " << completion << endl;
 #endif
     return completion;
 }
@@ -1811,20 +1869,40 @@ SpectrogramLayer::getError(LayerGeometryProvider *) const
     return fftModel->getError();
 }
 
-bool
-SpectrogramLayer::getValueExtents(double &min, double &max,
-                                  bool &logarithmic, QString &unit) const
+Layer::ScaleExtents
+SpectrogramLayer::getVerticalExtents() const
 {
     auto model = ModelById::getAs<DenseTimeValueModel>(m_model);
-    if (!model) return false;
+    if (!model) return NO_VERTICAL_EXTENTS;
 
     sv_samplerate_t sr = model->getSampleRate();
-    min = double(sr) / getFFTSize();
-    max = double(sr) / 2;
-    
-    logarithmic = (m_binScale == BinScale::Log);
-    unit = "Hz";
-    return true;
+    double min = 0.0;
+    double max = double(sr) / 2.0;
+    CoordinateScale::FrequencyMap map;
+    switch (m_frequencyMapping) {
+        //!!! merge FrequencyMapping (in LayerGeometryProvider) with
+        //!!! CoordinateScale::FrequencyMap... I don't really want
+        //!!! either to refer to the other for this, so put it
+        //!!! somewhere else ?
+    case FrequencyMapping::Log:
+        map = CoordinateScale::FrequencyMap::Log;
+        break;
+    case FrequencyMapping::Mel:
+        map = CoordinateScale::FrequencyMap::Mel;
+        break;
+    case FrequencyMapping::Linear:
+    default:
+        map = CoordinateScale::FrequencyMap::Linear;
+        break;
+    }
+    CoordinateScale scale(CoordinateScale::Direction::Vertical, map, min, max);
+    scale = scale.withDisplayExtents(getEffectiveMinFrequency(),
+                                     getEffectiveMaxFrequency());
+
+    return {
+        Layer::ScaleApplication::Normal,
+        scale
+    };
 }
 
 bool
@@ -1926,11 +2004,6 @@ SpectrogramLayer::getCrosshairExtents(LayerGeometryProvider *v, QPainter &paint,
                                       QPoint cursorPos,
                                       vector<QRect> &extents) const
 {
-    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-    // replacement (horizontalAdvance) was only added in Qt 5.11
-    // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
     QRect vertical(cursorPos.x() - 12, 0, 12, v->getPaintHeight());
     extents.push_back(vertical);
 
@@ -1940,22 +2013,22 @@ SpectrogramLayer::getCrosshairExtents(LayerGeometryProvider *v, QPainter &paint,
     int sw = getVerticalScaleWidth(v, m_haveDetailedScale, paint);
 
     QRect freq(sw, cursorPos.y() - paint.fontMetrics().ascent() - 2,
-               paint.fontMetrics().width("123456 Hz") + 2,
+               paint.fontMetrics().horizontalAdvance("123456 Hz") + 2,
                paint.fontMetrics().height());
     extents.push_back(freq);
 
     QRect pitch(sw, cursorPos.y() + 2,
-                paint.fontMetrics().width("C#10+50c") + 2,
+                paint.fontMetrics().horizontalAdvance("C#10+50c") + 2,
                 paint.fontMetrics().height());
     extents.push_back(pitch);
 
     QRect rt(cursorPos.x(),
              v->getPaintHeight() - paint.fontMetrics().height() - 2,
-             paint.fontMetrics().width("1234.567 s"),
+             paint.fontMetrics().horizontalAdvance("1234.567 s"),
              paint.fontMetrics().height());
     extents.push_back(rt);
 
-    int w(paint.fontMetrics().width("1234567890") + 2);
+    int w(paint.fontMetrics().horizontalAdvance("1234567890") + 2);
     QRect frame(cursorPos.x() - w - 2,
                 v->getPaintHeight() - paint.fontMetrics().height() - 2,
                 w,
@@ -2011,7 +2084,7 @@ SpectrogramLayer::paintCrosshairs(LayerGeometryProvider *v, QPainter &paint,
     QString frameLabel = QString("%1").arg(frame);
     PaintAssistant::drawVisibleText
         (v, paint,
-         cursorPos.x() - paint.fontMetrics().width(frameLabel) - 2,
+         cursorPos.x() - paint.fontMetrics().horizontalAdvance(frameLabel) - 2,
          v->getPaintHeight() - 2,
          frameLabel,
          PaintAssistant::OutlinedText);
@@ -2168,7 +2241,7 @@ SpectrogramLayer::getColourScaleWidth(QPainter &paint) const
 {
     int cw;
 
-    cw = paint.fontMetrics().width("-80dB");
+    cw = paint.fontMetrics().horizontalAdvance("-80dB");
 
     return cw;
 }
@@ -2182,15 +2255,15 @@ SpectrogramLayer::getVerticalScaleWidth(LayerGeometryProvider *, bool detailed,
     int cw = 0;
     if (detailed) cw = getColourScaleWidth(paint);
 
-    int tw = paint.fontMetrics().width(QString("%1")
+    int tw = paint.fontMetrics().horizontalAdvance(QString("%1")
                                      .arg(m_maxFrequency > 0 ?
                                           m_maxFrequency - 1 :
                                           model->getSampleRate() / 2));
 
-    int fw = paint.fontMetrics().width(tr("43Hz"));
+    int fw = paint.fontMetrics().horizontalAdvance(tr("43Hz"));
     if (tw < fw) tw = fw;
 
-    int tickw = (m_binScale == BinScale::Log ? 10 : 4);
+    int tickw = (m_frequencyMapping != FrequencyMapping::Linear ? 10 : 4);
     
     return cw + tickw + tw + 13;
 }
@@ -2216,8 +2289,8 @@ SpectrogramLayer::paintVerticalScale(LayerGeometryProvider *v, bool detailed,
     }
     m_haveDetailedScale = detailed;
 
-    int tickw = (m_binScale == BinScale::Log ? 10 : 4);
-    int pkw = (m_binScale == BinScale::Log ? 10 : 0);
+    int tickw = (m_frequencyMapping != FrequencyMapping::Linear ? 10 : 4);
+    int pkw = (m_frequencyMapping != FrequencyMapping::Linear ? 10 : 0);
 
     int bins = getFFTSize() / 2;
     sv_samplerate_t sr = model->getSampleRate();
@@ -2254,7 +2327,7 @@ SpectrogramLayer::paintVerticalScale(LayerGeometryProvider *v, bool detailed,
         int freq = int((sr * bin) / getFFTSize());
 
         if (py >= 0 && (vy - py) < textHeight - 1) {
-            if (m_binScale == BinScale::Linear) {
+            if (m_frequencyMapping == FrequencyMapping::Linear) {
                 paint.drawLine(w - tickw, h - vy, w, h - vy);
             }
             continue;
@@ -2265,20 +2338,21 @@ SpectrogramLayer::paintVerticalScale(LayerGeometryProvider *v, bool detailed,
         paint.drawLine(cw + 7, h - vy, w - pkw - 1, h - vy);
 
         if (h - vy - textHeight >= -2) {
-            int tx = w - 3 - paint.fontMetrics().width(text) - max(tickw, pkw);
+            int tx = w - 3 - paint.fontMetrics().horizontalAdvance(text) - max(tickw, pkw);
             paint.drawText(tx, h - vy + toff, text);
         }
 
         py = vy;
     }
 
-    if (m_binScale == BinScale::Log) {
+    if (m_frequencyMapping != FrequencyMapping::Linear) {
 
         // piano keyboard
 
+        CoordinateScale scale = getVerticalExtents().second;
+    
         PianoScale().paintPianoVertical
-            (v, paint, QRect(w - pkw - 1, 0, pkw, h),
-             getEffectiveMinFrequency(), getEffectiveMaxFrequency());
+            (v, paint, QRect(w - pkw - 1, 0, pkw, h), scale);
     }
 
     m_haveDetailedScale = detailed;
@@ -2300,7 +2374,7 @@ SpectrogramLayer::paintDetailedScale(LayerGeometryProvider *v,
     int toff = -textHeight + paint.fontMetrics().ascent() + 2;
 
     int cw = getColourScaleWidth(paint);
-    int cbw = paint.fontMetrics().width("dB");
+    int cbw = paint.fontMetrics().horizontalAdvance("dB");
 
     int topLines = 2;
 
@@ -2319,7 +2393,7 @@ SpectrogramLayer::paintDetailedScale(LayerGeometryProvider *v,
     double dBmax = AudioLevel::voltage_to_dB(max);
 
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "paintVerticalScale: for view id " << v->getId()
+    SVDEBUG << "paintVerticalScale: for view id " << v->getId()
          << ": min = " << min << ", max = " << max
          << ", dBmin = " << dBmin << ", dBmax = " << dBmax << endl;
 #endif
@@ -2331,17 +2405,17 @@ SpectrogramLayer::paintDetailedScale(LayerGeometryProvider *v,
     bottom = QString("%1").arg(lrint(dBmin));
 
 #ifdef DEBUG_SPECTROGRAM_REPAINT
-    cerr << "adjusted dB range to min = " << dBmin << ", max = " << dBmax
+    SVDEBUG << "adjusted dB range to min = " << dBmin << ", max = " << dBmax
          << endl;
 #endif
         
-    paint.drawText((cw + 6 - paint.fontMetrics().width("dB")) / 2,
+    paint.drawText((cw + 6 - paint.fontMetrics().horizontalAdvance("dB")) / 2,
                    2 + textHeight + toff, "dB");
 
-    paint.drawText(3 + cw - cbw - paint.fontMetrics().width(top),
+    paint.drawText(3 + cw - cbw - paint.fontMetrics().horizontalAdvance(top),
                    2 + textHeight * topLines + toff + textHeight/2, top);
 
-    paint.drawText(3 + cw - cbw - paint.fontMetrics().width(bottom),
+    paint.drawText(3 + cw - cbw - paint.fontMetrics().horizontalAdvance(bottom),
                    h + toff - 3 - textHeight/2, bottom);
 
     paint.save();
@@ -2373,7 +2447,7 @@ SpectrogramLayer::paintDetailedScale(LayerGeometryProvider *v,
                      idb % 5 == 0))) {
             paint.setPen(v->getForeground());
             QString text = QString("%1").arg(idb);
-            paint.drawText(3 + cw - cbw - paint.fontMetrics().width(text),
+            paint.drawText(3 + cw - cbw - paint.fontMetrics().horizontalAdvance(text),
                            y + toff + textHeight/2, text);
             paint.drawLine(5 + cw - cbw, y, 8 + cw - cbw, y);
             lasty = y;
@@ -2397,7 +2471,7 @@ SpectrogramLayer::paintDetailedScalePhase(LayerGeometryProvider *v,
 
     // Phase is not measured in dB of course, but this places the
     // scale at the same position as in the magnitude spectrogram
-    int cbw = paint.fontMetrics().width("dB");
+    int cbw = paint.fontMetrics().horizontalAdvance("dB");
 
     int topLines = 1;
 
@@ -2409,13 +2483,13 @@ SpectrogramLayer::paintDetailedScalePhase(LayerGeometryProvider *v,
     double min = -M_PI;
     double max =  M_PI;
 
-    paint.drawText(3 + cw - cbw - paint.fontMetrics().width(top),
+    paint.drawText(3 + cw - cbw - paint.fontMetrics().horizontalAdvance(top),
                    2 + textHeight * topLines + toff + textHeight/2, top);
 
-    paint.drawText(3 + cw - cbw - paint.fontMetrics().width(middle),
+    paint.drawText(3 + cw - cbw - paint.fontMetrics().horizontalAdvance(middle),
                    2 + textHeight * topLines + ch/2 + toff + textHeight/2, middle);
 
-    paint.drawText(3 + cw - cbw - paint.fontMetrics().width(bottom),
+    paint.drawText(3 + cw - cbw - paint.fontMetrics().horizontalAdvance(bottom),
                    h + toff - 3 - textHeight/2, bottom);
 
     paint.save();
@@ -2535,7 +2609,7 @@ SpectrogramLayer::setVerticalZoomStep(int step)
     double dmin = m_minFrequency, dmax = m_maxFrequency;
 //    getDisplayExtents(dmin, dmax);
 
-//    cerr << "current range " << dmin << " -> " << dmax << ", range " << dmax-dmin << ", mid " << (dmax + dmin)/2 << endl;
+//    SVDEBUG << "current range " << dmin << " -> " << dmax << ", range " << dmax-dmin << ", mid " << (dmax + dmin)/2 << endl;
     
     sv_samplerate_t sr = model->getSampleRate();
     SpectrogramRangeMapper mapper(sr, getFFTSize());
@@ -2543,7 +2617,7 @@ SpectrogramLayer::setVerticalZoomStep(int step)
 
     double newmin, newmax;
 
-    if (m_binScale == BinScale::Log) {
+    if (m_frequencyMapping != FrequencyMapping::Linear) {
 
         // need to pick newmin and newmax such that
         //
@@ -2569,7 +2643,7 @@ SpectrogramLayer::setVerticalZoomStep(int step)
         newmax = (newdist + sqrt(newdist*newdist + 4*dmin*dmax)) / 2;
         newmin = newmax - newdist;
 
-//        cerr << "newmin = " << newmin << ", newmax = " << newmax << endl;
+//        SVDEBUG << "newmin = " << newmin << ", newmax = " << newmax << endl;
 
     } else {
         double dmid = (dmax + dmin) / 2;
@@ -2653,14 +2727,20 @@ SpectrogramLayer::toXml(QTextStream &stream,
                  "maxFrequency=\"%2\" "
                  "colourScale=\"%3\" "
                  "colourRotation=\"%4\" "
-                 "frequencyScale=\"%5\" "
-                 "binDisplay=\"%6\" ")
+                 "frequencyScale=\"%5\" " // Compatibility property only
+                 "frequencyMapping=\"%6\" " // Replacement for the above
+                 "binDisplay=\"%7\" "
+                 "smooth=\"%8\" "
+                 "opaque=\"%9\" ")
         .arg(m_minFrequency)
         .arg(m_maxFrequency)
         .arg(convertFromColourScale(m_colourScale, m_colourScaleMultiple))
         .arg(m_colourRotation)
-        .arg(int(m_binScale))
-        .arg(int(m_binDisplay));
+        .arg(m_frequencyMapping == FrequencyMapping::Linear ? 0 : 1)
+        .arg(int(m_frequencyMapping))
+        .arg(int(m_binDisplay))
+        .arg(m_smooth ? "true" : "false")
+        .arg(m_opaque ? "true" : "false");
 
     // New-style colour map attribute, by string id rather than by
     // number
@@ -2695,12 +2775,12 @@ SpectrogramLayer::toXml(QTextStream &stream,
     
     s += QString("normalizeVisibleArea=\"%1\" ")
         .arg(m_normalizeVisibleArea ? "true" : "false");
-    
+
     Layer::toXml(stream, indent, extraAttributes + " " + s);
 }
 
 void
-SpectrogramLayer::setProperties(const QXmlAttributes &attributes)
+SpectrogramLayer::setProperties(const LayerAttributes &attributes)
 {
     bool ok = false;
 
@@ -2766,9 +2846,15 @@ SpectrogramLayer::setProperties(const QXmlAttributes &attributes)
     int colourRotation = attributes.value("colourRotation").toInt(&ok);
     if (ok) setColourRotation(colourRotation);
 
-    BinScale binScale = (BinScale)
-        attributes.value("frequencyScale").toInt(&ok);
-    if (ok) setBinScale(binScale);
+    if (attributes.contains("frequencyMapping")) {
+        FrequencyMapping frequencyMapping = (FrequencyMapping)
+            attributes.value("frequencyMapping").toInt(&ok);
+        if (ok) setFrequencyMapping(frequencyMapping);
+    } else {
+        int binScale = attributes.value("frequencyScale").toInt(&ok);
+        if (ok) setFrequencyMapping(binScale == 0 ? FrequencyMapping::Linear
+                                    : FrequencyMapping::Log);
+    }
 
     BinDisplay binDisplay = (BinDisplay)
         attributes.value("binDisplay").toInt(&ok);
@@ -2789,7 +2875,7 @@ SpectrogramLayer::setProperties(const QXmlAttributes &attributes)
         } else if (columnNormalization == "none") {
             setNormalization(ColumnNormalization::None);
         } else {
-            SVCERR << "NOTE: Unknown or unsupported columnNormalization attribute \""
+            SVDEBUG << "NOTE: Unknown or unsupported columnNormalization attribute \""
                  << columnNormalization << "\"" << endl;
         }
     }
@@ -2821,5 +2907,19 @@ SpectrogramLayer::setProperties(const QXmlAttributes &attributes)
         // gives us backward but not forward compatibility.
         setGain(m_gain / float(getFFTSize() / 2));
     }
+
+    if (attributes.contains("opaque")) {
+        bool opaque =
+            (attributes.value("opaque").trimmed() == "true");
+        setOpaque(opaque);
+    }
+
+    if (attributes.contains("smooth")) {
+        bool smooth =
+            (attributes.value("smooth").trimmed() == "true");
+        setSmooth(smooth);
+    }
 }
     
+} // end namespace sv
+
