@@ -32,14 +32,13 @@
 
 #include <QMutex>
 #include <QWaitCondition>
-#include <QImage>
-#include <QPixmap>
+
+class QPainter;
+class QTimer;
+
+namespace sv {
 
 class View;
-class QPainter;
-class QImage;
-class QPixmap;
-class QTimer;
 class FFTModel;
 class Dense3DModelPeakCache;
 
@@ -176,10 +175,17 @@ public:
     double getColourScaleMultiple() const;
     
     /**
-     * Specify the scale for the y axis.
+     * Specify the scale for the y axis. This function can be used to
+     * choose log or linear only and is here for backward
+     * compatibility. Use setFrequencyMapping in new code.
      */
     void setBinScale(BinScale);
-    BinScale getBinScale() const;
+
+    /**
+     * Specify the scale mapping for the y axis.
+     */
+    void setFrequencyMapping(FrequencyMapping);
+    FrequencyMapping getFrequencyMapping() const;
 
     /**
      * Specify the processing of frequency bins for the y axis.
@@ -212,11 +218,17 @@ public:
     void setColourRotation(int);
     int getColourRotation() const;
 
+    void setOpaque(bool i);
+    bool getOpaque() const;
+
+    void setSmooth(bool i);
+    bool getSmooth() const;
+
     VerticalPosition getPreferredFrameCountPosition() const override {
         return PositionTop;
     }
 
-    bool isLayerOpaque() const override { return true; }
+    bool isLayerOpaque() const override { return m_opaque; }
 
     ColourSignificance getLayerColourSignificance() const override {
         return ColourHasMeaningfulValue;
@@ -232,9 +244,8 @@ public:
     int getCompletion(LayerGeometryProvider *v) const override;
     QString getError(LayerGeometryProvider *v) const override;
 
-    bool getValueExtents(double &min, double &max,
-                         bool &logarithmic, QString &unit) const override;
-
+    ScaleExtents getVerticalExtents() const override;
+    
     bool getDisplayExtents(double &min, double &max) const override;
 
     bool setDisplayExtents(double min, double max) override;
@@ -244,7 +255,7 @@ public:
     void toXml(QTextStream &stream, QString indent = "",
                        QString extraAttributes = "") const override;
 
-    void setProperties(const QXmlAttributes &attributes) override;
+    void setProperties(const LayerAttributes &attributes) override;
 
     void setLayerDormant(const LayerGeometryProvider *v, bool dormant) override;
 
@@ -286,10 +297,12 @@ protected:
     int                 m_colourMap;
     bool                m_colourInverted;
     mutable QColor      m_crosshairColour;
-    BinScale            m_binScale;
+    FrequencyMapping    m_frequencyMapping;
     BinDisplay          m_binDisplay;
     ColumnNormalization m_normalization; // of individual columns
     bool                m_normalizeVisibleArea;
+    bool                m_opaque;
+    bool                m_smooth;
     int                 m_lastEmittedZoomStep;
     bool                m_synchronous;
 
@@ -366,4 +379,6 @@ protected:
                                       MeasureRect &r, bool start, int y) const override;
 };
 
+} // end namespace sv
+
 #endif
