@@ -39,13 +39,17 @@
 
 //#define DEBUG_TIME_INSTANT_LAYER 1
 
+namespace sv {
+
 TimeInstantLayer::TimeInstantLayer() :
     SingleColourLayer(),
     m_editing(false),
     m_editingPoint(0, tr("New Point")),
     m_editingCommand(nullptr),
     m_plotStyle(PlotInstants),
-    m_propertiesExplicitlySet(false)
+    m_propertiesExplicitlySet(false),
+    m_overrideHighlight(false),
+    m_highlightOverrideFrame(0)
 {
 }
 
@@ -160,6 +164,21 @@ TimeInstantLayer::setPlotStyle(PlotStyle style)
     emit layerParametersChanged();
 }
 
+void
+TimeInstantLayer::overrideHighlightForPointsAt(sv_frame_t frame)
+{
+    m_highlightOverrideFrame = frame;
+    m_overrideHighlight = true;
+    emit layerParametersChanged();
+}
+
+void
+TimeInstantLayer::removeOverrideHighlight()
+{
+    m_overrideHighlight = false;
+    emit layerParametersChanged();
+}
+
 bool
 TimeInstantLayer::needsTextLabelHeight() const
 {
@@ -223,14 +242,14 @@ TimeInstantLayer::getLocalPoints(LayerGeometryProvider *v, int x) const
 }
 
 QString
-TimeInstantLayer::getLabelPreceding(sv_frame_t frame) const
+TimeInstantLayer::getLabelAtOrPreceding(sv_frame_t frame) const
 {
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model || !model->hasTextLabels()) return "";
 
     Event e;
     if (model->getNearestEventMatching
-        (frame,
+        (frame + 1, // Backward search does not include the given frame
          [](Event e) { return e.hasLabel() && e.getLabel() != ""; },
          EventSeries::Backward,
          e)) {
@@ -388,7 +407,14 @@ TimeInstantLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) c
     QPoint localPos;
     sv_frame_t illuminateFrame = -1;
 
-    if (v->shouldIlluminateLocalFeatures(this, localPos)) {
+    if (m_overrideHighlight) {
+
+        illuminateFrame = m_highlightOverrideFrame;
+#ifdef DEBUG_TIME_INSTANT_LAYER
+        cerr << "TimeInstantLayer: using highlight override frame " << illuminateFrame << endl;
+#endif
+
+    } else if (v->shouldIlluminateLocalFeatures(this, localPos)) {
         EventVector localPoints = getLocalPoints(v, localPos.x());
         if (!localPoints.empty()) {
             illuminateFrame = localPoints.begin()->getFrame();
@@ -402,6 +428,8 @@ TimeInstantLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) c
     paint.setClipRect(rect);
     paint.setClipping(clippingRequired);
     
+    bool illuminated = false;
+    
     for (EventVector::const_iterator i = points.begin();
          i != points.end(); ++i) {
 
@@ -414,7 +442,7 @@ TimeInstantLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) c
 #ifdef DEBUG_TIME_INSTANT_LAYER
         SVCERR << "point frame = " << p.getFrame() << " -> x = " << x << endl;
 #endif
-        
+
         if (x == prevX && m_plotStyle == PlotInstants &&
             p.getFrame() != illuminateFrame) {
 #ifdef DEBUG_TIME_INSTANT_LAYER
@@ -438,6 +466,7 @@ TimeInstantLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) c
                 
         if (p.getFrame() == illuminateFrame) {
             paint.setPen(getForegroundQColor(v->getView()));
+            illuminated = true;
         } else {
             paint.setPen(brushColour);
         }
@@ -518,19 +547,24 @@ TimeInstantLayer::paint(LayerGeometryProvider *v, QPainter &paint, QRect rect) c
 
         prevX = x;
     }
+    
+    if (illuminated) {
+        // emit (but with const cast)
+        const_cast<TimeInstantLayer *>(this)->frameIlluminated(illuminateFrame);
+    }
 }
 
 void
 TimeInstantLayer::drawStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
 #ifdef DEBUG_TIME_INSTANT_LAYER
-    cerr << "TimeInstantLayer::drawStart(" << e->x() << ")" << endl;
+    cerr << "TimeInstantLayer::drawStart(" << e->position().x() << ")" << endl;
 #endif
 
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
@@ -547,13 +581,13 @@ void
 TimeInstantLayer::drawDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
 #ifdef DEBUG_TIME_INSTANT_LAYER
-    cerr << "TimeInstantLayer::drawDrag(" << e->x() << ")" << endl;
+    cerr << "TimeInstantLayer::drawDrag(" << e->position().x() << ")" << endl;
 #endif
 
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
     m_editingCommand->remove(m_editingPoint);
@@ -569,7 +603,7 @@ TimeInstantLayer::drawEnd(LayerGeometryProvider *, QMouseEvent *
     )
 {
 #ifdef DEBUG_TIME_INSTANT_LAYER
-    cerr << "TimeInstantLayer::drawEnd(" << e->x() << ")" << endl;
+    cerr << "TimeInstantLayer::drawEnd(" << e->position().x() << ")" << endl;
 #endif
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model || !m_editing) return;
@@ -589,7 +623,7 @@ TimeInstantLayer::eraseStart(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model) return;
 
-    EventVector points = getLocalPoints(v, e->x());
+    EventVector points = getLocalPoints(v, e->position().x());
     if (points.empty()) return;
 
     m_editingPoint = *points.begin();
@@ -615,7 +649,7 @@ TimeInstantLayer::eraseEnd(LayerGeometryProvider *v, QMouseEvent *e)
 
     m_editing = false;
 
-    EventVector points = getLocalPoints(v, e->x());
+    EventVector points = getLocalPoints(v, e->position().x());
     if (points.empty()) return;
     if (points.begin()->getFrame() != m_editingPoint.getFrame()) return;
 
@@ -630,13 +664,13 @@ void
 TimeInstantLayer::editStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
 #ifdef DEBUG_TIME_INSTANT_LAYER
-    cerr << "TimeInstantLayer::editStart(" << e->x() << ")" << endl;
+    cerr << "TimeInstantLayer::editStart(" << e->position().x() << ")" << endl;
 #endif
 
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model) return;
 
-    EventVector points = getLocalPoints(v, e->x());
+    EventVector points = getLocalPoints(v, e->position().x());
     if (points.empty()) return;
 
     m_editingPoint = *points.begin();
@@ -653,13 +687,13 @@ void
 TimeInstantLayer::editDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
 #ifdef DEBUG_TIME_INSTANT_LAYER
-    cerr << "TimeInstantLayer::editDrag(" << e->x() << ")" << endl;
+    cerr << "TimeInstantLayer::editDrag(" << e->position().x() << ")" << endl;
 #endif
 
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model || !m_editing) return;
 
-    sv_frame_t frame = v->getFrameForX(e->x());
+    sv_frame_t frame = v->getFrameForX(e->position().x());
     if (frame < 0) frame = 0;
     frame = frame / model->getResolution() * model->getResolution();
 
@@ -680,7 +714,7 @@ TimeInstantLayer::editEnd(LayerGeometryProvider *, QMouseEvent *
     )
 {
 #ifdef DEBUG_TIME_INSTANT_LAYER
-    cerr << "TimeInstantLayer::editEnd(" << e->x() << ")" << endl;
+    cerr << "TimeInstantLayer::editEnd(" << e->position().x() << ")" << endl;
 #endif
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model || !m_editing) return;
@@ -702,7 +736,7 @@ TimeInstantLayer::editOpen(LayerGeometryProvider *v, QMouseEvent *e)
     auto model = ModelById::getAs<SparseOneDimensionalModel>(m_model);
     if (!model) return false;
 
-    EventVector points = getLocalPoints(v, e->x());
+    EventVector points = getLocalPoints(v, e->position().x());
     if (points.empty()) return false;
 
     Event point = *points.begin();
@@ -904,7 +938,7 @@ TimeInstantLayer::toXml(QTextStream &stream,
 }
 
 void
-TimeInstantLayer::setProperties(const QXmlAttributes &attributes)
+TimeInstantLayer::setProperties(const LayerAttributes &attributes)
 {
     SingleColourLayer::setProperties(attributes);
 
@@ -916,3 +950,5 @@ TimeInstantLayer::setProperties(const QXmlAttributes &attributes)
     m_propertiesExplicitlySet = true;
 }
 
+} // end namespace sv
+
