@@ -38,6 +38,8 @@
 //#define DEBUG_WAVEFORM_PAINT 1
 //#define DEBUG_WAVEFORM_PAINT_BY_PIXEL 1
 
+namespace sv {
+
 using std::vector;
 
 double
@@ -375,47 +377,39 @@ WaveformLayer::getCompletion(LayerGeometryProvider *) const
     return completion;
 }
 
-bool
-WaveformLayer::getValueExtents(double &min, double &max,
-                               bool &log, QString &unit) const
+Layer::ScaleExtents
+WaveformLayer::getVerticalExtents() const
 {
-    // This function serves two purposes. It's used to gather min and
-    // max values for a given unit, for cases where there are
-    // auto-align layers out there that aren't providing extents of
-    // their own and that have no specific other layer with display
-    // extents to align to. It's also used to determine whether a
-    // layer might be capable of drawing a scale for itself.
-    //
-    // This makes our situation a bit tricky. There's no point in
-    // returning extents that anyone else might try to align to unless
-    // we have a scale that they can actually calculate with, which is
-    // only the case for certain linear/log arrangements (see
-    // getDisplayExtents - we can test this case by checking whether
-    // getDisplayExtents returns successfully).
-    //
-    // However, there is a point in returning something that indicates
-    // our own capacity to draw a scale. If we don't do that, then we
-    // won't get a scale at all if e.g. we have a time-instant layer
-    // on top (or something else that doesn't care about the y axis).
-    //
-    // Our "solution" to this is to always return true and our
-    // extents, but with an empty unit unless we have the sort of nice
-    // linear/log scale that others can actually align to.
-    //
-    // It might be better to respond to capability requests - can draw
-    // scale, care about scale, can align unit X etc.
+    CoordinateScale scale(CoordinateScale::Direction::Vertical,
+                          "V", m_scale == dBScale, -1.0, 1.0);
+
+    bool alignable = true;
     
-    if (getDisplayExtents(min, max)) {
-        unit = "V";
-        log = (m_scale == dBScale);
-    } else {
-        max = 1.0;
-        min = -1.0;
-        log = false;
-        unit = "";
+    if (m_scale == MeterScale) {
+        alignable = false;
+    }
+    
+    if (m_channelCount > 1) {
+        if (m_channelMode == SeparateChannels) {
+            alignable = false;
+        }
+        if (m_channelMode == MergeChannels && m_scale != LinearScale) {
+            alignable = false;
+        }
     }
 
-    return true;
+    if (alignable && m_scale == dBScale) {
+        scale = scale.withDisplayExtents
+            (AudioLevel::dB_to_voltage(m_dBMin), 1.0);
+    }
+
+    //!!! display extents not set based on gain (but then
+    //!!! getDisplayExtents doesn't handle it either?)
+
+    return {
+        alignable ? ScaleApplication::Normal : ScaleApplication::Personal,
+        scale
+    };
 }
 
 bool
@@ -543,7 +537,7 @@ WaveformLayer::getSourceFramesForX(LayerGeometryProvider *v,
     f0 = f0 / modelZoomLevel;
     f0 = f0 * modelZoomLevel;
 
-    if (v->getZoomLevel().zone == ZoomLevel::PixelsPerFrame) {
+    if (v->getRoundedZoomLevel().zone == ZoomLevel::PixelsPerFrame) {
         f1 = f0 + 1;
     } else {
         viewFrame = v->getFrameForX(x + 1);
@@ -608,7 +602,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
         return;
     }
   
-    ZoomLevel zoomLevel = v->getZoomLevel();
+    ZoomLevel zoomLevel = v->getRoundedZoomLevel();
 
 #ifdef DEBUG_WAVEFORM_PAINT
     Profiler profiler("WaveformLayer::paint", true);
@@ -733,7 +727,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
 
     RangeVec ranges;
 
-    if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel) {
+    if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
         getSummaryRanges(minChannel, maxChannel,
                          mixingChannels || mergingChannels,
                          frame0, frame1,
@@ -742,7 +736,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
         getOversampledRanges(minChannel, maxChannel,
                              mixingChannels || mergingChannels,
                              frame0, frame1,
-                             v->getZoomLevel().level, ranges);
+                             zoomLevel.level, ranges);
     }
 
     if (!ranges.empty()) {
@@ -952,6 +946,8 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 
     bool firstPoint = true;
     double prevRangeBottom = 0, prevRangeTop = 0;
+
+    auto zoomLevel = v->getRoundedZoomLevel();
     
     for (int x = x0; x <= x1; ++x) {
 
@@ -960,7 +956,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 
         bool showIndividualSample = false;
         
-        if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel) {
+        if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
             if (!getSourceFramesForX(v, x, blockSize, f0, f1)) {
                 continue;
             }
@@ -968,7 +964,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
             i0 = (f0 - frame0) / blockSize;
             i1 = (f1 - frame0) / blockSize;
         } else {
-            int oversampleBy = v->getZoomLevel().level;
+            int oversampleBy = zoomLevel.level;
             f0 = f1 = v->getFrameForX(x);
             int xf0 = v->getXForFrame(f0);
             showIndividualSample = (x == xf0);
@@ -986,7 +982,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 #endif
 
         if (i1 > i0 + 1) {
-            SVCERR << "WaveformLayer::paint: ERROR: i1 " << i1 << " > i0 " << i0 << " plus one (zoom = " << v->getZoomLevel() << ", model zoom = " << blockSize << ")" << endl;
+            SVCERR << "WaveformLayer::paint: ERROR: i1 " << i1 << " > i0 " << i0 << " plus one (zoom = " << zoomLevel << ", model zoom = " << blockSize << ")" << endl;
         }
 
         const auto &r = ranges[rangeix];
@@ -1172,7 +1168,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
     }
 
     double penWidth = 1.0;
-    if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel) {
+    if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
         penWidth = 0.0;
     }
     
@@ -1182,7 +1178,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
         paint->setPen(QPen(midColour, penWidth));
     }
 
-    if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel ||
+    if (zoomLevel.zone == ZoomLevel::FramesPerPixel ||
         m_oversampling) {
         
         paint->drawPath(waveformPath);
@@ -1205,8 +1201,8 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
     
     if (!individualSamplePoints.empty()) {
         double sz = v->scaleSize(2.0);
-        if (v->getZoomLevel().zone == ZoomLevel::PixelsPerFrame) {
-            if (v->getZoomLevel().level < 10) {
+        if (zoomLevel.zone == ZoomLevel::PixelsPerFrame) {
+            if (zoomLevel.level < 10) {
                 sz = v->scaleSize(1.2);
             }
         }
@@ -1287,7 +1283,7 @@ WaveformLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) cons
     auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
     if (!model || !model->isOK()) return "";
 
-    ZoomLevel zoomLevel = v->getZoomLevel();
+    ZoomLevel zoomLevel = v->getRoundedZoomLevel();
 
     int desiredBlockSize = 1;
     if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
@@ -1521,11 +1517,6 @@ WaveformLayer::getYScaleDifference(const LayerGeometryProvider *v, int y0, int y
 int
 WaveformLayer::getVerticalScaleWidth(LayerGeometryProvider *, bool, QPainter &paint) const
 {
-    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-    // replacement (horizontalAdvance) was only added in Qt 5.11
-    // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
     if (m_scale == LinearScale) {
         QString sampleText = "0.0";
         if (m_gain != 1.f) {
@@ -1539,10 +1530,10 @@ WaveformLayer::getVerticalScaleWidth(LayerGeometryProvider *, bool, QPainter &pa
                 }
             }
         }
-        return paint.fontMetrics().width(sampleText) + 13;
+        return paint.fontMetrics().horizontalAdvance(sampleText) + 13;
     } else {
-        return std::max(paint.fontMetrics().width(tr("0dB")),
-                        paint.fontMetrics().width(Strings::minus_infinity)) + 13;
+        return std::max(paint.fontMetrics().horizontalAdvance(tr("0dB")),
+                        paint.fontMetrics().horizontalAdvance(Strings::minus_infinity)) + 13;
     }
 }
 
@@ -1648,7 +1639,7 @@ WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pain
 
                 int tx = 3;
                 if (m_scale != LinearScale) {
-                    tx = w - 10 - paint.fontMetrics().width(text);
+                    tx = w - 10 - paint.fontMetrics().horizontalAdvance(text);
                 }
                   
                 int ty = y;
@@ -1726,7 +1717,7 @@ WaveformLayer::toXml(QTextStream &stream,
 }
 
 void
-WaveformLayer::setProperties(const QXmlAttributes &attributes)
+WaveformLayer::setProperties(const LayerAttributes &attributes)
 {
     bool ok = false;
 
@@ -1788,3 +1779,5 @@ WaveformLayer::setVerticalZoomStep(int step)
     setGain(AudioLevel::dB_to_voltage(step - 50));
 }
 
+} // end namespace sv
+
