@@ -19,6 +19,7 @@
 #include "base/ZoomConstraint.h"
 #include "base/RealTime.h"
 #include "base/Profiler.h"
+#include "base/LogRange.h"
 #include "ViewManager.h"
 #include "widgets/CommandHistory.h"
 #include "widgets/TextAbbrev.h"
@@ -48,6 +49,7 @@
 #include <QMimeData>
 #include <QApplication>
 #include <QMenu>
+#include <QRegularExpression>
 
 #include <iostream>
 #include <cmath>
@@ -67,6 +69,8 @@
 //#define DEBUG_PANE 1
 //#define DEBUG_PANE_SCALE_CHOICE 1
 
+namespace sv {
+
 QCursor *Pane::m_measureCursor1 = nullptr;
 QCursor *Pane::m_measureCursor2 = nullptr;
 
@@ -82,6 +86,7 @@ Pane::Pane(QWidget *w) :
     m_editing(false),
     m_releasing(false),
     m_centreLineVisible(true),
+    m_selectionSnap(true),
     m_scaleWidth(0),
     m_pendingWheelAngle(0),
     m_headsUpDisplay(nullptr),
@@ -126,7 +131,7 @@ Pane::updateHeadsUpDisplay()
         m_headsUpDisplay = new QFrame(this);
 
         QGridLayout *layout = new QGridLayout;
-        layout->setMargin(0);
+        layout->setContentsMargins(0, 0, 0, 0);
         layout->setSpacing(0);
         m_headsUpDisplay->setLayout(layout);
         
@@ -196,7 +201,7 @@ Pane::updateHeadsUpDisplay()
     }
 
     int count = countZoomLevels();
-    int current = getZoomLevelIndex(getZoomLevel());
+    int current = getZoomLevelIndex(m_zoomLevel);
     
     m_hthumb->setMinimumValue(1);
     m_hthumb->setMaximumValue(count);
@@ -223,7 +228,7 @@ Pane::updateHeadsUpDisplay()
             m_vthumb->setValue(layer->getCurrentVerticalZoomStep());
             m_vthumb->blockSignals(false);
 
-//            cerr << "Vertical thumbwheel: min 0, max " << max
+//            SVCERR << "Vertical thumbwheel: min 0, max " << max
 //                      << ", default " << defaultStep << ", value "
 //                      << m_vthumb->getValue() << endl;
 
@@ -356,9 +361,16 @@ Pane::setCentreLineVisible(bool visible)
     update();
 }
 
+void
+Pane::setSelectionSnapToFeatures(bool snap)
+{
+    m_selectionSnap = snap;
+}
+
 void
 Pane::paintEvent(QPaintEvent *e)
 {
+//    SVDEBUG << "Pane[" << getId() << "]::paintEvent" << endl;
 //    Profiler profiler("Pane::paintEvent", true);
 
     QPainter paint;
@@ -517,7 +529,7 @@ Pane::paintEvent(QPaintEvent *e)
 
     if (toolMode == ViewManager::MeasureMode && topLayer) {
         bool showFocus = false;
-        if (!m_manager || !m_manager->isPlaying()) showFocus = true;
+        if (!isCurrentlyMovingWithPlayback()) showFocus = true;
         topLayer->paintMeasurementRects(this, paint, showFocus, m_identifyPoint);
     }
     
@@ -538,15 +550,13 @@ Pane::getVerticalScaleWidth() const
 void
 Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
 {
-    double min, max;
-    bool log;
-    QString unit;
-
     bool includeColourScale = m_manager->shouldShowVerticalColourScale();
     
     Layer *scaleLayer = nullptr;
     int scaleWidth = 0;
 
+    QString unit;
+
 #ifdef DEBUG_PANE_SCALE_CHOICE
         SVCERR << "Pane[" << getId() << "]::drawVerticalScale: Have "
                << getLayerCount() << " layer(s)" << endl;
@@ -579,7 +589,7 @@ Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
                << endl;
 #endif
     }
-
+    
     if (scaleWidth > 0) {
         scaleLayer = topLayer;
 
@@ -602,8 +612,12 @@ Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
                 continue;
             }
 
-            if (layer->getValueExtents(min, max, log, unit)) {
+            auto extents = layer->getVerticalExtents();
+
+            if (extents.first == Layer::ScaleApplication::Normal) {
+                
                 scaleLayer = layer;
+                unit = extents.second.getUnit();
 
 #ifdef DEBUG_PANE_SCALE_CHOICE
                 SVCERR << "Pane[" << getId() << "]::drawVerticalScale: "
@@ -613,6 +627,21 @@ Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
                        << unit << "), using this layer or unit" << endl;
 #endif
                 break;
+
+            } else if (extents.second.getUnit() != "" &&
+                       extents.first == Layer::ScaleApplication::Deferring) {
+
+#ifdef DEBUG_PANE_SCALE_CHOICE
+                SVCERR << "Pane[" << getId() << "]::drawVerticalScale: "
+                       << "Layer " << layer
+                       << ", " << layer->getLayerPresentationName()
+                       << " is deferring and we have no unit yet, adopt its unit (unit = "
+                       << unit << ")" << endl;
+#endif
+
+                if (unit == "") {
+                    unit = extents.second.getUnit();
+                }
             }
 
             if (layer->isLayerOpaque()) {
@@ -646,8 +675,9 @@ Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
                     << "but we have a unit of " << unit
                     << " - seeking scale-providing layer for that" << endl;
 #endif
-            
-            scaleLayer = getScaleProvidingLayerForUnit(unit);
+
+            (void)getEffectiveVerticalExtentsAndLayerFromWhich
+                (unit, &scaleLayer);
             
 #ifdef DEBUG_PANE_SCALE_CHOICE
             SVDEBUG << "Pane[" << getId()
@@ -700,14 +730,9 @@ Pane::drawFeatureDescription(Layer *topLayer, QPainter &paint)
     if (desc != "") {
         
         paint.save();
-        
-    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-    // replacement (horizontalAdvance) was only added in Qt 5.11
-    // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 
         int tabStop =
-            paint.fontMetrics().width(tr("Some lengthy prefix:"));
+            paint.fontMetrics().horizontalAdvance(tr("Some lengthy prefix:"));
         
         QRect boundingRect = 
             paint.fontMetrics().boundingRect
@@ -738,7 +763,7 @@ Pane::drawFeatureDescription(Layer *topLayer, QPainter &paint)
         QTextOption option;
         option.setWrapMode(QTextOption::NoWrap);
         option.setAlignment(Qt::AlignRight | Qt::AlignTop);
-        option.setTabStop(tabStop);
+        option.setTabStopDistance(tabStop);
         paint.drawText(QRectF(width() - boundingRect.width() - 10, 10,
                               boundingRect.width(),
                               boundingRect.height()),
@@ -809,7 +834,7 @@ Pane::drawCentreLine(sv_samplerate_t sampleRate, QPainter &paint, bool omitLine)
                           (m_centreFrame, sampleRate)
                           .toText(true)));
             
-            int tw = paint.fontMetrics().width(text);
+            int tw = paint.fontMetrics().horizontalAdvance(text);
             int x = width()/2 - 4 - tw;
             
             PaintAssistant::drawVisibleText(this, paint, x, y, text, PaintAssistant::OutlinedText);
@@ -874,11 +899,11 @@ Pane::drawAlignmentStatus(QRect r, QPainter &paint, ModelId modelId,
     ModelId reference = model->getAlignmentReference();
 /*
     if (!reference) {
-        cerr << "Pane[" << this << "]::drawAlignmentStatus: No reference" << endl;
+        SVCERR << "Pane[" << this << "]::drawAlignmentStatus: No reference" << endl;
     } else if (reference == model->getId()) {
-        cerr << "Pane[" << this << "]::drawAlignmentStatus: This is the reference model" << endl;
+        SVCERR << "Pane[" << this << "]::drawAlignmentStatus: This is the reference model" << endl;
     } else {
-        cerr << "Pane[" << this << "]::drawAlignmentStatus: This is not the reference" << endl;
+        SVCERR << "Pane[" << this << "]::drawAlignmentStatus: This is not the reference" << endl;
     }
 */
     QString text;
@@ -916,7 +941,7 @@ Pane::drawAlignmentStatus(QRect r, QPainter &paint, ModelId modelId,
 
     int y = 5;
     if (down) y += paint.fontMetrics().height();
-    int w = paint.fontMetrics().width(text);
+    int w = paint.fontMetrics().horizontalAdvance(text);
     int h = paint.fontMetrics().height();
     if (r.top() > h + y || r.left() > w + m_scaleWidth + 5) {
         paint.restore();
@@ -958,7 +983,7 @@ Pane::drawWorkTitle(QRect r, QPainter &paint, ModelId modelId)
     paint.setFont(font);
 
     int y = 5;
-    int w = paint.fontMetrics().width(text);
+    int w = paint.fontMetrics().horizontalAdvance(text);
     int h = paint.fontMetrics().height();
     if (r.top() > h + y || r.left() > w + m_scaleWidth + 5) {
         paint.restore();
@@ -1006,7 +1031,7 @@ Pane::drawLayerNames(QRect r, QPainter &paint)
     std::vector<QPixmap> pixmaps;
     for (LayerList::iterator i = m_layerStack.begin(); i != m_layerStack.end(); ++i) {
         texts.push_back((*i)->getLayerPresentationName());
-//        cerr << "Pane " << this << ": Layer presentation name for " << *i << ": "
+//        SVCERR << "Pane " << this << ": Layer presentation name for " << *i << ": "
 //                  << texts[texts.size()-1] << endl;
         pixmaps.push_back((*i)->getLayerPresentationPixmap
                           (QSize(fontAscent, fontAscent)));
@@ -1025,7 +1050,7 @@ Pane::drawLayerNames(QRect r, QPainter &paint)
     
         for (int i = 0; i < texts.size(); ++i) {
 
-//            cerr << "Pane "<< this << ": text " << i << ": " << texts[i] << endl;
+//            SVCERR << "Pane "<< this << ": text " << i << ": " << texts[i] << endl;
             
             if (i + 1 == texts.size()) {
                 paint.setPen(getForeground());
@@ -1159,7 +1184,7 @@ Pane::drawDurationAndRate(QRect r, ModelId waveformModelId,
     int pbw = getProgressBarWidth();
     if (x < pbw + 5) x = pbw + 5;
 
-    if (r.x() < x + paint.fontMetrics().width(desc)) {
+    if (r.x() < x + paint.fontMetrics().horizontalAdvance(desc)) {
         PaintAssistant::drawVisibleText(this, paint, x,
                         height() - fontHeight + fontAscent - 6,
                         desc, PaintAssistant::OutlinedText);
@@ -1200,8 +1225,8 @@ Pane::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
 QImage *
 Pane::renderPartToNewImage(sv_frame_t f0, sv_frame_t f1)
 {
-    int x0 = int(round(getZoomLevel().framesToPixels(double(f0))));
-    int x1 = int(round(getZoomLevel().framesToPixels(double(f1))));
+    int x0 = int(round(m_zoomLevel.framesToPixels(double(f0))));
+    int x1 = int(round(m_zoomLevel.framesToPixels(double(f1))));
 
     QImage *image = new QImage(x1 - x0 + m_scaleWidth,
                                height(), QImage::Format_RGB32);
@@ -1315,12 +1340,14 @@ Pane::getTopLayerDisplayExtents(double &vmin, double &vmax,
 {
     Layer *layer = getTopLayer();
     if (!layer) return false;
-    bool vlog;
-    QString vunit;
-    bool rv = (layer->getValueExtents(vmin, vmax, vlog, vunit) &&
-               layer->getDisplayExtents(dmin, dmax));
-    if (unit) *unit = vunit;
-    return rv;
+    auto extents = layer->getVerticalExtents();
+    auto scale = extents.second;
+    vmin = scale.getValueMinimum();
+    vmax = scale.getValueMaximum();
+    dmin = scale.getDisplayMinimum();
+    dmax = scale.getDisplayMaximum();
+    if (unit) *unit = scale.getUnit();
+    return (extents.first == Layer::ScaleApplication::Normal);
 }
 
 bool
@@ -1335,25 +1362,37 @@ void
 Pane::registerShortcuts(KeyReference &kr)
 {
     kr.setCategory(tr("Zoom"));
-    kr.registerAlternativeShortcut(tr("Zoom In"), tr("Wheel Up"));
-    kr.registerAlternativeShortcut(tr("Zoom Out"), tr("Wheel Down"));
+    kr.registerAlternativeMouseAction(tr("Zoom In"), tr("Wheel Up"));
+    kr.registerAlternativeMouseAction(tr("Zoom Out"), tr("Wheel Down"));
 
     kr.setCategory(tr("General Pane Mouse Actions"));
     
-    kr.registerShortcut(tr("Zoom"), tr("Wheel"),
-                        tr("Zoom in or out in time axis"));
-    kr.registerShortcut(tr("Scroll"), tr("Ctrl+Wheel"),
-                        tr("Scroll rapidly left or right in time axis"));
-    kr.registerShortcut(tr("Scroll Vertically"), tr("Shift+Wheel"), 
-                        tr("Scroll up or down in the vertical axis"));
-    kr.registerShortcut(tr("Zoom Vertically"), tr("Alt+Wheel"), 
-                        tr("Zoom in or out in the vertical axis"));
-    kr.registerShortcut(tr("Navigate"), tr("Middle"), 
-                        tr("Click middle button and drag to navigate with any tool"));
-    kr.registerShortcut(tr("Relocate"), tr("Double-Click Middle"), 
-                        tr("Double-click middle button to relocate with any tool"));
-    kr.registerShortcut(tr("Menu"), tr("Right"),
-                        tr("Show pane context menu"));
+    kr.registerMouseAction(tr("Zoom"), tr("Wheel"),
+                           tr("Zoom in or out in time axis"));
+    kr.registerMouseAction(tr("Scroll"), tr("Ctrl+Wheel"),
+                           tr("Scroll rapidly left or right in time axis"));
+    kr.registerMouseAction(tr("Scroll Vertically"), tr("Shift+Wheel"), 
+                           tr("Scroll up or down in the vertical axis"));
+    kr.registerMouseAction(tr("Zoom Vertically"), tr("Alt+Wheel"), 
+                           tr("Zoom in or out in the vertical axis"));
+    kr.registerMouseAction(tr("Navigate"), tr("Alt+Click+Drag"),
+                           tr("Navigate with any tool selected"));
+    kr.registerMouseAction(tr("Relocate"), tr("Alt+DoubleClick"),
+                           tr("Relocate with any tool selected"));
+#ifndef Q_OS_MAC
+    // It's so unlikely that a Mac user will ever use these. They
+    // should work, but let's not clog up the documentation
+    kr.registerAlternativeMouseAction(tr("Navigate"), tr("MiddleClick"));
+    kr.registerAlternativeMouseAction(tr("Relocate"), tr("DoubleMiddleClick"));
+#endif
+    kr.registerMouseAction(tr("Menu"),
+                           tr("RightClick"),
+#ifdef Q_OS_MAC
+                           tr("Ctrl+click mouse button to show pane context menu")
+#else
+                           tr("Click right mouse button to show pane context menu")
+#endif
+        );
 }
 
 Layer *
@@ -1394,9 +1433,9 @@ Pane::mousePressEvent(QMouseEvent *e)
         return;
     }
 
-    SVDEBUG << "Pane[" << getId() << "]::mousePressEvent: buttons = "
-            << e->buttons() << ", modifiers = "
-            << modifierNames(e->modifiers()) << endl;
+//    SVDEBUG << "Pane[" << getId() << "]::mousePressEvent: buttons = "
+//            << e->buttons() << ", modifiers = "
+//            << modifierNames(e->modifiers()) << endl;
 
     m_clickPos = e->pos();
     m_mousePos = m_clickPos;
@@ -1416,16 +1455,24 @@ Pane::mousePressEvent(QMouseEvent *e)
     m_editing = false;
     m_releasing = false;
 
-    if (mode == ViewManager::NavigateMode ||
-        (e->buttons() & Qt::MidButton) ||
-        (mode == ViewManager::MeasureMode &&
-         (e->buttons() & Qt::LeftButton) && m_shiftPressed)) {
+    if (mode == ViewManager::NavigateMode) {
+        m_navigating = true;
+    } else if (e->buttons() & Qt::MiddleButton) {
+        m_navigating = true;
+    } else if (m_altPressed) {
+        m_navigating = true;
+    } else if (mode == ViewManager::MeasureMode &&
+               m_shiftPressed &&
+               (e->buttons() & Qt::LeftButton)) {
+        m_navigating = true;
+    }
+    
+    if (m_navigating) {
 
         if (mode != ViewManager::NavigateMode) {
             setCursor(Qt::PointingHandCursor);
         }
 
-        m_navigating = true;
         m_dragCentreFrame = m_centreFrame;
         m_dragStartMinValue = 0;
         
@@ -1439,7 +1486,7 @@ Pane::mousePressEvent(QMouseEvent *e)
             // location. This will happen only if nothing else of
             // interest happens (double-click, drag) before the
             // timeout.
-            schedulePlaybackFrameMove(getFrameForX(e->x()));
+            schedulePlaybackFrameMove(getFrameForX(e->position().x()));
         }
 
     } else if (mode == ViewManager::SelectMode) {
@@ -1447,7 +1494,7 @@ Pane::mousePressEvent(QMouseEvent *e)
         if (!hasTopLayerTimeXAxis()) return;
 
         bool closeToLeft = false, closeToRight = false;
-        Selection selection = getSelectionAt(e->x(), closeToLeft, closeToRight);
+        Selection selection = getSelectionAt(e->position().x(), closeToLeft, closeToRight);
 
         if ((closeToLeft || closeToRight) && !(closeToLeft && closeToRight)) {
 
@@ -1464,15 +1511,17 @@ Pane::mousePressEvent(QMouseEvent *e)
             
         } else {
             
-            sv_frame_t mouseFrame = getFrameForX(e->x());
+            sv_frame_t mouseFrame = getFrameForX(e->position().x());
             int resolution = 1;
             sv_frame_t snapFrame = mouseFrame;
     
             Layer *layer = getInteractionLayer();
-            if (layer && !m_shiftPressed &&
+            bool shouldSnap = m_selectionSnap;
+            if (m_shiftPressed) shouldSnap = !shouldSnap;
+            if (layer && shouldSnap &&
                 !qobject_cast<TimeRulerLayer *>(layer)) { // don't snap to secs
                 layer->snapToFeatureFrame(this, snapFrame,
-                                          resolution, Layer::SnapLeft, e->y());
+                                          resolution, Layer::SnapLeft, e->position().y());
             }
         
             if (snapFrame < 0) snapFrame = 0;
@@ -1573,7 +1622,7 @@ Pane::mouseReleaseEvent(QMouseEvent *e)
         mouseMoveEvent(e);
     }
 
-    sv_frame_t mouseFrame = e ? getFrameForX(e->x()) : 0;
+    sv_frame_t mouseFrame = e ? getFrameForX(e->position().x()) : 0;
     if (mouseFrame < 0) mouseFrame = 0;
 
     if (m_navigating || mode == ViewManager::NavigateMode) {
@@ -1705,7 +1754,7 @@ Pane::mouseMoveEvent(QMouseEvent *e)
         // the window after pressing button first time).
 
         if (!(e->buttons() & Qt::LeftButton) &&
-            !(e->buttons() & Qt::MidButton)) {
+            !(e->buttons() & Qt::MiddleButton)) {
             m_clickedInRange = false;
             return;
         }
@@ -1732,7 +1781,7 @@ Pane::mouseMoveEvent(QMouseEvent *e)
     
         if (mode == ViewManager::SelectMode && hasTopLayerTimeXAxis()) {
             bool closeToLeft = false, closeToRight = false;
-            getSelectionAt(e->x(), closeToLeft, closeToRight);
+            getSelectionAt(e->position().x(), closeToLeft, closeToRight);
             if ((closeToLeft || closeToRight) && !(closeToLeft && closeToRight)) {
                 setCursor(Qt::SizeHorCursor);
             } else {
@@ -1740,11 +1789,12 @@ Pane::mouseMoveEvent(QMouseEvent *e)
             }
         }
 
-        if (m_manager && !m_manager->isPlaying()) {
+        if (!isCurrentlyMovingWithPlayback()) {
 
             bool updating = false;
 
             if (getInteractionLayer() &&
+                m_manager &&
                 m_manager->shouldIlluminateLocalFeatures()) {
 
                 bool previouslyIdentifying = m_identifyFeatures;
@@ -1851,8 +1901,8 @@ Pane::mouseMoveEvent(QMouseEvent *e)
 
                 if (layer) {
 
-                    int x = e->x();
-                    int y = e->y();
+                    int x = e->position().x();
+                    int y = e->position().y();
                     if (m_dragMode == VerticalDrag) x = m_clickPos.x();
                     else if (m_dragMode == HorizontalDrag) y = m_clickPos.y();
 
@@ -1917,8 +1967,8 @@ Pane::mouseMoveEvent(QMouseEvent *e)
 
                 if (layer && layer->isLayerEditable()) {
 
-                    int x = e->x();
-                    int y = e->y();
+                    int x = e->position().x();
+                    int y = e->position().y();
                     if (m_dragMode == VerticalDrag) x = m_clickPos.x();
                     else if (m_dragMode == HorizontalDrag) y = m_clickPos.y();
 
@@ -1940,7 +1990,7 @@ Pane::mouseMoveEvent(QMouseEvent *e)
         Layer *layer = getTopLayer();
         if (layer) {
             layer->measureDrag(this, e);
-            if (layer->hasTimeXAxis()) edgeScrollMaybe(e->x());
+            if (layer->hasTimeXAxis()) edgeScrollMaybe(e->position().x());
         }
 
         update();
@@ -1987,32 +2037,33 @@ Pane::zoomToRegion(QRect r)
     setZoomLevel(getZoomConstraintLevel(newZoomLevel));
     setStartFrame(newStartFrame);
 
-    QString unit;
-    double min, max;
-    bool log;
+    CoordinateScale scale = Layer::NO_VERTICAL_EXTENTS.second;
     Layer *layer = nullptr;
+    
     for (LayerList::const_iterator i = m_layerStack.begin();
-         i != m_layerStack.end(); ++i) { 
-        if ((*i)->getValueExtents(min, max, log, unit) &&
-            (*i)->getDisplayExtents(min, max)) {
+         i != m_layerStack.end(); ++i) {
+        auto extents = (*i)->getVerticalExtents();
+        if (extents.first == Layer::ScaleApplication::Normal) {
             layer = *i;
+            scale = extents.second;
             break;
         }
     }
             
     if (layer) {
-        if (log) {
-            min = (min < 0.0) ? -log10(-min) : (min == 0.0) ? 0.0 : log10(min);
-            max = (max < 0.0) ? -log10(-max) : (max == 0.0) ? 0.0 : log10(max);
+        double min = scale.getDisplayMinimum();
+        double max = scale.getDisplayMaximum();
+        if (scale.isLogarithmic()) {
+            LogRange::mapRange(min, max);
         }
         double rmin = min + ((max - min) * (height() - y1)) / height();
         double rmax = min + ((max - min) * (height() - y0)) / height();
-        cerr << "min: " << min << ", max: " << max << ", y0: " << y0 << ", y1: " << y1 << ", h: " << height() << ", rmin: " << rmin << ", rmax: " << rmax << endl;
-        if (log) {
-            rmin = pow(10, rmin);
-            rmax = pow(10, rmax);
+        SVCERR << "min: " << min << ", max: " << max << ", y0: " << y0 << ", y1: " << y1 << ", h: " << height() << ", rmin: " << rmin << ", rmax: " << rmax << endl;
+        if (scale.isLogarithmic()) {
+            rmin = LogRange::unmap(rmin);
+            rmax = LogRange::unmap(rmax);
         }
-        cerr << "finally: rmin: " << rmin << ", rmax: " << rmax << " " << unit << endl;
+        SVCERR << "finally: rmin: " << rmin << ", rmax: " << rmax << endl;
 
         layer->setDisplayExtents(rmin, rmax);
         updateVerticalPanner();
@@ -2053,14 +2104,14 @@ Pane::dragTopLayer(QMouseEvent *e)
          e->pos(),
          true, // can move horiz
          canTopLayerMoveVertical(), // can move vert
-         canTopLayerMoveVertical() || (m_manager && m_manager->isPlaying()), // resist horiz
+         canTopLayerMoveVertical() || isCurrentlyMovingWithPlayback(), // resist horiz
          true); // resist vert
 
     if (m_dragMode == HorizontalDrag ||
         m_dragMode == FreeDrag) {
 
         sv_frame_t fromFrame = getFrameForX(m_clickPos.x());
-        sv_frame_t toFrame = getFrameForX(e->x());
+        sv_frame_t toFrame = getFrameForX(e->position().x());
         sv_frame_t frameOff = toFrame - fromFrame;
 
         sv_frame_t newCentreFrame = m_dragCentreFrame;
@@ -2074,7 +2125,7 @@ Pane::dragTopLayer(QMouseEvent *e)
 
 #ifdef DEBUG_PANE
         SVDEBUG << "Pane::dragTopLayer: dragged from x = "
-                << m_clickPos.x() << " to " << e->x()
+                << m_clickPos.x() << " to " << e->position().x()
                 << ", from frame = " << fromFrame
                 << " to " << toFrame
                 << ", for frame offset of " << frameOff << endl;
@@ -2101,12 +2152,12 @@ Pane::dragTopLayer(QMouseEvent *e)
 
         if (getTopLayerDisplayExtents(vmin, vmax, dmin, dmax)) {
 
-//            cerr << "ydiff = " << ydiff << endl;
+//            SVCERR << "ydiff = " << ydiff << endl;
 
-            int ydiff = e->y() - m_clickPos.y();
+            int ydiff = e->position().y() - m_clickPos.y();
             double perpix = (dmax - dmin) / height();
             double valdiff = ydiff * perpix;
-//            cerr << "valdiff = " << valdiff << endl;
+//            SVCERR << "valdiff = " << valdiff << endl;
 
             if (m_dragMode == UnresolvedDrag && ydiff != 0) {
                 m_dragMode = VerticalDrag;
@@ -2122,7 +2173,7 @@ Pane::dragTopLayer(QMouseEvent *e)
                 newmin -= newmax - vmax;
                 newmax -= newmax - vmax;
             }
-//            cerr << "(" << dmin << ", " << dmax << ") -> ("
+//            SVCERR << "(" << dmin << ", " << dmax << ") -> ("
 //                      << newmin << ", " << newmax << ") (drag start " << m_dragStartMinValue << ")" << endl;
 
             setTopLayerDisplayExtents(newmin, newmax);
@@ -2198,21 +2249,23 @@ Pane::updateDragMode(DragMode dragMode,
 void
 Pane::dragExtendSelection(QMouseEvent *e)
 {
-    sv_frame_t mouseFrame = getFrameForX(e->x());
+    sv_frame_t mouseFrame = getFrameForX(e->position().x());
     int resolution = 1;
     sv_frame_t snapFrameLeft = mouseFrame;
     sv_frame_t snapFrameRight = mouseFrame;
     
     Layer *layer = getInteractionLayer();
-    if (layer && !m_shiftPressed &&
+    bool shouldSnap = m_selectionSnap;
+    if (m_shiftPressed) shouldSnap = !shouldSnap;
+    if (layer && shouldSnap &&
         !qobject_cast<TimeRulerLayer *>(layer)) { // don't snap to secs
         layer->snapToFeatureFrame(this, snapFrameLeft,
-                                  resolution, Layer::SnapLeft, e->y());
+                                  resolution, Layer::SnapLeft, e->position().y());
         layer->snapToFeatureFrame(this, snapFrameRight,
-                                  resolution, Layer::SnapRight, e->y());
+                                  resolution, Layer::SnapRight, e->position().y());
     }
         
-//        cerr << "snap: frame = " << mouseFrame << ", start frame = " << m_selectionStartFrame << ", left = " << snapFrameLeft << ", right = " << snapFrameRight << endl;
+//        SVCERR << "snap: frame = " << mouseFrame << ", start frame = " << m_selectionStartFrame << ", left = " << snapFrameLeft << ", right = " << snapFrameRight << endl;
 
     if (snapFrameLeft < 0) snapFrameLeft = 0;
     if (snapFrameRight < 0) snapFrameRight = 0;
@@ -2245,7 +2298,7 @@ Pane::dragExtendSelection(QMouseEvent *e)
         m_manager->setInProgressSelection(sel, !m_resizing && !m_ctrlPressed);
 
         if (!same) {
-            edgeScrollMaybe(e->x());
+            edgeScrollMaybe(e->position().x());
         }
     }
 
@@ -2309,12 +2362,12 @@ Pane::mouseDoubleClickEvent(QMouseEvent *e)
     if (m_manager) mode = m_manager->getToolModeFor(this);
 
     bool relocate = (mode == ViewManager::NavigateMode ||
-                     (e->buttons() & Qt::MidButton));
+                     (e->buttons() & Qt::MiddleButton));
 
     if (mode == ViewManager::SelectMode) {
         m_clickedInRange = false;
         if (m_manager) m_manager->clearInProgressSelection();
-        emit doubleClickSelectInvoked(getFrameForX(e->x()));
+        emit doubleClickSelectInvoked(getFrameForX(e->position().x()));
         return;
     }
 
@@ -2336,7 +2389,7 @@ Pane::mouseDoubleClickEvent(QMouseEvent *e)
 
     if (relocate) {
 
-        sv_frame_t f = getFrameForX(e->x());
+        sv_frame_t f = getFrameForX(e->position().x());
 
         setCentreFrame(f);
 
@@ -2362,14 +2415,18 @@ Pane::mouseDoubleClickEvent(QMouseEvent *e)
 }
 
 void
-Pane::enterEvent(QEvent *)
+Pane::enterEvent(QEnterEvent *)
 {
+//    SVDEBUG << "Pane[" << getId() << "]::enterEvent" << endl;
+
     m_mouseInWidget = true;
 }
 
 void
 Pane::leaveEvent(QEvent *)
 {
+//    SVDEBUG << "Pane[" << getId() << "]::leaveEvent" << endl;
+
     m_mouseInWidget = false;
     bool previouslyIdentifying = m_identifyFeatures;
     m_identifyFeatures = false;
@@ -2386,12 +2443,14 @@ Pane::resizeEvent(QResizeEvent *)
 void
 Pane::wheelEvent(QWheelEvent *e)
 {
-    SVDEBUG << "Pane[" << getId() << "]::wheelEvent: delta = " << e->delta()
-            << ", angleDelta = (" << e->angleDelta().x()
+/*
+  SVDEBUG << "Pane[" << getId() << "]::wheelEvent: pixelDelta = ("
+            << e->pixelDelta().x() << "," << e->pixelDelta().y()
+            << "), angleDelta = (" << e->angleDelta().x()
             << "," << e->angleDelta().y() << "), pixelDelta = ("
             << e->pixelDelta().x() << "," << e->pixelDelta().y()
             << "), modifiers = " << modifierNames(e->modifiers()) << endl;
-
+*/
     e->accept(); // we never want wheel events on the pane to be propagated
     
     int dx = e->angleDelta().x();
@@ -2481,8 +2540,8 @@ Pane::wheelEvent(QWheelEvent *e)
 void
 Pane::wheelVertical(int sign, Qt::KeyboardModifiers mods)
 {
-    SVDEBUG << "Pane[" << getId() << "]::wheelVertical: sign = " << sign
-            << ", modifiers = " << modifierNames(mods) << endl;
+//    SVDEBUG << "Pane[" << getId() << "]::wheelVertical: sign = " << sign
+//            << ", modifiers = " << modifierNames(mods) << endl;
 
     if (mods & Qt::ShiftModifier) {
 
@@ -2589,7 +2648,7 @@ Pane::verticalPannerMoved(float , float y0, float , float h)
     double y1 = y0 + h;
     double newmax = vmin + ((1.0 - y0) * (vmax - vmin));
     double newmin = vmin + ((1.0 - y1) * (vmax - vmin));
-//    cerr << "verticalPannerMoved: (" << x0 << "," << y0 << "," << w
+//    SVCERR << "verticalPannerMoved: (" << x0 << "," << y0 << "," << w
 //              << "," << h << ") -> (" << newmin << "," << newmax << ")" << endl;
     setTopLayerDisplayExtents(newmin, newmax);
 }
@@ -2703,8 +2762,8 @@ Pane::dropEvent(QDropEvent *e)
             SVDEBUG << "accepting... data is \"" << e->mimeData()->data("text/uri-list").data() << "\"" << endl;
             emit dropAccepted(QString::fromLocal8Bit
                               (e->mimeData()->data("text/uri-list").data())
-                              .split(QRegExp("[\\r\\n]+"), 
-                                     QString::SkipEmptyParts));
+                              .split(QRegularExpression("[\\r\\n]+"), 
+                                     Qt::SkipEmptyParts));
         } else {
             emit dropAccepted(QString::fromLocal8Bit
                               (e->mimeData()->data("text/plain").data()));
@@ -2722,7 +2781,7 @@ Pane::editSelectionStart(QMouseEvent *e)
     }
 
     bool closeToLeft, closeToRight;
-    Selection s(getSelectionAt(e->x(), closeToLeft, closeToRight));
+    Selection s(getSelectionAt(e->position().x(), closeToLeft, closeToRight));
     if (s.isEmpty()) return false;
     m_editingSelection = s;
     m_editingSelectionEdge = (closeToLeft ? -1 : closeToRight ? 1 : 0);
@@ -2855,7 +2914,7 @@ Pane::zoomWheelsEnabledChanged()
 void
 Pane::viewZoomLevelChanged(View *v, ZoomLevel z, bool locked)
 {
-//    cerr << "Pane[" << this << "]::zoomLevelChanged (global now "
+//    SVCERR << "Pane[" << this << "]::zoomLevelChanged (global now "
 //              << (m_manager ? m_manager->getGlobalZoom() : 0) << ")" << endl;
 
     View::viewZoomLevelChanged(v, z, locked);
@@ -2945,13 +3004,21 @@ Pane::updateContextHelp(const QPoint *pos)
         if (haveSelection) {
 #ifdef Q_OS_MAC
             if (editable) {
-                help = tr("Click and drag to select a range; hold Shift to avoid snapping to items; hold Cmd for multi-select; middle-click and drag to navigate");
+                if (m_selectionSnap) {
+                    help = tr("Click and drag to select a range; hold Shift to avoid snapping to items; hold Cmd for multi-select; middle-click and drag to navigate");
+                } else {
+                    help = tr("Click and drag to select a range; hold Shift to snap to items; hold Cmd for multi-select; middle-click and drag to navigate");
+                }
             } else {
                 help = tr("Click and drag to select a range; hold Cmd for multi-select; middle-click and drag to navigate");
             }
 #else
             if (editable) {
-                help = tr("Click and drag to select a range; hold Shift to avoid snapping to items; hold Ctrl for multi-select; middle-click and drag to navigate");
+                if (m_selectionSnap) {
+                    help = tr("Click and drag to select a range; hold Shift to avoid snapping to items; hold Ctrl for multi-select; middle-click and drag to navigate");
+                } else {
+                    help = tr("Click and drag to select a range; hold Shift to snap to items; hold Ctrl for multi-select; middle-click and drag to navigate");
+                }
             } else {
                 help = tr("Click and drag to select a range; hold Ctrl for multi-select; middle-click and drag to navigate");
             }                
@@ -2967,7 +3034,11 @@ Pane::updateContextHelp(const QPoint *pos)
             }
         } else {
             if (editable) {
-                help = tr("Click and drag to select a range; hold Shift to avoid snapping to items; middle-click to navigate");
+                if (m_selectionSnap) {
+                    help = tr("Click and drag to select a range; hold Shift to avoid snapping to items; middle-click to navigate");
+                } else {
+                    help = tr("Click and drag to select a range; hold Shift to snap to items; middle-click to navigate");
+                }
             } else {
                 help = tr("Click and drag to select a range; middle-click and drag to navigate");
             }
@@ -3039,3 +3110,5 @@ Pane::toXml(QTextStream &stream,
 }
 
 
+} // end namespace sv
+
