@@ -25,12 +25,15 @@
 
 class QWidget;
 class QPaintEvent;
+class QMenu;
+
+namespace sv {
+
 class Layer;
 class Thumbwheel;
 class Panner;
 class NotifyingPushButton;
 class KeyReference;
-class QMenu;
 
 class Pane : public View
 {
@@ -51,6 +54,14 @@ public:
     void setCentreLineVisible(bool visible);
     bool getCentreLineVisible() const { return m_centreLineVisible; }
 
+    /** Set whether to snap to features by default when dragging a
+        selection. The default is true unless Shift is pressed;
+        setting this to false reverses the behaviour (no snapping
+        unless Shift is pressed).
+    */
+    void setSelectionSnapToFeatures(bool snap);
+    bool getSelectionSnapToFeatures() const { return m_selectionSnap; }
+    
     virtual sv_frame_t getFirstVisibleFrame() const override;
 
     int getVerticalScaleWidth() const;
@@ -120,7 +131,7 @@ protected:
     virtual void mouseReleaseEvent(QMouseEvent *e) override;
     virtual void mouseMoveEvent(QMouseEvent *e) override;
     virtual void mouseDoubleClickEvent(QMouseEvent *e) override;
-    virtual void enterEvent(QEvent *e) override;
+    virtual void enterEvent(QEnterEvent *e) override;
     virtual void leaveEvent(QEvent *e) override;
     virtual void wheelEvent(QWheelEvent *e) override;
     virtual void resizeEvent(QResizeEvent *e) override;
@@ -168,6 +179,13 @@ protected:
 
     void schedulePlaybackFrameMove(sv_frame_t frame);
 
+    bool isCurrentlyMovingWithPlayback() const {
+        if (m_followPlay != PlaybackScrollContinuous) {
+            return false;
+        }
+        return (m_manager && m_manager->isPlaying());
+    }
+    
     bool m_identifyFeatures;
     QPoint m_identifyPoint;
     QPoint m_clickPos;
@@ -184,6 +202,7 @@ protected:
     sv_frame_t m_dragCentreFrame;
     double m_dragStartMinValue;
     bool m_centreLineVisible;
+    bool m_selectionSnap;
     sv_frame_t m_selectionStartFrame;
     Selection m_editingSelection;
     int m_editingSelectionEdge;
@@ -223,5 +242,7 @@ protected:
     static QCursor *m_measureCursor2;
 };
 
+} // end namespace sv
+
 #endif
 
