@@ -21,6 +21,7 @@
 #include "base/Pitch.h"
 #include "base/Preferences.h"
 #include "base/HitCount.h"
+#include "base/LogRange.h"
 #include "ViewProxy.h"
 
 #include "layer/TimeRulerLayer.h"
@@ -43,6 +44,8 @@
 #include <QPushButton>
 #include <QSettings>
 #include <QSvgGenerator>
+#include <QThreadPool>
+#include <QSemaphore>
 
 #include <iostream>
 #include <cassert>
@@ -53,6 +56,8 @@
 //#define DEBUG_PROGRESS_STUFF 1
 //#define DEBUG_VIEW_SCALE_CHOICE 1
 
+namespace sv {
+
 View::View(QWidget *w, bool showProgress) :
     QFrame(w),
     m_id(getNextId()),
@@ -197,174 +202,172 @@ View::getPropertyContainer(int i)
     return m_fixedOrderLayers[i-1];
 }
 
-bool
-View::getVisibleExtentsForUnit(QString unit,
-                               double &min, double &max,
-                               bool &log) const
+CoordinateScale
+View::getEffectiveVerticalExtentsForLayer(const Layer *layer) const
 {
+    auto declaredExtents = layer->getVerticalExtents();
+
 #ifdef DEBUG_VIEW_SCALE_CHOICE
-    SVCERR << "View[" << getId() << "]::getVisibleExtentsForUnit("
-           << unit << ")" << endl;
+    SVDEBUG << "View::getEffectiveVerticalExtentsForLayer: layer = " << layer
+            << ": " << layer->getLayerPresentationName() << endl;
+#endif
+
+    if (declaredExtents.first == Layer::ScaleApplication::Deferring &&
+        declaredExtents.second.getUnit() != "") {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+        SVDEBUG << "layer is deferring and has a unit, deferring to something"
+                << endl;
+#endif
+        return getEffectiveVerticalExtents(declaredExtents.second.getUnit());
+    } else {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+        SVDEBUG << "layer isn't deferring or has no unit, using it directly"
+                << endl;
+#endif
+        return declaredExtents.second;
+    }
+}
+
+CoordinateScale
+View::getEffectiveVerticalExtents(QString unit) const
+{
+    return (getEffectiveVerticalExtentsAndLayerFromWhich(unit, nullptr));
+}
+
+CoordinateScale
+View::getEffectiveVerticalExtentsAndLayerFromWhich(QString unit,
+                                                   Layer **layerReturnMaybe)
+    const
+{    
+    // Three passes:
+
+    // 1. Look for the topmost non-dormant layer with that unit that
+    // has an alignable (normal) scale.
+
+    // 2. Failing that, look for the topmost layer with any dormancy
+    // with that unit that has an alignable (normal) scale. A dormant
+    // layer can still draw a scale, and it makes sense for layers
+    // aligned to it not to jump about when its visibility is toggled.
+
+    // 3. Failing any of that, if a unit is provided, take the union
+    // of the scales of non-dormant deferring layers with that unit.
+
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+    SVDEBUG << "View::getEffectiveVerticalExtents: unit = " << unit << ", have "
+            << m_layerStack.size() << " layer(s)" << endl;
 #endif
     
-    Layer *layer = getScaleProvidingLayerForUnit(unit);
+    for (int acceptDormant = 0; acceptDormant <= 1; ++acceptDormant) {
 
-    QString layerUnit;
-    double layerMin, layerMax;
-
-    if (!layer) {
 #ifdef DEBUG_VIEW_SCALE_CHOICE
-        SVCERR << "View[" << getId() << "]::getVisibleExtentsForUnit("
-               << unit << "): No scale-providing layer for this unit, "
-               << "taking union of extents of layers with this unit" << endl;
+        SVDEBUG << "pass with acceptDormant " << acceptDormant << endl;
 #endif
-        bool haveAny = false;
-        bool layerLog;
+    
         for (auto i = m_layerStack.rbegin(); i != m_layerStack.rend(); ++i) { 
+
             Layer *layer = *i;
-            if (layer->getValueExtents(layerMin, layerMax,
-                                       layerLog, layerUnit)) {
-                if (unit.toLower() != layerUnit.toLower()) {
-                    continue;
-                }
-                if (!haveAny || layerMin < min) {
-                    min = layerMin;
-                }
-                if (!haveAny || layerMax > max) {
-                    max = layerMax;
-                }
-                if (!haveAny || layerLog) {
-                    log = layerLog;
-                }
-                haveAny = true;
+
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+            SVDEBUG << "considering layer " << layer
+                    << ": " << layer->getLayerPresentationName()
+                    << endl;
+#endif
+        
+            if (!acceptDormant && layer->isLayerDormant(this)) {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+                SVDEBUG << "... it's dormant" << endl;
+#endif
+                continue;
             }
+
+            auto extents = layer->getVerticalExtents();
+
+            if (extents.first != Layer::ScaleApplication::Normal) {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+                SVDEBUG << "... it's not alignable" << endl;
+#endif
+                continue;
+            }
+
+            if (unit != "" && extents.second.getUnit() != unit) {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+                SVDEBUG << "... it has the wrong unit ("
+                        << extents.second.getUnit() << ")" << endl;
+#endif
+                continue;
+            }
+        
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+            SVDEBUG << "... it's good, returning it" << endl;
+#endif
+            if (layerReturnMaybe) {
+                *layerReturnMaybe = layer;
+            }
+                
+            return extents.second;
         }
-        return haveAny;
     }
 
-    return (layer->getValueExtents(layerMin, layerMax, log, layerUnit) &&
-            layer->getDisplayExtents(min, max));
-}
-        
-Layer *
-View::getScaleProvidingLayerForUnit(QString unit) const
-{
-    // Return the layer which is used to provide the min/max/log for
-    // any auto-align layer of a given unit. This is also the layer
-    // that will draw the scale, if possible.
-    //
-    // The returned layer is
-    // 
-    // - the topmost visible layer having that unit that is not also
-    // auto-aligning; or if there is no such layer,
-    //
-    // - the topmost layer of any visibility having that unit that is
-    // not also auto-aligning (because a dormant layer can still draw
-    // a scale, and it makes sense for layers aligned to it not to
-    // jump about when its visibility is toggled); or if there is no
-    // such layer,
-    //
-    // - none
-
-    Layer *dormantOption = nullptr;
+    CoordinateScale scale = Layer::NO_VERTICAL_EXTENTS.second;
+    bool haveAny = false;
     
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+    SVDEBUG << "View::getEffectiveVerticalExtents: found nothing suitable on its own, looking to merge scales" << endl;
+#endif
+
     for (auto i = m_layerStack.rbegin(); i != m_layerStack.rend(); ++i) { 
 
         Layer *layer = *i;
 
 #ifdef DEBUG_VIEW_SCALE_CHOICE
-        SVCERR << "View[" << getId() << "]::getScaleProvidingLayerForUnit("
-               << unit << "): Looking at layer " << layer
-               << " (" << layer->getLayerPresentationName() << ")" << endl;
+        SVDEBUG << "considering layer " << layer
+                << ": " << layer->getLayerPresentationName()
+                << endl;
 #endif
         
-        QString layerUnit;
-        double layerMin = 0.0, layerMax = 0.0;
-        bool layerLog = false;
-
-        if (!layer->getValueExtents(layerMin, layerMax, layerLog, layerUnit)) {
-#ifdef DEBUG_VIEW_SCALE_CHOICE
-            SVCERR << "... it has no value extents" << endl;
-#endif
-            continue;
-        }
-
-        if (layerUnit.toLower() != unit.toLower()) {
-#ifdef DEBUG_VIEW_SCALE_CHOICE
-            SVCERR << "... it has the wrong unit (" << layerUnit << ")" << endl;
-#endif
-            continue;
-        }
-
-        double displayMin = 0.0, displayMax = 0.0;
-        if (!layer->getDisplayExtents(displayMin, displayMax)) {
-#ifdef DEBUG_VIEW_SCALE_CHOICE
-            SVCERR << "... it has no display extents (is auto-aligning or not alignable)" << endl;
-#endif
-            continue;
-        }
-
         if (layer->isLayerDormant(this)) {
 #ifdef DEBUG_VIEW_SCALE_CHOICE
-            SVCERR << "... it's dormant" << endl;
+            SVDEBUG << "... it's dormant" << endl;
 #endif
-            if (!dormantOption) {
-                dormantOption = layer;
-            }
             continue;
         }
 
+        auto extents = layer->getVerticalExtents();
+
+        if (extents.first == Layer::ScaleApplication::Personal ||
+            extents.first == Layer::ScaleApplication::None) {
 #ifdef DEBUG_VIEW_SCALE_CHOICE
-        SVCERR << "... it's good" << endl;
+            SVDEBUG << "... it's personal" << endl;
 #endif
-        return layer;
-    }
-
-    return dormantOption;
-}
-
-bool
-View::getVisibleExtentsForAnyUnit(double &min, double &max,
-                                  bool &log, QString &unit) const
-{
-    bool have = false;
-
-    // Iterate in reverse order, so as to return display extents of
-    // topmost layer that fits the bill
-    
-    for (auto i = m_layerStack.rbegin(); i != m_layerStack.rend(); ++i) { 
-
-        Layer *layer = *i;
-
-        if (layer->isLayerDormant(this)) {
-            continue;
-        }
-        
-        QString layerUnit;
-        double layerMin = 0.0, layerMax = 0.0;
-        bool layerLog = false;
-
-        if (!layer->getValueExtents(layerMin, layerMax, layerLog, layerUnit)) {
-            continue;
-        }
-        if (layerUnit == "") {
             continue;
         }
 
-        double displayMin = 0.0, displayMax = 0.0;
-        
-        if (layer->getDisplayExtents(displayMin, displayMax)) {
+        if (unit != "" && extents.second.getUnit() != unit) {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+            SVDEBUG << "... it has the wrong unit ("
+                    << extents.second.getUnit() << ")" << endl;
+#endif
+            continue;
+        }
 
-            min = displayMin;
-            max = displayMax;
-            log = layerLog;
-            unit = layerUnit;
-            have = true;
-            break;
+        if (haveAny) {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+            SVDEBUG << "... it's good, merging with it" << endl;
+#endif
+            scale = scale.unionWith(extents.second);
+        } else {
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+            SVDEBUG << "... it's good, initialising with it" << endl;
+#endif
+            scale = extents.second;
+            haveAny = true;
         }
     }
 
-    return have;
+#ifdef DEBUG_VIEW_SCALE_CHOICE
+    SVDEBUG << "... returning merged result" << endl;
+#endif
+    return scale;
 }
 
 int
@@ -465,7 +468,7 @@ View::setStartFrame(sv_frame_t f)
 }
 
 bool
-View::setCentreFrame(sv_frame_t f, bool e)
+View::setCentreFrame(sv_frame_t f, bool doEmit)
 {
     bool changeVisible = false;
 
@@ -512,7 +515,7 @@ View::setCentreFrame(sv_frame_t f, bool e)
             }
         }
 
-        if (e) {
+        if (doEmit) {
             sv_frame_t rf = alignToReference(m_centreFrame);
 #ifdef DEBUG_VIEW
             SVCERR << "View[" << getId() << "]::setCentreFrame(" << f
@@ -670,70 +673,90 @@ double
 View::getYForFrequency(double frequency,
                        double minf,
                        double maxf, 
-                       bool logarithmic) const
+                       FrequencyMapping mapping) const
 {
     Profiler profiler("View::getYForFrequency");
 
     int h = height();
 
-    if (logarithmic) {
+    switch (mapping) {
 
-        static double lastminf = 0.0, lastmaxf = 0.0;
-        static double logminf = 0.0, logmaxf = 0.0;
-
-        if (lastminf != minf) {
-            lastminf = (minf == 0.0 ? 1.0 : minf);
-            logminf = log10(minf);
-        }
-        if (lastmaxf != maxf) {
-            lastmaxf = (maxf < lastminf ? lastminf : maxf);
-            logmaxf = log10(maxf);
-        }
-
-        if (logminf == logmaxf) return 0;
-        return h - (h * (log10(frequency) - logminf)) / (logmaxf - logminf);
-
-    } else {
-        
+    case FrequencyMapping::Linear:
         if (minf == maxf) return 0;
         return h - (h * (frequency - minf)) / (maxf - minf);
+
+    case FrequencyMapping::Log:
+    {
+        double logminf = minf, logmaxf = maxf;
+        LogRange::mapRange(logminf, logmaxf);
+        if (logminf == logmaxf) return 0;
+        double logf = LogRange::map(frequency);
+        return h - (h * (logf - logminf)) / (logmaxf - logminf);
     }
+
+    case FrequencyMapping::Mel:
+    {
+        auto formula = Pitch::MelFormula::OShaughnessy;
+        double melminf = Pitch::getMelForFrequency(minf, formula);
+        double melmaxf = Pitch::getMelForFrequency(maxf, formula);
+        if (melminf == melmaxf) return 0;
+        double y = h - (h * (Pitch::getMelForFrequency(frequency, formula) -
+                             melminf) /
+                        (melmaxf - melminf));
+        SVDEBUG << "mel: frequency " << frequency << " -> " << y << endl;
+        return y;
+    }
+    }
+
+    return 0;
 }
 
 double
 View::getFrequencyForY(double y,
                        double minf,
                        double maxf,
-                       bool logarithmic) const
+                       FrequencyMapping mapping) const
 {
     double h = height();
 
-    if (logarithmic) {
-
-        static double lastminf = 0.0, lastmaxf = 0.0;
-        static double logminf = 0.0, logmaxf = 0.0;
-
-        if (lastminf != minf) {
-            lastminf = (minf == 0.0 ? 1.0 : minf);
-            logminf = log10(minf);
-        }
-        if (lastmaxf != maxf) {
-            lastmaxf = (maxf < lastminf ? lastminf : maxf);
-            logmaxf = log10(maxf);
-        }
-
-        if (logminf == logmaxf) return 0;
-        return pow(10.0, logminf + ((logmaxf - logminf) * (h - y)) / h);
-
-    } else {
+    switch (mapping) {
 
+    case FrequencyMapping::Linear:
         if (minf == maxf) return 0;
         return minf + ((h - y) * (maxf - minf)) / h;
+
+    case FrequencyMapping::Log:
+    {
+        double logminf = minf, logmaxf = maxf;
+        LogRange::mapRange(logminf, logmaxf);
+        if (logminf == logmaxf) return 0;
+        return LogRange::unmap(logminf + ((logmaxf - logminf) * (h - y)) / h);
     }
+    
+    case FrequencyMapping::Mel:
+    {
+        auto formula = Pitch::MelFormula::OShaughnessy;
+        double melminf = Pitch::getMelForFrequency(minf, formula);
+        double melmaxf = Pitch::getMelForFrequency(maxf, formula);
+        if (melminf == melmaxf) return 0;
+        double frequency = Pitch::getFrequencyForMel
+            (melminf + ((melmaxf - melminf) * (h - y)) / h, formula);
+        SVDEBUG << "mel: y " << y << " -> frequency " << frequency << endl;
+        return frequency;
+    }
+    }
+
+    return 0;
 }
 
 ZoomLevel
-View::getZoomLevel() const
+View::getRoundedZoomLevel() const
+{
+    return getRawZoomLevel();
+}
+
+ZoomLevel
+View::getRawZoomLevel() const
 {
 #ifdef DEBUG_VIEW_WIDGET_PAINT
 //        cout << "zoom level: " << m_zoomLevel << endl;
@@ -744,8 +767,17 @@ View::getZoomLevel() const
 int
 View::effectiveDevicePixelRatio() const
 {
+    // This is the ratio in force when we are drawing to our internal
+    // buffer image. We aren't equipped to update correctly with
+    // fractional scaling, so it's an integer value. But we round up,
+    // because we get much better results drawing large and scaling
+    // down than the reverse. That does make the actual drawing very
+    // expensive, e.g. if the scale is 125% and we draw at 200%, but
+    // if we don't do this the results can look terrible.
+    
+    int dpratio = int(ceil(devicePixelRatioF()));
+    
 #ifdef Q_OS_MAC
-    int dpratio = devicePixelRatio();
     // devicePixelRatio can change if the window is moved, so we don't
     // want to cache it. But we no longer allow the scaledHiDpi
     // setting to be changed without recommending a restart, so we
@@ -762,10 +794,9 @@ View::effectiveDevicePixelRatio() const
             dpratio = 1;
         }
     }
-    return dpratio;
-#else
-    return 1;
 #endif
+    
+    return dpratio;
 }
 
 void
@@ -2239,17 +2270,13 @@ View::setPaintFont(QPainter &paint)
     paint.setFont(font);
 }
 
-QRect
-View::getPaintRect() const
-{
-    return rect();
-}
-
 void
 View::paintEvent(QPaintEvent *e)
 {
 //    Profiler prof("View::paintEvent", false);
 
+    QFrame::paintEvent(e);
+
 #ifdef DEBUG_VIEW_WIDGET_PAINT
     {
         sv_frame_t startFrame = getStartFrame();
@@ -2258,7 +2285,6 @@ View::paintEvent(QPaintEvent *e)
 #endif
 
     if (m_layerStack.empty()) {
-        QFrame::paintEvent(e);
         return;
     }
 
@@ -2320,6 +2346,9 @@ View::paintEvent(QPaintEvent *e)
     LayerList scrollables = getScrollableBackLayers(true, layersChanged);
     LayerList nonScrollables = getNonScrollableFrontLayers(true, layersChanged);
 
+    // In the case where we never encounter an opaque layer, we ought
+    // to clear the background before drawing from buffer to widget
+    
 #ifdef DEBUG_VIEW_WIDGET_PAINT
     SVCERR << "View[" << getId() << "]::paintEvent: have " << scrollables.size()
               << " scrollable back layers and " << nonScrollables.size()
@@ -2335,7 +2364,7 @@ View::paintEvent(QPaintEvent *e)
 
     if (!m_buffer || wholeSize != m_buffer->size()) {
         delete m_buffer;
-        m_buffer = new QPixmap(wholeSize);
+        m_buffer = new QImage(wholeSize, QImage::Format_ARGB32_Premultiplied);
     }
 
     bool shouldUseCache = false;
@@ -2375,10 +2404,9 @@ View::paintEvent(QPaintEvent *e)
 #endif
             } else {
 
-                if (!m_cache ||
-                    m_cache->size() != wholeSize) {
+                if (!m_cache || m_cache->size() != wholeSize) {
                     delete m_cache;
-                    m_cache = new QPixmap(wholeSize);
+                    m_cache = new QImage(wholeSize, QImage::Format_ARGB32_Premultiplied);
                 }
 
 #ifdef DEBUG_VIEW_WIDGET_PAINT
@@ -2399,8 +2427,15 @@ View::paintEvent(QPaintEvent *e)
 
             if (dx > -m_cache->width() && dx < m_cache->width()) {
 
-                m_cache->scroll(dx, 0, m_cache->rect(), nullptr);
-
+                int mx0 = std::max(0, -dx);
+                int mx1 = std::max(0, dx);
+                int mw = m_cache->width() - std::max(dx, -dx);
+                
+                for (int row = 0; row < m_cache->height(); ++row) {
+                    QRgb *sl = reinterpret_cast<QRgb *>(m_cache->scanLine(row));
+                    breakfastquay::v_move(sl + mx1, sl + mx0, mw);
+                }
+                
                 if (dx < 0) {
                     cacheAreaToRepaint = 
                         QRect(m_cache->width() + dx, 0, -dx, m_cache->height());
@@ -2473,41 +2508,19 @@ View::paintEvent(QPaintEvent *e)
     // Scrollable (cacheable) items first. If we are repainting the
     // cache, then we paint these to the cache; otherwise straight to
     // the buffer.
-    QRect areaToPaint;
     QPainter paint;
-
     if (shouldRepaintCache) {
         paint.begin(m_cache);
-        areaToPaint = cacheAreaToRepaint;
+        paint.fillRect(cacheAreaToRepaint, getBackground());
+        paint.end();
     } else {
         paint.begin(m_buffer);
-        areaToPaint = requestedPaintArea;
+        paint.fillRect(requestedPaintArea, getBackground());
+        paint.end();
     }
 
-    setPaintFont(paint);
-
-    // This clipping is not a good idea I think - layers often
-    // intentionally paint outside the lines a little because they
-    // don't have a very precise idea about e.g. parts of text labels
-    // which overlay an area. We pass areaToPaint to the paint
-    // function anyway, so if clipping matters to it, it should enable
-    // it itself
-//    paint.setClipRect(areaToPaint);
-
-    paint.setPen(getBackground());
-    paint.setBrush(getBackground());
-    paint.drawRect(areaToPaint);
-
-    paint.setPen(getForeground());
-    paint.setBrush(Qt::NoBrush);
-        
-    for (LayerList::iterator i = scrollables.begin();
-         i != scrollables.end(); ++i) {
-
-        paint.setRenderHint(QPainter::Antialiasing, false);
-        paint.save();
-
-        Layer *layer = *i;
+    auto paintLayer = [&](Layer *layer, QImage *target,
+                          QRect area, bool enforceClipping) {
         
         bool useAligningProxy = false;
         if (m_useAligningProxy) {
@@ -2518,17 +2531,35 @@ View::paintEvent(QPaintEvent *e)
         }
 
 #ifdef DEBUG_VIEW_WIDGET_PAINT
-        SVCERR << "Painting scrollable layer " << layer << " (model " << layer->getModel() << ", source model " << layer->getSourceModel() << ") with shouldRepaintCache = " << shouldRepaintCache << ", useAligningProxy = " << useAligningProxy << ", dpratio = " << dpratio << ", areaToPaint = " << areaToPaint.x() << "," << areaToPaint.y() << " " << areaToPaint.width() << "x" << areaToPaint.height() << endl;
+        SVCERR << "Painting layer " << layer << " (model " << layer->getModel() << ", source model " << layer->getSourceModel() << ") with useAligningProxy = " << useAligningProxy << ", area = " << area.x() << "," << area.y() << " " << area.width() << "x" << area.height() << ", enforceClipping = " << enforceClipping << endl;
 #endif
-        
-        layer->paint(useAligningProxy ? &aligningProxy : &proxy,
-                     paint, areaToPaint);
 
-        paint.restore();
+        QPainter p(target);
+        p.setRenderHint(QPainter::Antialiasing, false);
+        if (enforceClipping) {
+            p.setClipRect(area);
+        }
+        p.setPen(getForeground());
+        p.setBrush(Qt::NoBrush);
+        setPaintFont(p);
+        layer->paint(useAligningProxy ? &aligningProxy : &proxy, p, area);
+        p.end();
+    };
+
+    for (auto layer : scrollables) {
+        // Clipping is not a good idea here - layers often
+        // intentionally paint outside the lines a little because they
+        // don't have a very precise idea about e.g. parts of text
+        // labels which overlay an area. We pass the area to the paint
+        // function anyway, so if clipping matters to it, it should
+        // enable it itself
+        if (shouldRepaintCache) {
+            paintLayer(layer, m_cache, cacheAreaToRepaint, false);
+        } else {
+            paintLayer(layer, m_buffer, requestedPaintArea, false);
+        }
     }
 
-    paint.end();
-
     if (shouldRepaintCache) {
         // and now we have
         m_cacheValid = true;
@@ -2538,64 +2569,38 @@ View::paintEvent(QPaintEvent *e)
 
     if (shouldUseCache) {
         paint.begin(m_buffer);
-        paint.drawPixmap(requestedPaintArea, *m_cache, requestedPaintArea);
+        paint.drawImage(requestedPaintArea, *m_cache, requestedPaintArea);
         paint.end();
     }
 
     // Now non-cacheable items.
 
-    paint.begin(m_buffer);
-    paint.setClipRect(requestedPaintArea);
-    setPaintFont(paint);
-    if (scrollables.empty()) {
-        paint.setPen(getBackground());
-        paint.setBrush(getBackground());
-        paint.drawRect(requestedPaintArea);
+    for (auto layer : nonScrollables) {
+        paintLayer(layer, m_buffer, requestedPaintArea, true);
     }
         
-    paint.setPen(getForeground());
-    paint.setBrush(Qt::NoBrush);
-        
-    for (LayerList::iterator i = nonScrollables.begin(); 
-         i != nonScrollables.end(); ++i) {
-        
-        Layer *layer = *i;
-        
-        bool useAligningProxy = false;
-        if (m_useAligningProxy) {
-            if (layer->getModel() == alignmentReferenceId ||
-                layer->getSourceModel() == alignmentReferenceId) {
-                useAligningProxy = true;
-            }
-        }
-
-#ifdef DEBUG_VIEW_WIDGET_PAINT
-        SVCERR << "Painting non-scrollable layer " << layer << " (model " << layer->getModel() << ", source model " << layer->getSourceModel() << ") with shouldRepaintCache = " << shouldRepaintCache << ", useAligningProxy = " << useAligningProxy << ", dpratio = " << dpratio << ", requestedPaintArea = " << requestedPaintArea.x() << "," << requestedPaintArea.y() << " " << requestedPaintArea.width() << "x" << requestedPaintArea.height() << endl;
-#endif
-
-        layer->paint(useAligningProxy ? &aligningProxy : &proxy,
-                     paint, requestedPaintArea);
-    }
-        
-    paint.end();
-
     // Now paint to widget from buffer: target rects from here on,
     // unlike all the preceding, are at formal (1x) resolution
 
     paint.begin(this);
-    setPaintFont(paint);
-    if (e) paint.setClipRect(e->rect());
-
+    
     QRect finalPaintRect = e ? e->rect() : rect();
-    paint.drawPixmap(finalPaintRect, *m_buffer, 
-                     scaledRect(finalPaintRect, dpratio));
+
+    paint.setClipRect(finalPaintRect);
+    setPaintFont(paint);
+
+    paint.setRenderHint(QPainter::SmoothPixmapTransform);
+    paint.setCompositionMode(QPainter::CompositionMode_Source);
+
+    paint.drawImage(finalPaintRect, *m_buffer, 
+                    scaledRect(finalPaintRect, dpratio));
+
+    paint.setCompositionMode(QPainter::CompositionMode_SourceOver);
 
     drawSelections(paint);
     drawPlayPointer(paint);
 
     paint.end();
-
-    QFrame::paintEvent(e);
 }
 
 void
@@ -2711,14 +2716,9 @@ View::drawSelections(QPainter &paint)
                       .toText(true)))
                 .arg(i->getEndFrame() - i->getStartFrame());
             
-    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
-    // replacement (horizontalAdvance) was only added in Qt 5.11
-    // which is too new for us
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
-            int sw = metrics.width(startText),
-                ew = metrics.width(endText),
-                dw = metrics.width(durationText);
+            int sw = metrics.horizontalAdvance(startText),
+                ew = metrics.horizontalAdvance(endText),
+                dw = metrics.horizontalAdvance(durationText);
 
             int sy = metrics.ascent() + metrics.height() + 4;
             int ey = sy;
@@ -2865,7 +2865,7 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
             axs = QString("%1 (%2)").arg(axs)
                 .arg(Pitch::getPitchLabelForFrequency(v0));
         }
-        aw = paint.fontMetrics().width(axs);
+        aw = paint.fontMetrics().horizontalAdvance(axs);
         ++labelCount;
     }
 
@@ -2878,7 +2878,7 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
                 bxs = QString("%1 (%2)").arg(bxs)
                     .arg(Pitch::getPitchLabelForFrequency(v1));
             }
-            bw = paint.fontMetrics().width(bxs);
+            bw = paint.fontMetrics().horizontalAdvance(bxs);
         }
     }
 
@@ -2886,7 +2886,7 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
         
     if (b0 && b1 && v1 != v0 && u0 == u1) {
         dxs = QString("[%1 %2]").arg(fabs(v1 - v0)).arg(u1);
-        dw = paint.fontMetrics().width(dxs);
+        dw = paint.fontMetrics().horizontalAdvance(dxs);
     }
     
     b0 = false;
@@ -2900,7 +2900,7 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
             ays = QString("%1 (%2)").arg(ays)
                 .arg(Pitch::getPitchLabelForFrequency(v0));
         }
-        aw = std::max(aw, paint.fontMetrics().width(ays));
+        aw = std::max(aw, paint.fontMetrics().horizontalAdvance(ays));
         ++labelCount;
     }
 
@@ -2913,7 +2913,7 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
                 bys = QString("%1 (%2)").arg(bys)
                     .arg(Pitch::getPitchLabelForFrequency(v1));
             }
-            bw = std::max(bw, paint.fontMetrics().width(bys));
+            bw = std::max(bw, paint.fontMetrics().horizontalAdvance(bys));
         }
     }
 
@@ -2940,7 +2940,7 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
         } else {
             dys = QString("[%1]").arg(dy);
         }
-        dw = std::max(dw, paint.fontMetrics().width(dys));
+        dw = std::max(dw, paint.fontMetrics().horizontalAdvance(dys));
     }
 
     int mw = r.width();
@@ -3194,8 +3194,8 @@ View::renderToNewImage()
 QImage *
 View::renderPartToNewImage(sv_frame_t f0, sv_frame_t f1)
 {
-    int x0 = int(round(getZoomLevel().framesToPixels(double(f0))));
-    int x1 = int(round(getZoomLevel().framesToPixels(double(f1))));
+    int x0 = int(round(m_zoomLevel.framesToPixels(double(f0))));
+    int x1 = int(round(m_zoomLevel.framesToPixels(double(f1))));
     
     QImage *image = new QImage(x1 - x0, height(), QImage::Format_RGB32);
 
@@ -3222,8 +3222,8 @@ View::getRenderedImageSize()
 QSize
 View::getRenderedPartImageSize(sv_frame_t f0, sv_frame_t f1)
 {
-    int x0 = int(round(getZoomLevel().framesToPixels(double(f0))));
-    int x1 = int(round(getZoomLevel().framesToPixels(double(f1))));
+    int x0 = int(round(m_zoomLevel.framesToPixels(double(f0))));
+    int x1 = int(round(m_zoomLevel.framesToPixels(double(f1))));
 
     return QSize(x1 - x0, height());
 }
@@ -3240,8 +3240,8 @@ View::renderToSvgFile(QString filename)
 bool
 View::renderPartToSvgFile(QString filename, sv_frame_t f0, sv_frame_t f1)
 {
-    int x0 = int(round(getZoomLevel().framesToPixels(double(f0))));
-    int x1 = int(round(getZoomLevel().framesToPixels(double(f1))));
+    int x0 = int(round(m_zoomLevel.framesToPixels(double(f0))));
+    int x1 = int(round(m_zoomLevel.framesToPixels(double(f1))));
 
     QSvgGenerator generator;
     generator.setFileName(filename);
@@ -3313,3 +3313,6 @@ ViewPropertyContainer::ViewPropertyContainer(View *v) :
 ViewPropertyContainer::~ViewPropertyContainer()
 {
 }
+
+} // end namespace sv
+
