@@ -20,6 +20,7 @@
 #include <QProgressBar>
 
 #include "layer/LayerGeometryProvider.h"
+#include "layer/CoordinateScale.h"
 
 #include "base/ZoomConstraint.h"
 #include "base/PropertyContainer.h"
@@ -31,14 +32,16 @@
 
 // #define DEBUG_VIEW_WIDGET_PAINT 1
 
-class Layer;
-class ViewPropertyContainer;
-
 class QPushButton;
 
 #include <map>
 #include <set>
 
+namespace sv {
+
+class Layer;
+class ViewPropertyContainer;
+
 /**
  * View is the base class of widgets that display one or more
  * overlaid views of data against a horizontal time scale. 
@@ -70,8 +73,6 @@ public:
      * ids, but ViewProxy objects share the id of their View.
      */
     int getId() const override { return m_id; }
-
-
     
     /**
      * Retrieve the first visible sample frame on the widget.
@@ -134,33 +135,37 @@ public:
      * scaling case.
      */
     int getViewXForX(int x) const override { return x; }
-
+    
     /**
-     * Return the pixel y-coordinate corresponding to a given
-     * frequency, if the frequency range is as specified.  This does
-     * not imply any policy about layer frequency ranges, but it might
-     * be useful for layers to match theirs up if desired.
-     *
-     * Not thread-safe in logarithmic mode.  Call only from GUI thread.
+     * Return the (maybe fractional) pixel y-coordinate corresponding
+     * to a given frequency, if the frequency range is as specified.
+     * This does not imply any policy about layer frequency ranges,
+     * but it might be useful for layers to match theirs up if
+     * desired.
      */
     double getYForFrequency(double frequency, double minFreq, double maxFreq, 
-                           bool logarithmic) const override;
+                            FrequencyMapping mapping) const override;
 
     /**
-     * Return the closest frequency to the given pixel y-coordinate,
-     * if the frequency range is as specified.
-     *
-     * Not thread-safe in logarithmic mode.  Call only from GUI thread.
+     * Return the closest frequency to the given (maybe fractional)
+     * pixel y-coordinate, if the frequency range is as specified.
      */
     double getFrequencyForY(double y, double minFreq, double maxFreq,
-                            bool logarithmic) const override;
+                            FrequencyMapping mapping) const override;
 
     /**
      * Return the zoom level, i.e. the number of frames per pixel or
-     * pixels per frame
+     * pixels per frame. In View (unlike ViewProxy) this is identical
+     * to getRawZoomLevel().
      */
-    ZoomLevel getZoomLevel() const override;
+    ZoomLevel getRoundedZoomLevel() const override;
 
+    /**
+     * Return the zoom level, i.e. the number of frames per pixel or
+     * pixels per frame.
+     */
+    ZoomLevel getRawZoomLevel() const override;
+    
     /**
      * Set the zoom level, i.e. the number of frames per pixel or
      * pixels per frame.  The centre frame will be unchanged; the
@@ -353,22 +358,13 @@ public:
     virtual bool renderPartToSvgFile(QString filename,
                                      sv_frame_t f0, sv_frame_t f1);
 
-    /**
-     * Return the visible vertical extents for the given unit, if any.
-     * Overridden from LayerGeometryProvider (see docs there).
-     */
-    bool getVisibleExtentsForUnit(QString unit, double &min, double &max,
-                                  bool &log) const override;
+    //!!! DOC
+    CoordinateScale
+    getEffectiveVerticalExtentsForLayer(const Layer *) const override;
 
-    /**
-     * Return some visible vertical extents and unit. That is, if at
-     * least one non-dormant layer has a non-empty unit and returns
-     * some values from its getDisplayExtents() method, return the
-     * extents and unit from the topmost of those. Otherwise return
-     * false.
-     */
-    bool getVisibleExtentsForAnyUnit(double &min, double &max,
-                                     bool &logarithmic, QString &unit) const;
+    //!!! DOC
+    CoordinateScale
+    getEffectiveVerticalExtents(QString unit = {}) const override;
     
     int getTextLabelYCoord(const Layer *layer, QPainter &) const override;
 
@@ -388,11 +384,11 @@ public:
      * of the view (if 1x display scaling is in effect) or of a larger
      * cached pixmap (if greater display scaling is in effect).
      */
-    QRect getPaintRect() const override;
+    QRect getPaintRect() const override { return rect(); }
 
-    QSize getPaintSize() const override { return getPaintRect().size(); }
-    int getPaintWidth() const override { return getPaintRect().width(); }
-    int getPaintHeight() const override { return getPaintRect().height(); }
+    QSize getPaintSize() const override { return size(); }
+    int getPaintWidth() const override { return width(); }
+    int getPaintHeight() const override { return height(); }
 
     double scaleSize(double size) const override;
     int scalePixelSize(int size) const override;
@@ -506,7 +502,8 @@ protected:
     LayerList getScrollableBackLayers(bool testChanged, bool &changed) const;
     LayerList getNonScrollableFrontLayers(bool testChanged, bool &changed) const;
 
-    Layer *getScaleProvidingLayerForUnit(QString unit) const;
+    CoordinateScale getEffectiveVerticalExtentsAndLayerFromWhich
+    (QString unit, Layer **layer) const;
     
     ZoomLevel getZoomConstraintLevel(ZoomLevel level,
                                      ZoomConstraint::RoundingDirection dir =
@@ -551,8 +548,10 @@ protected:
     bool                m_lightBackground;
     bool                m_showProgress;
 
-    QPixmap            *m_cache;  // I own this
-    QPixmap            *m_buffer; // I own this
+    // I own both m_cache and m_buffer
+    QImage             *m_cache;
+    QImage             *m_buffer;
+    
     bool                m_cacheValid;
     sv_frame_t          m_cacheCentreFrame;
     ZoomLevel           m_cacheZoomLevel;
@@ -639,5 +638,7 @@ protected:
     View *m_v;
 };
 
+} // end namespace sv
+
 #endif
 
