@@ -19,6 +19,8 @@
 
 #include "data/model/AlignmentModel.h"
 
+namespace sv {
+
 class ViewProxy : public LayerGeometryProvider
 {
 public:
@@ -88,25 +90,27 @@ public:
         return alignToReference(m_view->getModelsEndFrame());
     }
     double getYForFrequency(double frequency,
-                                    double minFreq, double maxFreq, 
-                                    bool logarithmic) const override {
+                            double minFreq, double maxFreq, 
+                            FrequencyMapping mapping) const override {
         return m_scaleFactor *
-            m_view->getYForFrequency(frequency, minFreq, maxFreq, logarithmic);
+            m_view->getYForFrequency(frequency, minFreq, maxFreq, mapping);
     }
     double getFrequencyForY(double y, double minFreq, double maxFreq,
-                                    bool logarithmic) const override {
+                            FrequencyMapping mapping) const override {
         return m_view->getFrequencyForY
-            (y / m_scaleFactor, minFreq, maxFreq, logarithmic);
+            (y / m_scaleFactor, minFreq, maxFreq, mapping);
     }
     int getTextLabelYCoord(const Layer *layer, QPainter &paint) const override {
         return m_scaleFactor * m_view->getTextLabelYCoord(layer, paint);
     }
-    bool getVisibleExtentsForUnit(QString unit, double &min, double &max,
-                                  bool &log) const override {
-        return m_view->getVisibleExtentsForUnit(unit, min, max, log);
+    CoordinateScale getEffectiveVerticalExtentsForLayer(const Layer *layer) const override {
+        return m_view->getEffectiveVerticalExtentsForLayer(layer);
     }
-    ZoomLevel getZoomLevel() const override {
-        ZoomLevel z = m_view->getZoomLevel();
+    CoordinateScale getEffectiveVerticalExtents(QString unit = {}) const override {
+        return m_view->getEffectiveVerticalExtents(unit);
+    }
+    ZoomLevel getRoundedZoomLevel() const override {
+        ZoomLevel z = m_view->getRawZoomLevel();
         if (z.zone == ZoomLevel::FramesPerPixel) {
             z.level /= m_scaleFactor;
             if (z.level < 1) {
@@ -117,6 +121,9 @@ public:
         }
         return z;
     }
+    ZoomLevel getRawZoomLevel() const override {
+        return m_view->getRawZoomLevel();
+    }
     QRect getPaintRect() const override {
         QRect r = m_view->getPaintRect();
         return QRect(r.x() * m_scaleFactor,
@@ -125,13 +132,15 @@ public:
                      r.height() * m_scaleFactor);
     }
     QSize getPaintSize() const override {
-        return getPaintRect().size();
+        QSize s = m_view->getPaintSize();
+        return QSize(s.width() * m_scaleFactor,
+                     s.height() * m_scaleFactor);
     }
-    int getPaintWidth() const override { 
-        return getPaintRect().width();
+    int getPaintWidth() const override {
+        return m_view->getPaintWidth() * m_scaleFactor;
     }
     int getPaintHeight() const override { 
-        return getPaintRect().height();
+        return m_view->getPaintHeight() * m_scaleFactor;
     }
     bool hasLightBackground() const override {
         return m_view->hasLightBackground();
@@ -233,4 +242,6 @@ private:
     }
 };
 
+} // end namespace sv
+
 #endif
