@@ -28,6 +28,8 @@
 
 using namespace std;
 
+namespace sv {
+
 ColourComboBox::ColourComboBox(bool withAddNewColourEntry, QWidget *parent) :
     NotifyingComboBox(parent),
     m_withAddNewColourEntry(withAddNewColourEntry)
@@ -123,3 +125,5 @@ ColourComboBox::rebuild()
     blockSignals(false);
 }
 
+} // end namespace sv
+
