@@ -18,6 +18,8 @@
 
 #include <QToolButton>
 
+namespace sv {
+
 /**
  * Very trivial enhancement to QToolButton to make it emit signals
  * when the mouse enters and leaves (for context help). See also
@@ -39,9 +41,11 @@ signals:
     void mouseLeft();
 
 protected:
-    void enterEvent(QEvent *) override;
+    void enterEvent(QEnterEvent *) override;
     void leaveEvent(QEvent *) override;
 };
 
+} // end namespace sv
+
 #endif
 
