@@ -22,12 +22,15 @@
 #include <map>
 #include <memory>
 
-class AudioDial;
 class QDoubleSpinBox;
 class QCheckBox;
 class QGridLayout;
 class QComboBox;
 
+namespace sv {
+
+class AudioDial;
+
 class PluginParameterBox : public QFrame
 {
     Q_OBJECT
@@ -70,5 +73,7 @@ protected:
     Vamp::PluginBase::ProgramList m_programs;
 };
 
+} // end namespace sv
+
 #endif
 
