@@ -20,6 +20,8 @@
 class QProgressDialog;
 class QTimer;
 
+namespace sv {
+
 class ProgressDialog : public ProgressReporter
 {
     Q_OBJECT
@@ -56,4 +58,6 @@ protected:
     bool m_cancelled;
 };
 
+} // end namespace sv
+
 #endif
