@@ -20,9 +20,12 @@
 
 #include "base/Window.h"
 
-class WindowShapePreview;
 class QComboBox;
 
+namespace sv {
+
+class WindowShapePreview;
+
 class WindowTypeSelector : public QFrame
 {
     Q_OBJECT
@@ -52,4 +55,6 @@ protected:
     void init(WindowType type);
 };
 
+} // end namespace sv
+
 #endif
